@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/const.h,v 3.59 1998/11/05 22:57:10 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/const.h,v 4.38 2000/01/15 22:25:18 william Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -39,30 +39,55 @@
 #ifdef _NO_PROTO
    /* just in case ANSI or C++ doesn't handle function prototypes well */
 #define ARGS_DECL(args) ()
-#else
+#undef ARGS_DECL_USED
+#else /* ~_NO_PROTO */
 #define ARGS_DECL(args) args
-#endif
-#else
+#undef ARGS_DECL_USED
+#define ARGS_DECL_USED 1
+#endif /* _NO_PROTO */
+#else /* ~(__STDC__ || defined(__cplusplus) || defined(c_plusplus)) */
 #define ARGS_DECL(args) ()
-#endif
+#undef ARGS_DECL_USED
+#endif /* __STDC__ || defined(__cplusplus) || defined(c_plusplus) */
 
 #include "tgif_dbg.h"
 
+#ifndef _NO_LOCALE_SUPPORT
+#ifdef ENABLE_NLS
+#define _(text)  gettext(text)
+#define N_(text) text
+#else /* ~ENABLE_NLS */
+#define _(text)  text
+#define N_(text) text
+#endif /* ENABLE_NLS */
+#else /* _NO_LOCALE_SUPPORT */
+#define _(text)  text
+#define N_(text) text
+#endif /* ~_NO_LOCALE_SUPPORT */
+
 #define TOOL_NAME "Tgif"
 
 #ifndef NULL
 #define NULL 0
-#endif
+#endif /* ~NULL */
 
 #ifndef TRUE
 #define FALSE 0
 #define TRUE 1
-#endif
+#endif /* ~TRUE */
 
 #define INVALID (-1)
 #undef BAD
 #define BAD (-2)
 
+#ifdef WIN32
+#define DIR_SEP '\\'
+#define DIR_SEP_STR "\\"
+#else /* ~WIN32 */
+#define DIR_SEP '/'
+#define DIR_SEP_STR "/"
+#endif /* WIN32 */
+
 #define SINGLECOLOR (FALSE)
 #define MULTICOLOR (TRUE)
 
@@ -71,15 +96,50 @@
 #ifndef max
 #define max(A,B) (((A)>(B)) ? (A) : (B))
 #define min(A,B) (((A)>(B)) ? (B) : (A))
-#endif
+#endif /* ~max */
 
 #ifndef round
 #define round(X) (((X) >= 0) ? (int)((X)+0.5) : (int)((X)-0.5))
-#endif
+#endif /* ~round */
 
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
-#endif
+#endif /* ~M_PI */
+
+#define INT_TOL (1.0e-5)
+#define EQ_TOL (1.0e-8)
+
+#ifndef XK_KP_Left
+#define XK_KP_Left      0xFF96
+#define XK_KP_Up        0xFF97
+#define XK_KP_Right     0xFF98
+#define XK_KP_Down      0xFF99
+#endif /* ~XK_KP_LEFT */
+
+#ifndef XK_KP_Delete
+#define XK_KP_Insert    0xFF9E
+#define XK_KP_Delete    0xFF9F
+#endif /* ~XK_KP_Delete */
+
+#ifndef XK_Page_Up
+#define XK_Page_Up      0xFF55
+#define XK_Page_Down    0xFF56
+#endif /* ~XK_Page_Up */
+
+#ifndef XK_KP_Page_Up
+#define XK_KP_Page_Up   0xFF9A
+#define XK_KP_Page_Down 0xFF9B
+#endif /* ~XK_KP_Page_Up */
+
+#ifndef XK_KP_Home
+#define XK_KP_Home      0xFF95
+#define XK_KP_End       0xFF9C
+#endif /* ~XK_KP_Home */
+
+#ifndef XK_Home
+#define XK_Home         0xFF50
+#define XK_End          0xFF57
+#endif /* ~XK_Home */
 
 #define MAX_ZOOMED_IN 3
 #define HALF_W(w) (((w)&0x1) ? ((w)>>1)+1 : ((w)>>1))
@@ -136,9 +196,34 @@
       (PtY) <= (MarkY)+handleSize)
 #define MyDashedLine(W,GC,V,N) XDrawLines(mainDisplay,W,GC,V,N,CoordModeOrigin)
 
+#define GETINT(category,val,name) ScanValue("%d", &(val), name, category)
+#define GETFLT(category,val,name) ScanValue("%f", &(val), name, category)
+#define GETDBL(category,val,name) ScanValue("%lg", &(val), name, category)
+#define GETSTR(category,val,name) ScanValue("%s", (val), name, category)
+
+#define GetPinObj(ObjPtr) ((ObjPtr)->detail.r->pin_connected ? \
+      (ObjPtr)->detail.r->last : (ObjPtr)->detail.r->first)
+#define GetTextObjFirstStrSeg(ObjPtr) \
+      ((ObjPtr)->detail.t->minilines.first->first_block->seg)
+#define GetTextPtrFirstStrSeg(TextPtr) \
+      ((TextPtr)->minilines.first->first_block->seg)
+
 #define MAXSTRING 256
 #define MAXPATHLENGTH 256
 
+#define COLORSTRLEN 40
+
+/* geometry stuff */
+
+#define MENU_IMAGE_W 64
+#define MENU_IMAGE_H 20
+#define CHOICE_IMAGE_W 32
+#define CHOICE_IMAGE_H 20
+#define SCROLLBAR_W 16
+#define RULER_W 20
+#define BRDR_W 1
+#define WINDOW_PADDING 2
+
 /* object types */
 
 #define OBJ_POLY 0
@@ -153,22 +238,27 @@
 #define OBJ_RCBOX 9
 #define OBJ_XBM 10
 #define OBJ_XPM 11
+#define OBJ_PIN 12 /* Please note that pins are not implemented yet! */
+#define OBJ_SS 13 /* This is reserved */
 
 /* drawing modes */
 
 #define NOTHING 0
 #define DRAWTEXT 1
 #define DRAWBOX 2
-#define DRAWCIRCLE 3
-#define DRAWPOLY 4
-#define DRAWPOLYGON 5
-#define DRAWARC 6
-#define DRAWRCBOX 7
-#define FREEHAND 8
-#define VERTEXMODE 9
-#define ROTATEMODE 10
+#define DRAWCORNEROVAL 3
+#define DRAWCENTEROVAL 4
+#define DRAWEDGECIRCLE 5
+#define DRAWPOLY 6
+#define DRAWPOLYGON 7
+#define DRAWARC 8
+#define DRAWEDGEARC 9
+#define DRAWRCBOX 10
+#define FREEHAND 11
+#define VERTEXMODE 12
+#define ROTATEMODE 13
 
-#define MAXCHOICES 11
+#define MAXCHOICES 14
 
 /* stipple patterns */
 
@@ -178,6 +268,11 @@
 #define SCROLLPAT 7
 #define MAXPATTERNS 32
 
+#define NO_TRANSPAT_MODE 0
+#define TRANSPAT_MODE 1
+
+#define MAXTRANSPATMODES 2
+
 /* line stuff */
 
 #define LINE_THIN 0
@@ -208,6 +303,35 @@
 #define NORB (FALSE)
 #define RB (TRUE)
 
+#define CHANGE_WIDTH 0x1
+#define CHANGE_AW    0x2
+#define CHANGE_AH    0x4
+
+#define CHANGE_LINE_ALL (CHANGE_WIDTH|CHANGE_AW|CHANGE_AH)
+
+#define LCAP_BUTT 0
+#define LCAP_ROUND 1
+#define LCAP_SQUARE 2
+
+#define MAXLINECAPS 3
+
+#define ASTY_MASK_COMPAT  0x0001
+#define ASTY_MASK_FILL    0x0002
+#define ASTY_MASK_HALVES  0x000c
+#define ASTY_MASK_SHAPES  0x0010
+
+#define ASTY_COMPAT       0x0 /* ASTY_MASK_COMPAT */
+#define ASTY_FLEX         0x1 /* ASTY_MASK_COMPAT */
+
+#define ASTY_SOLIDPAT     0x0 /* ASTY_MASK_FILL */
+#define ASTY_BACKPAT      0x2 /* ASTY_MASK_FILL */
+
+#define ASTY_LF_HALF_ONLY 0x4 /* ASTY_MASK_HALVES */
+#define ASTY_RT_HALF_ONLY 0x8 /* ASTY_MASK_HALVES */
+
+#define ASTY_CIRCLE       1 /* not used */
+#define ASTY_BOX          2 /* not used */
+
 /* font stuff */
 
 #define FONT_TIM 0
@@ -239,16 +363,24 @@
 #define MAXJUSTS 3
 
 #define PUSH_FONT 0
-#define PUSH_SIZE 1
+#define PUSH_SZ_UNIT 1
 #define PUSH_STYLE 2
 #define PUSH_JUST 3
 #define PUSH_ROTATE 4
 #define PUSH_PEN 5
 #define PUSH_FILL 6
-#define PUSH_VSPACE 7
-#define PUSH_COLORINDEX 8
-#define PUSH_UNDERLINEON 9
-#define PUSH_UNDERLINE 10
+#define PUSH_TRANSPAT 7
+#define PUSH_VSPACE 8
+#define PUSH_COLORINDEX 9
+#define PUSH_UNDERLINE_ON 10
+#define PUSH_UNDERLINE_Y_OFFSET 11
+#define PUSH_DOUBLEBYTE 12
+
+#define NUM_SZ_UNIT_PER_FONT_SIZE 5760
+#define NUM_SZ_UNIT_PER_POINT_SIZE 10240
+
+#define FontSizeToSzUnit(font_sz) ((font_sz)*NUM_SZ_UNIT_PER_FONT_SIZE)
+#define PointSizeToSzUnit(point_sz) ((point_sz)*NUM_SZ_UNIT_PER_POINT_SIZE)
 
 /* alignment */
 
@@ -294,18 +426,16 @@
 #define ALIGN_SB ((ALIGN_S<<ALIGN_SHIFT)|ALIGN_B)
 #define ALIGN_SS ((ALIGN_S<<ALIGN_SHIFT)|ALIGN_S)
 
+#define ALIGN_OBJS_DIRECT 0
+#define ALIGN_TO_GRID_DIRECT 1
+#define ALIGN_TO_PAGE_DIRECT 2
+
 /* color */
 
 #define MAXCOLORS 11
 
 /* button stuff */
 
-#define CONFIRM_YES 0
-#define CONFIRM_NO 1
-#define CONFIRM_CANCEL 2
-
-#define MAX_CONFIRMS 3
-
 #define BUTTON_INVERT 0
 #define BUTTON_NORMAL 1
 
@@ -330,8 +460,11 @@
 #define EPSI_FILE 5
 #define GIF_FILE 6
 #define HTML_FILE 7
+#define PDF_FILE 8
+#define TIFFEPSI_FILE 9
+#define PNG_FILE 10
 
-#define MAXWHERETOPRINT 8
+#define MAXDEFWHERETOPRINT 11
 
 /* measurement */
 
@@ -383,34 +516,40 @@
 
 /* main menu */
 
-#define MENU_MODE 0
-#define MENU_FILE 1
-#define MENU_EDIT 2
-#define MENU_LAYOUT 3
-#define MENU_MOVEMODE 4
-#define MENU_ARRANGE 5
-#define MENU_PAGE 6
-#define MENU_PAGELAYOUT 7
-#define MENU_HORIALIGN 8
-#define MENU_VERTALIGN 9
-#define MENU_FONT 10
-#define MENU_STYLE 11
-#define MENU_SIZE 12
-#define MENU_SHAPE 13
-#define MENU_STRETCHTEXT 14
-#define MENU_LINEDASH 15
-#define MENU_LINESTYLE 16
-#define MENU_LINETYPE 17
-#define MENU_LINEWIDTH 18
-#define MENU_FILL 19
-#define MENU_PEN 20
-#define MENU_COLOR 21
-#define MENU_IMAGEPROC 22
-#define MENU_NAVIGATE 23
-#define MENU_SPECIAL 24
-#define MENU_HELP 25
+#define MENU_FILE 0
+#define MENU_EDIT 1
+#define MENU_LAYOUT 2
+#define MENU_ARRANGE 3
+#define MENU_PROPERTIES 4
+#define MENU_MOVEMODE 5
+#define MENU_STACKEDPAGE 6
+#define MENU_TILEDPAGE 7
+#define MENU_PAGE 8
+#define MENU_PAGELAYOUT 9
+#define MENU_HORIALIGN 10
+#define MENU_VERTALIGN 11
+#define MENU_FONT 12
+#define MENU_STYLE 13
+#define MENU_SIZE 14
+#define MENU_SHAPE 15
+#define MENU_STRETCHTEXT 16
+#define MENU_LINEDASH 17
+#define MENU_LINESTYLE 18
+#define MENU_LINETYPE 19
+#define MENU_LINEWIDTH 20
+#define MENU_FILL 21
+#define MENU_PEN 22
+#define MENU_TRANSPAT 23
+#define MENU_COLOR 24
+#define MENU_IMAGEPROC 25
+#define MENU_NAVIGATE 26
+#define MENU_SPECIAL 27
+#define MENU_HELP 28
+#define MENU_MODE 29
+#define MENU_TANGRAM2 30
 
-#define MAXMENUS 26
+#define MAXMENUS 31
+#define MENU_MAIN 31 /* mainMenu is treated differently */
 
 /* file menu */
 
@@ -448,83 +587,6 @@
 
 #define FILEMENUENTRIES 31
 
-/* edit menu */
-
-#define EDIT_REDRAW 0
-#define EDIT_DUP 1
-#define EDIT_DELETE 2
-#define EDIT_SELALL 3
-#define EDIT_UNDO 4
-#define EDIT_REDO 5
-#define EDIT_DEL_POINT 6
-#define EDIT_ADD_POINT 7
-#define EDIT_COPY 8
-#define EDIT_CUT 9
-#define EDIT_PASTE 10
-#define EDIT_PASTE_FROM_FILE 11
-#define EDIT_UPDATE 12
-#define EDIT_SCALE 13
-#define EDIT_FLUSH_UNDO 14
-#define EDIT_PRINT_MSG_BUF 15
-#define EDIT_INV_XBM 16
-#define EDIT_PUSH_CUR_CHOICE 17
-#define EDIT_MAKE_PREC_ARC 18
-#define EDIT_CUT_MAPS 19
-#define EDIT_RESTORE_MAPS 20
-#define EDIT_UPDATE_EPS 21
-#define EDIT_CV_INTSPLINE 22
-#define EDIT_SMOOTH_HINGE 23
-#define EDIT_MAKE_REGULAR 24
-#define EDIT_BREAK_TEXT 25
-#define EDIT_SET_SEL_LINEWIDTH 26
-#define EDIT_SET_SEL_FONT_SIZE 27
-#define EDIT_ADD_COLOR 28
-#define EDIT_BREAK_MAPS 29
-#define EDIT_LAYOUT_ON_ARC 30
-#define EDIT_PRECISE_ROTATE 31
-#define EDIT_JOIN_POLY 32
-#define EDIT_CUT_POLY 33
-#define EDIT_GET_BBOX 34
-#define EDIT_SET_SHAPE_SHADOW 35
-#define EDIT_NO_TRANSFORM 36
-#define EDIT_SET_EDIT_TEXT_SIZE 37
-#define EDIT_FIND_CASE 38
-#define EDIT_FIND_NOCASE 39
-#define EDIT_FIND_AGAIN 40
-
-#define MAXEDITMENUS 41
-
-/* layout menu */
-
-#define LAYOUT_INCGRID 0
-#define LAYOUT_DECGRID 1
-#define LAYOUT_GRID 2
-#define LAYOUT_SNAP 3
-#define LAYOUT_GOHOME 4
-#define LAYOUT_SETHOME 5
-#define LAYOUT_ZOOMIN 6
-#define LAYOUT_ZOOMOUT 7
-#define LAYOUT_DEFZOOM 8
-#define LAYOUT_ZOOMWAYOUT 9
-#define LAYOUT_LAND 10
-#define LAYOUT_PORT 11
-#define LAYOUT_PRINTMAG 12
-#define LAYOUT_TOGGLE_WHERE_TO_PRINT 13
-#define LAYOUT_TOGGLE_COLOR_PS 14
-#define LAYOUT_TOGGLE_MOVE_MODE 15
-#define LAYOUT_TOGGLE_GRID_SYSTEM 16
-#define LAYOUT_TOGGLE_MAP_SHOWN 17
-#define LAYOUT_TOGGLE_USE_GRAY 18
-#define LAYOUT_TOGGLE_SHOW_MEASURE 19
-#define LAYOUT_TOGGLE_SET_MEASURE_UNIT 20
-#define LAYOUT_TOGGLE_SHOW_MENUBAR 21
-#define LAYOUT_TOGGLE_SHOW_STATUS 22
-#define LAYOUT_TOGGLE_ONEMOTION_SELMOVE 23
-/* make sure the LAYOUT_TOGGLE_COLOR_LAYERS is the last one */
-#define LAYOUT_TOGGLE_COLOR_LAYERS 24
-
-#define MAXLAYOUTMENUS 25
-
 /* move mode stuff */
 
 #define CONST_MOVE 0
@@ -541,33 +603,8 @@
 
 /* arrange menu */
 
-#define ARRANGE_FRONT 0
-#define ARRANGE_BACK 1
-#define ARRANGE_GROUP 2
-#define ARRANGE_UNGROUP 3
-#define ARRANGE_LOCK 4
-#define ARRANGE_UNLOCK 5
-#define ARRANGE_ALIGNOBJ 6
-#define ARRANGE_ALIGNGRID 7
-#define ARRANGE_ALIGNPAGE 8
-#define ARRANGE_DISTROBJ 9
-#define FLIP_HORIZONTAL 10
-#define FLIP_VERTICAL 11
-#define ROTATE_CLOCKWISE 12
-#define ROTATE_COUNTER 13
-#define SET_TEXT_ROTATE 14
-#define SET_ROTATE_INC 15
-#define ALIGN_OBJ_TOP 16
-#define ALIGN_OBJ_MIDDLE 17
-#define ALIGN_OBJ_BOTTOM 18
-#define ALIGN_OBJ_LEFT 19
-#define ALIGN_OBJ_CENTER 20
-#define ALIGN_OBJ_RIGHT 21
-#define ABUT_HORIZONTAL 22
-#define ABUT_VERTICAL 23
-#define ALIGN_CENTERANENDPOINT 24
-
-#define MAXARRANGEMENUS 25
+#define ABUT_HORIZONTAL 0
+#define ABUT_VERTICAL 1
 
 /* nagivate menu */
 
@@ -581,43 +618,6 @@
 
 #define MAXNAVIGATEMENUS 7
 
-/* special menu */
-
-#define SPECIAL_SYM 0
-#define SPECIAL_UNSYM 1
-#define SPECIAL_INST 2
-#define SPECIAL_ICON 3
-#define SPECIAL_UNICON 4
-#define SPECIAL_PUSH 5
-#define SPECIAL_POP 6
-#define SPECIAL_ADDATTR 7
-#define SPECIAL_DETACHATTR 8
-#define SPECIAL_SHOWATTR 9
-#define SPECIAL_SHOWATTRNAME 10
-#define SPECIAL_HIDEATTR 11
-#define SPECIAL_HIDEATTRNAME 12
-#define SPECIAL_MOVEATTR 13
-#define SPECIAL_EDITATTR 14
-#define SPECIAL_ANIMATESEND 15
-#define SPECIAL_ANIMATEFLASH 16
-#define SPECIAL_ADDFILEATTR 17
-#define SPECIAL_DETACHFILEATTR 18
-#define SPECIAL_EDITFILEATTR 19
-#define SPECIAL_UPDATESYMS 20
-#define SPECIAL_IMPORT_ATTRS 21
-#define SPECIAL_EXPORT_ATTRS 22
-#define SPECIAL_MERGE_WITH_TABLE 23
-#define SPECIAL_EXPORT_TO_TABLE 24
-#ifdef _TGIF_WB
-#define SPECIAL_WHITE_BOARD 25
-#endif /* _TGIF_WB */
-
-#ifdef _TGIF_WB
-#define MAXSPECIALMENUS 26
-#else /* ~_TGIF_WB */
-#define MAXSPECIALMENUS 25
-#endif /* _TGIF_WB */
-
 /* page menu */
 
 #define PAGE_NEXT 0
@@ -648,6 +648,9 @@
 
 /* scroll stuff */
 
+#define VERT_SCROLLBAR 0
+#define HORI_SCROLLBAR 1
+
 #define SCROLL_LEFTEND 0
 #define SCROLL_LEFT 1
 #define SCROLL_RIGHT 2
@@ -661,26 +664,36 @@
 
 #define MAXSCROLLBUTTONS 10
 
+#define SCRL_UP 0
+#define SCRL_DN 1
+#define SCRL_LF 2
+#define SCRL_RT 3
+
+#define NO_UPDATE_SCROLLING 0
+#define JUMP_SCROLLING 1
+#define SMOOTH_SCROLLING 2
+
 /* pixel stuff */
 
 #ifdef sun
 #define PUT_A_POINT(dpy,win,gc,x,y) XDrawPoint(dpy,win,gc,x,y)
-#else
+#else /* ~sun */
 #ifdef ultrix
 #define PUT_A_POINT(dpy,win,gc,x,y) XDrawPoint(dpy,win,gc,x,y)
-#else
+#else /* ~ultrix */
 #ifdef _USE_XDRAWPOINT_TO_PUT_A_POINT
 #define PUT_A_POINT(dpy,win,gc,x,y) XDrawPoint(dpy,win,gc,x,y)
-#else
+#else /* ~_USE_XDRAWPOINT_TO_PUT_A_POINT */
 #define PUT_A_POINT(dpy,win,gc,x,y) XDrawLine(dpy,win,gc,x,y,x,y)
-#endif
-#endif
-#endif
+#endif /* _USE_XDRAWPOINT_TO_PUT_A_POINT */
+#endif /* ultrix */
+#endif /* sun */
 
 /* file stuff */
 
 #define OBJ_FILE_TYPE 0
 #define SYM_FILE_TYPE 1
+#define PIN_FILE_TYPE 2
 
 /* xbm real_type */
 
@@ -690,29 +703,23 @@
 
 /* cmd stuff */
 
-#define CMD_COMPOSITE	0
-#define CMD_NEW		1
-#define CMD_DELETE	2
-#define CMD_MOVE	3
-#define CMD_STRETCH	4
-#define CMD_ONE_TO_MANY	5
-#define CMD_MANY_TO_ONE	6
-#define CMD_REPLACE	7
-#define CMD_GOTO_PAGE	8
+#define CMD_COMPOSITE	 0
+#define CMD_NEW		 1
+#define CMD_DELETE	 2
+#define CMD_MOVE	 3
+#define CMD_STRETCH	 4
+#define CMD_ONE_TO_MANY	 5
+#define CMD_MANY_TO_ONE	 6
+#define CMD_REPLACE	 7
+#define CMD_GOTO_PAGE	 8
+#define CMD_WB_CLEARALL	 9
+#define CMD_CHAT_A_LINE	10
 
 /* rcb radius stuff */
 
 #define DEF_RCB_RADIUS (EIGHTH_INCH)
 #define MIN_RCB_RADIUS 4
 
-/* text editing stuff */
-
-#define KEY_NONE 0
-#define KEY_UP 1
-#define KEY_DOWN 2
-#define KEY_LEFT 3
-#define KEY_RIGHT 4
-
 /* select name dialog box stuff */
 
 #define ITEM_DSPED 10
@@ -723,6 +730,10 @@
 #define BUTTON_CANCEL 2
 #define MAXBUTTONS 3
 
+/* user redraw window stuff */
+
+#define MAX_USER_REDRAWS 2
+
 /* status window stuff */
 
 #define MAX_STATUS_BTNS 3
@@ -768,8 +779,9 @@
 #define CTM_SIN 1
 #define CTM_MSIN 2
 #define CTM_SY 3
-#define CTM_TX 4
-#define CTM_TY 5
+
+#define CTM_TX 0
+#define CTM_TY 1
 
 /* msgbox stuff */
 
@@ -818,9 +830,92 @@
 
 #define START_HAVING_ATTRS 8
 
-/* choice window width */
+/* input method stuff */
 
-#define MAXCHOICEWINDOWCOLS 13
+#define TGIM_NONE       INVALID
+#define TGIM_XCIN       0
+#define TGIM_CHINPUT    1
+#define TGIM_KINPUT     2
+#define TGIM_XIM        3
+
+/* postscript stuff */
+
+#define PS_GSAVE             0
+#define PS_GRESTORE          1
+#define PS_NEWPATH           2
+#define PS_CLOSEPATH         3
+#define PS_CHARPATH          4
+#define PS_CURVETO           5
+#define PS_LINETO            6
+#define PS_RLINETO           7
+#define PS_MOVETO            8
+#define PS_RMOVETO           9
+#define PS_STROKE            10
+#define PS_FILL              11
+#define PS_TRANSLATE         12
+#define PS_ROTATE            13
+#define PS_SCALE             14
+#define PS_MUL               15
+#define PS_DIV               16
+#define PS_DUP               17
+#define PS_NEG               18
+#define PS_ADD               19
+#define PS_SUB               20
+#define PS_POP               21
+#define PS_EXCH              22
+#define PS_CONCAT            23
+#define PS_CLIP              24
+#define PS_EOCLIP            25
+#define PS_EOFILL            26
+#define PS_IMAGE             27
+#define PS_IMAGEMASK         28
+#define PS_ARRAY             29
+#define PS_SETGRAY           30
+#define PS_SETRGBCOLOR       31
+#define PS_SETDASH           32
+#define PS_SETLINEWIDTH      33
+#define PS_SETMITERLIMIT     34
+#define PS_SETLINECAP        35
+#define PS_SETLINEJOIN       36
+#define PS_SHOW              37
+#define PS_FINDFONT          38
+#define PS_MAKESETFONT       39
+#define PS_ARCTO4            40
+#define PS_CURRENTPOINT      41
+#define PS_FLATTENPATH       42
+#define PS_TGIFSETMATRIX     43
+#define PS_TGIFRESTOREMATRIX 44
+
+/* shape stuff */
+
+#define SHAPE_BOX 0
+#define SHAPE_PARALLEL 1
+#define SHAPE_TRAPEZOID 2
+#define SHAPE_RHOMBUS 3
+#define SHAPE_RCBOX 4
+#define SHAPE_OCTAGON 5
+#define SHAPE_CROSS 6
+#define SHAPE_OVAL 7
+#define SHAPE_WORDS 8
+#define SHAPE_HEXAGON 9
+#define SHAPE_TRIANGLE 10
+#define SHAPE_BLAST 11
+#define SHAPE_STAR 12
+#define SHAPE_DISK 13
+#define SHAPE_RIGHTARROW 14
+#define SHAPE_UPARROW 15
+#define SHAPE_FATRIGHTARROW 16
+#define SHAPE_FATUPARROW 17
+#define SHAPE_RIGHTTAB 18
+#define SHAPE_UPTAB 19
+
+#define MAXSHAPES 20
+
+/* export filter stuff */
+
+#define EXPORT_NEXT 1
+#define EXPORT_PREV -1
+#define EXPORT_THIS 0
 
 /* generic object stuff */
 
@@ -830,6 +925,16 @@
 #define TGO_DATE 3
 #define TGO_USER 4
 
+/* break up text stuff */
+
+#define BREAK_CHAR 0
+#define BREAK_WORD 1
+#define BREAK_LINE 2
+
+/* xpm stuff */
+
+#define MAXXPMEXPORTCOLORS 6400
+
 /* choose a file stuff */
 
 #define NAMES_SIMPLE_SELECT_NAME 0
@@ -841,4 +946,25 @@
 #define NAMES_LOOP_MANY 0
 #define NAMES_LOOP_ONCE 1
 
+/* properties stuff */
+
+#define PROP_MASK_CTM		0x00000001
+#define PROP_MASK_COLOR		0x00000002
+#define PROP_MASK_WIDTH		0x00000004
+#define PROP_MASK_AW		0x00000008
+#define PROP_MASK_AH		0x00000010
+#define PROP_MASK_TRANSPAT	0x00000020
+#define PROP_MASK_FILL		0x00000040
+#define PROP_MASK_PEN		0x00000080
+#define PROP_MASK_DASH		0x00000100
+#define PROP_MASK_ARROW_STYLE	0x00000200
+#define PROP_MASK_CURVED	0x00000400
+#define PROP_MASK_RCB_RADIUS	0x00000800
+#define PROP_MASK_TEXT_JUST	0x00001000
+#define PROP_MASK_TEXT_SZ_UNIT	0x00002000
+#define PROP_MASK_TEXT_FONT	0x00004000
+#define PROP_MASK_TEXT_STYLE	0x00008000 /* ChangeEditTextProperty() only */
+#define PROP_MASK_VSPACE	0x00010000
+#define PROP_MASK_UNDERLINE_ON	0x00020000
+
 #endif /*_TGIF_CONST_H_*/
