@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,43 +28,40 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/dialog.c,v 3.23 1998/11/05 22:56:09 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/dialog.c,v 4.17 2000/01/24 20:13:12 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/keysym.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_DIALOG_C_
+
+#include "tgifdefs.h"
 
 #include "auxtext.e"
 #include "box.e"
 #include "button.e"
 #include "cutpaste.e"
 #include "cursor.e"
-#ifndef _NO_EXTERN
 #include "dialog.e"
-#endif
 #include "drawing.e"
 #include "file.e"
 #include "font.e"
 #include "grid.e"
 #include "mainloop.e"
 #include "mainmenu.e"
+#include "menu.e"
 #include "msg.e"
 #include "raster.e"
+#include "rect.e"
 #include "ruler.e"
 #include "setup.e"
+#include "strtbl.e"
 #include "text.e"
 #include "util.e"
 
 int	doPassword=FALSE;
 char	gszMsgBox[2048];
 
+int	dialogboxUse3DBorder=TRUE;
+
 unsigned int CornerLoop(OrigX, OrigY)
    int *OrigX, *OrigY;
 {
@@ -178,7 +175,7 @@ unsigned int PickAPoint(OrigX, OrigY, cursor)
 #define ICON_H (info_height)
 
 #define X_MARGIN 20
-#define Y_MARGIN 8
+#define Y_MARGIN 16
 #define X_GAP 20
 #define Y_GAP 20
 
@@ -233,9 +230,6 @@ static struct MBRec	mbInfo;
 
 static int	numButtons=MAX_BUTTONS;
 static char	extraBtnChar='q';
-static char	*btnStr[] = {
-      "INVALID", "OK", "CANCEL", "YES", "NO", "Extra", NULL
-};
 
 static
 void SetupMBButton(MBInfoPtr, BtnDesc, BtnCh, BtnID)
@@ -243,14 +237,18 @@ void SetupMBButton(MBInfoPtr, BtnDesc, BtnCh, BtnID)
    int BtnDesc, BtnID;
    char BtnCh;
 {
-   int btn_index=(BtnDesc & 0x0f)-1;
-   int i=0;
+   char *psz=NULL;
+   int btn_index=(BtnDesc & 0x0f)-1, i=0;
 
-   if (BtnID == MB_ID_FAILED) {
-      MBInfoPtr->btn_info[btn_index].str = NULL;
-   } else {
-      MBInfoPtr->btn_info[btn_index].str = btnStr[BtnID];
+   switch (BtnID) {
+   case MB_ID_FAILED: psz = NULL; break;
+   case MB_ID_OK: psz = TgLoadCachedString(CSTID_OK); break;
+   case MB_ID_CANCEL: psz = TgLoadCachedString(CSTID_CANCEL); break;
+   case MB_ID_YES: psz = TgLoadCachedString(CSTID_YES); break;
+   case MB_ID_NO: psz = TgLoadCachedString(CSTID_NO); break;
+   case MB_ID_EXTRA: psz = TgLoadCachedString(CSTID_EXTRA); break;
    }
+   MBInfoPtr->btn_info[btn_index].str = psz;
 
    MBInfoPtr->btn_info[btn_index].id = BtnID;
    switch (BtnCh) {
@@ -281,99 +279,133 @@ void SetupMBButton(MBInfoPtr, BtnDesc, BtnCh, BtnID)
    MBInfoPtr->btn_info[btn_index].key_sym[i] = (KeySym)0;
 }
 
-static
-void CalcSimpleGeometry(MBInfoPtr, Message)
-   struct MBRec *MBInfoPtr;
-   char *Message;
+void CalcFormatStringInBox(buf, xfs, font_height, format_box_w, pn_num_lines,
+      pn_max_w, pn_max_h, ppsz_formated_buf)
+   char *buf, **ppsz_formated_buf;
+   XFontStruct *xfs;
+   int font_height, format_box_w, *pn_num_lines, *pn_max_w, *pn_max_h;
 {
-   int len=strlen(Message), max_len=0, max_h=0, left, inc, i, a_btn_w, a_btn_h;
-   char *dest_ptr, *c_ptr;
-   int font_height=0, font_asc=0, font_des=0;
+   char *dest_ptr=NULL, *c_ptr=NULL, *msg_copy=NULL;
+   int len=strlen(buf), max_len=0, max_w=0, max_h=0, total=0;
+   int num_lines=1, spacing=SPACING, sz=((((len+10)<<1)+1)*sizeof(char));
 
-   font_height = defaultFontHeight;
-   font_asc = defaultFontAsc;
-   font_des = defaultFontDes;
+   total = BoldMsgTextWidth(xfs, buf, len);
 
-   MBInfoPtr->msg_copy = (char*)malloc(((len<<1)+1)*sizeof(char));
-   if (MBInfoPtr->msg_copy == NULL) FailAllocMessage();
+   msg_copy = (char*)malloc(sz);
+   if (msg_copy == NULL) FailAllocMessage();
+   memset(msg_copy, 0, sz);
 
-   dest_ptr = MBInfoPtr->msg_copy;
+   dest_ptr = msg_copy;
    *dest_ptr = '\0';
-   c_ptr = Message;
+   c_ptr = buf;
    while (c_ptr != NULL) {
       /* a line at a time */
-      char *lf_ptr=strchr(c_ptr, '\n');
-      int full_str_len;
+      char *lf_ptr=BoldMsgStrChr(c_ptr, '\n');
+      int full_str_len, full_str_total;
 
       if (lf_ptr != NULL) *lf_ptr = '\0';
       full_str_len = strlen(c_ptr);
-      if (full_str_len > max_len) {
-         if (full_str_len > MBInfoPtr->max_msg_str_len) {
+      full_str_total = BoldMsgTextWidth(xfs, c_ptr, full_str_len);
+      if (full_str_total > max_w) {
+         if (full_str_total > format_box_w) {
             /* line too long for the message box */
             char *line=c_ptr;
 
-            max_len = MBInfoPtr->max_msg_str_len;
+            max_w = format_box_w;
             while (line != NULL && *line != '\0') {
-               int line_len;
+               int line_len, line_total;
 
                while (*line == ' ') line++;
                line_len = strlen(line);
-               if (line_len > MBInfoPtr->max_msg_str_len) {
-                  int index=MBInfoPtr->max_msg_str_len;
+               line_total = BoldMsgTextWidth(xfs, line, line_len);
+               if (line_total > format_box_w) {
+                  char *lead_ptr=line, *last_ptr=NULL, saved_last_ch='\0';
+                  int lead_index=0, last_index=0, still_going=TRUE;
 
-                  if (line[index] == ' ') {
-                     int saved_index=index--;
+                  while (still_going && *lead_ptr != '\0') {
+                     char saved_ch='\0';
+                     int w=0;
 
-                     while (index >=0 && line[index] == ' ') index--;
-                     line[++index] = '\0';
-                     sprintf(dest_ptr, "%s\n", line);
-                     dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                     max_h += font_height+SPACING;
-                     line = &line[saved_index+1];
-                  } else {
-                     int saved_index=index;
-
-                     while (index >=0 && line[index] != ' ') index--;
-                     if (index < 0) {
+                     while (*lead_ptr != ' ' && *lead_ptr != '\0') {
+                        if (BoldMsgCharBytes(lead_ptr) == 2) {
+                           /* for double-byte chars, one can break anywhere */
+                           break;
+                        }
+                        lead_ptr++;
+                        lead_index++;
+                     }
+                     saved_ch = (*lead_ptr);
+                     *lead_ptr = '\0';
+                     w = BoldMsgTextWidth(xfs, line, lead_index);
+                     if (w > format_box_w) {
+                        if (last_ptr == NULL) {
+                           /* very long word */
+                           sprintf(dest_ptr, "%s\n", line);
+                           dest_ptr = &dest_ptr[strlen(dest_ptr)];
+                           max_h += font_height+spacing;
+                           num_lines++;
+                           line = &line[lead_index];
+                        } else {
+                           /* find a good space */
+                           saved_last_ch = (*last_ptr);
+                           *last_ptr = '\0';
+                           sprintf(dest_ptr, "%s\n", line);
+                           *last_ptr = saved_last_ch;
+                           dest_ptr = &dest_ptr[strlen(dest_ptr)];
+                           max_h += font_height+spacing;
+                           num_lines++;
+                           line = BoldMsgNextChar(&line[last_index]);
+                        }
+                        still_going = FALSE;
+                     } else {
+                        last_ptr = lead_ptr;
+                        last_index = lead_index;
+                     }
+                     *lead_ptr++ = saved_ch;
+                     lead_index++;
+                  }
+                  if (still_going && *lead_ptr == '\0') {
+                     if (last_ptr == NULL) {
                         /* very long word */
-                        char saved_ch=line[saved_index];
-
-                        line[saved_index] = '\0';
                         sprintf(dest_ptr, "%s\n", line);
                         dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                        max_h += font_height+SPACING;
-                        line[saved_index] = saved_ch;
-                        line = &line[saved_index];
+                        max_h += font_height+spacing;
+                        num_lines++;
+                        line = &line[lead_index];
                      } else {
                         /* find a good space */
-                        saved_index = index--;
-                        while (index >=0 && line[index] == ' ') index--;
-                        line[++index] = '\0';
+                        saved_last_ch = (*last_ptr);
+                        *last_ptr = '\0';
                         sprintf(dest_ptr, "%s\n", line);
+                        *last_ptr = saved_last_ch;
                         dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                        max_h += font_height+SPACING;
-                        line = &line[saved_index+1];
+                        max_h += font_height+spacing;
+                        num_lines++;
+                        line = BoldMsgNextChar(&line[last_index]);
                      }
                   }
                } else if (line_len > 0) {
                   sprintf(dest_ptr, "%s\n", line);
                   dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                  max_h += font_height+SPACING;
+                  max_h += font_height+spacing;
                   break;
                }
             }
          } else {
             max_len = full_str_len;
+            max_w = full_str_total;
             sprintf(dest_ptr, "%s\n", c_ptr);
             dest_ptr = &dest_ptr[strlen(dest_ptr)];
-            max_h += font_height+SPACING;
+            max_h += font_height+spacing;
+            if (lf_ptr != NULL) num_lines++;
          }
       } else {
          sprintf(dest_ptr, "%s\n", c_ptr);
          dest_ptr = &dest_ptr[strlen(dest_ptr)];
-         max_h += font_height+SPACING;
+         max_h += font_height+spacing;
+         if (lf_ptr != NULL) num_lines++;
       }
-      max_h -= SPACING;
+      max_h -= spacing;
       if (lf_ptr != NULL) {
          *lf_ptr = '\n';
          c_ptr = &lf_ptr[1];
@@ -381,14 +413,36 @@ void CalcSimpleGeometry(MBInfoPtr, Message)
          break;
       }
    }
+   if (pn_num_lines != NULL) *pn_num_lines = num_lines;
+   if (pn_max_w != NULL) *pn_max_w = max_w;
+   if (pn_max_h != NULL) *pn_max_h = max_h;
+   if (ppsz_formated_buf == NULL) {
+      free(msg_copy);
+   } else {
+      *ppsz_formated_buf = msg_copy;
+   }
+}
+
+static
+void CalcSimpleGeometry(MBInfoPtr, Message)
+   struct MBRec *MBInfoPtr;
+   char *Message;
+{
+   int i=0, max_h=0, left=0, inc=0, a_btn_w=0, a_btn_h=0, y=0, max_w=0;;
+
+   CalcFormatStringInBox(Message, defaultFontPtr, defaultFontHeight,
+         MBInfoPtr->max_msg_str_total, NULL, &max_w, &max_h,
+         &MBInfoPtr->msg_copy);
+
    if (MBInfoPtr->is_dialog) {
       MBInfoPtr->msg_win_w = MBInfoPtr->max_msg_str_total;
    } else {
-      MBInfoPtr->msg_win_w = max_len*defaultFontWidth;
+      MBInfoPtr->msg_win_w = max_w;
    }
    MBInfoPtr->msg_win_h = max_h;
    MBInfoPtr->icon_win_w = ICON_W;
    MBInfoPtr->icon_win_h = ICON_H;
+   if (info_bits) { }
    if (MBInfoPtr->msg_win_h > MBInfoPtr->icon_win_h) {
       MBInfoPtr->icon_win_h = MBInfoPtr->msg_win_h;
    } else {
@@ -396,13 +450,13 @@ void CalcSimpleGeometry(MBInfoPtr, Message)
    }
    a_btn_w = MAX_BTN_STR_LEN*defaultFontWidth + (BTN_XY_EXTRA<<1);
    if (MBInfoPtr->is_dialog) {
-      if (msgFontPtr == NULL) {
+      if (msgFontSet == NULL && msgFontPtr == NULL) {
          a_btn_h = defaultFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
       } else {
          a_btn_h = msgFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
       }
    } else {
-      if (boldMsgFontPtr == NULL) {
+      if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
          a_btn_h = defaultFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
       } else {
          a_btn_h = boldMsgFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
@@ -410,7 +464,7 @@ void CalcSimpleGeometry(MBInfoPtr, Message)
    }
 
    MBInfoPtr->btn_win_w = numButtons*a_btn_w+BTN_MIN_X_GAP*(numButtons-1)+2;
-   MBInfoPtr->btn_win_h = a_btn_h+2;
+   MBInfoPtr->btn_win_h = a_btn_h+2+(windowPadding<<2);
 
    if (MBInfoPtr->btn_win_w >
          MBInfoPtr->msg_win_w+MBInfoPtr->icon_win_w+X_GAP) {
@@ -428,13 +482,17 @@ void CalcSimpleGeometry(MBInfoPtr, Message)
          BTN_MIN_X_GAP*(numButtons-1))>>1);
    inc = a_btn_w + ((MBInfoPtr->btn_win_w-(left<<1)-numButtons*a_btn_w) /
          (numButtons-1));
+   y = ((MBInfoPtr->btn_win_h - ((BTN_Y_MARGIN<<1) +
+         ((boldMsgFontSet==NULL && boldMsgFontPtr==NULL) ? defaultFontHeight :
+         boldMsgFontHeight))) >> 1);
    for (i=0; i < numButtons; i++) {
       MBInfoPtr->btn_info[i].bbox.ltx = left+BTN_XY_EXTRA;
-      MBInfoPtr->btn_info[i].bbox.lty = BTN_XY_EXTRA;
+      MBInfoPtr->btn_info[i].bbox.lty = y;
       MBInfoPtr->btn_info[i].bbox.rbx = left +
             MAX_BTN_STR_LEN*defaultFontWidth + BTN_XY_EXTRA;
-      MBInfoPtr->btn_info[i].bbox.rby = BTN_XY_EXTRA + defaultFontHeight +
-            (BTN_Y_MARGIN<<1);
+      MBInfoPtr->btn_info[i].bbox.rby = y + (BTN_Y_MARGIN<<1) +
+            ((boldMsgFontSet==NULL && boldMsgFontPtr==NULL) ?
+            defaultFontHeight : boldMsgFontHeight);
       left += inc;
    }
    MBInfoPtr->main_win_x = ((DisplayWidth(mainDisplay, mainScreen) -
@@ -450,129 +508,17 @@ void CalcGeometry(MBInfoPtr, Message)
    struct MBRec *MBInfoPtr;
    char *Message;
 {
-   int len=strlen(Message), max_h=0, left, inc, i, a_btn_w, a_btn_h;
-   int max_len=0, max_total=0;
-   char *dest_ptr, *c_ptr;
-   int font_height, font_asc, font_des, total;
+   int i=0, max_h=0, left=0, inc=0, a_btn_w=0, a_btn_h=0;
+   int font_height=boldMsgFontHeight, y=0, max_w=0;
 
-   font_height = boldMsgFontHeight;
-   font_asc = boldMsgFontAsc;
-   font_des = boldMsgFontDes;
-   total = XTextWidth(boldMsgFontPtr, Message, len);
+   CalcFormatStringInBox(Message, boldMsgFontPtr, boldMsgFontHeight,
+         MBInfoPtr->max_msg_str_total, NULL, &max_w, &max_h,
+         &MBInfoPtr->msg_copy);
 
-   MBInfoPtr->msg_copy = (char*)malloc(((len<<1)+1)*sizeof(char));
-   if (MBInfoPtr->msg_copy == NULL) FailAllocMessage();
-
-   dest_ptr = MBInfoPtr->msg_copy;
-   *dest_ptr = '\0';
-   c_ptr = Message;
-   while (c_ptr != NULL) {
-      /* a line at a time */
-      char *lf_ptr=strchr(c_ptr, '\n');
-      int full_str_len, full_str_total;
-
-      if (lf_ptr != NULL) *lf_ptr = '\0';
-      full_str_len = strlen(c_ptr);
-      full_str_total = XTextWidth(boldMsgFontPtr, c_ptr, full_str_len);
-      if (full_str_total > max_total) {
-         if (full_str_total > MBInfoPtr->max_msg_str_total) {
-            /* line too long for the message box */
-            char *line=c_ptr;
-
-            max_total = MBInfoPtr->max_msg_str_total;
-            while (line != NULL && *line != '\0') {
-               int line_len, line_total;
-
-               while (*line == ' ') line++;
-               line_len = strlen(line);
-               line_total = XTextWidth(boldMsgFontPtr, line, line_len);
-               if (line_total > MBInfoPtr->max_msg_str_total) {
-                  char *lead_ptr=line, *last_ptr=NULL;
-                  int lead_index=0, last_index=0, still_going=TRUE;
-
-                  while (still_going && *lead_ptr != '\0') {
-                     char saved_ch;
-                     int w;
-
-                     while (*lead_ptr != ' ' && *lead_ptr != '\0') {
-                        lead_ptr++;
-                        lead_index++;
-                     }
-                     saved_ch = (*lead_ptr);
-                     *lead_ptr = '\0';
-                     w = XTextWidth(boldMsgFontPtr, line, lead_index);
-                     if (w > MBInfoPtr->max_msg_str_total) {
-                        if (last_ptr == NULL) {
-                           /* very long word */
-                           sprintf(dest_ptr, "%s\n", line);
-                           dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                           max_h += font_height+SPACING;
-                           line = &line[lead_index];
-                        } else {
-                           /* find a good space */
-                           *last_ptr = '\0';
-                           sprintf(dest_ptr, "%s\n", line);
-                           *last_ptr = ' ';
-                           dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                           max_h += font_height+SPACING;
-                           line = &line[last_index+1];
-                        }
-                        still_going = FALSE;
-                     } else {
-                        last_ptr = lead_ptr;
-                        last_index = lead_index;
-                     }
-                     *lead_ptr++ = saved_ch;
-                     lead_index++;
-                  }
-                  if (still_going && *lead_ptr == '\0') {
-                     if (last_ptr == NULL) {
-                        /* very long word */
-                        sprintf(dest_ptr, "%s\n", line);
-                        dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                        max_h += font_height+SPACING;
-                        line = &line[lead_index];
-                     } else {
-                        /* find a good space */
-                        *last_ptr = '\0';
-                        sprintf(dest_ptr, "%s\n", line);
-                        *last_ptr = ' ';
-                        dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                        max_h += font_height+SPACING;
-                        line = &line[last_index+1];
-                     }
-                  }
-               } else if (line_len > 0) {
-                  sprintf(dest_ptr, "%s\n", line);
-                  dest_ptr = &dest_ptr[strlen(dest_ptr)];
-                  max_h += font_height+SPACING;
-                  break;
-               }
-            }
-         } else {
-            max_len = full_str_len;
-            max_total = full_str_total;
-            sprintf(dest_ptr, "%s\n", c_ptr);
-            dest_ptr = &dest_ptr[strlen(dest_ptr)];
-            max_h += font_height+SPACING;
-         }
-      } else {
-         sprintf(dest_ptr, "%s\n", c_ptr);
-         dest_ptr = &dest_ptr[strlen(dest_ptr)];
-         max_h += font_height+SPACING;
-      }
-      max_h -= SPACING;
-      if (lf_ptr != NULL) {
-         *lf_ptr = '\n';
-         c_ptr = &lf_ptr[1];
-      } else {
-         break;
-      }
-   }
    if (MBInfoPtr->is_dialog) {
       MBInfoPtr->msg_win_w = MBInfoPtr->max_msg_str_total;
    } else {
-      MBInfoPtr->msg_win_w = max_total;
+      MBInfoPtr->msg_win_w = max_w;
    }
    MBInfoPtr->msg_win_h = max_h;
    MBInfoPtr->icon_win_w = ICON_W;
@@ -582,23 +528,20 @@ void CalcGeometry(MBInfoPtr, Message)
    } else {
       MBInfoPtr->msg_win_h = MBInfoPtr->icon_win_h;
    }
-   a_btn_w = XTextWidth(boldMsgFontPtr, "  CANCEL  ", 10) + (BTN_XY_EXTRA<<1);
+   /* do not translate -- the string is used to measure things */
+   a_btn_w = BoldMsgTextWidth(boldMsgFontPtr, "  CANCEL  ", 10) +
+         (BTN_XY_EXTRA<<1);
    if (MBInfoPtr->is_dialog) {
-      if (msgFontPtr == NULL) {
-         a_btn_h = defaultFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
-      } else {
-         a_btn_h = msgFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
-      }
+      a_btn_h = (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1) +
+            ((msgFontSet==NULL && msgFontPtr==NULL) ? defaultFontHeight :
+            msgFontHeight);
    } else {
-      if (boldMsgFontPtr == NULL) {
-         a_btn_h = defaultFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
-      } else {
-         a_btn_h = boldMsgFontHeight + (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1);
-      }
+      a_btn_h = (BTN_Y_MARGIN<<1) + (BTN_XY_EXTRA<<1) +
+            ((boldMsgFontSet==NULL && boldMsgFontPtr==NULL) ?
+            defaultFontHeight : boldMsgFontHeight);
    }
-
    MBInfoPtr->btn_win_w = numButtons*a_btn_w+BTN_MIN_X_GAP*(numButtons-1)+2;
-   MBInfoPtr->btn_win_h = a_btn_h+2;
+   MBInfoPtr->btn_win_h = a_btn_h+2+(windowPadding<<1);
 
    if (MBInfoPtr->btn_win_w >
          MBInfoPtr->msg_win_w+MBInfoPtr->icon_win_w+X_GAP) {
@@ -616,13 +559,13 @@ void CalcGeometry(MBInfoPtr, Message)
          BTN_MIN_X_GAP*(numButtons-1))>>1);
    inc = a_btn_w + ((MBInfoPtr->btn_win_w-(left<<1)-numButtons*a_btn_w) /
          (numButtons-1));
+   y = ((MBInfoPtr->btn_win_h - ((BTN_Y_MARGIN<<1) + font_height)) >> 1);
    for (i=0; i < numButtons; i++) {
       MBInfoPtr->btn_info[i].bbox.ltx = left+BTN_XY_EXTRA;
-      MBInfoPtr->btn_info[i].bbox.lty = BTN_XY_EXTRA;
+      MBInfoPtr->btn_info[i].bbox.lty = y;
       MBInfoPtr->btn_info[i].bbox.rbx = left +
             (a_btn_w-(BTN_XY_EXTRA<<1)) + BTN_XY_EXTRA;
-      MBInfoPtr->btn_info[i].bbox.rby = BTN_XY_EXTRA + font_height +
-            (BTN_Y_MARGIN<<1);
+      MBInfoPtr->btn_info[i].bbox.rby = y + font_height + (BTN_Y_MARGIN<<1);
       left += inc;
    }
    MBInfoPtr->main_win_x = ((DisplayWidth(mainDisplay, mainScreen) -
@@ -639,6 +582,7 @@ int SetupMBWindow(MBInfoPtr, Message, Title, IconAndBtns, IsDialog)
    char *Message, *Title;
    int IconAndBtns, IsDialog;
 {
+   int bg_pixel=(threeDLook ? myLtGryPixel : myBgPixel);
    XWMHints wmhints;
    XSizeHints sizehints;
    XSetWindowAttributes win_attrs;
@@ -652,7 +596,7 @@ int SetupMBWindow(MBInfoPtr, Message, Title, IconAndBtns, IsDialog)
 
    numButtons = MAX_BUTTONS;
    if (IconAndBtns & MB_BTN_EXTRA) numButtons++;
-   if (boldMsgFontPtr == NULL) {
+   if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
       CalcSimpleGeometry(MBInfoPtr, Message);
    } else {
       CalcGeometry(MBInfoPtr, Message);
@@ -716,29 +660,29 @@ int SetupMBWindow(MBInfoPtr, Message, Title, IconAndBtns, IsDialog)
    if ((MBInfoPtr->main_win=XCreateSimpleWindow(mainDisplay, rootWindow,
          MBInfoPtr->main_win_x, MBInfoPtr->main_win_y,
          MBInfoPtr->main_win_w, MBInfoPtr->main_win_h, brdrW,
-         myBorderPixel, myBgPixel)) == 0) {
-      fprintf(stderr, "Fatal error!  Can not create MsgBox window!\n");
+         myBorderPixel, bg_pixel)) == 0) {
+      fprintf(stderr, "%s\n", TgLoadString(STID_FAIL_TO_CREATE_WINDOW));
       return FALSE;
    }
    if ((MBInfoPtr->icon_win=XCreateSimpleWindow(mainDisplay,
          MBInfoPtr->main_win, X_MARGIN, Y_MARGIN,
          MBInfoPtr->icon_win_w, MBInfoPtr->icon_win_h, 0,
-         myBorderPixel, myBgPixel)) == 0) {
-      fprintf(stderr, "Fatal error!  Can not create MsgBox window!\n");
+         myBorderPixel, bg_pixel)) == 0) {
+      fprintf(stderr, "%s\n", TgLoadString(STID_FAIL_TO_CREATE_WINDOW));
       return FALSE;
    }
    if ((MBInfoPtr->msg_win=XCreateSimpleWindow(mainDisplay,
          MBInfoPtr->main_win, X_MARGIN+MBInfoPtr->icon_win_w+X_GAP, Y_MARGIN,
          MBInfoPtr->msg_win_w, MBInfoPtr->msg_win_h, 0,
-         myBorderPixel, myBgPixel)) == 0) {
-      fprintf(stderr, "Fatal error!  Can not create MsgBox window!\n");
+         myBorderPixel, bg_pixel)) == 0) {
+      fprintf(stderr, "%s\n", TgLoadString(STID_FAIL_TO_CREATE_WINDOW));
       return FALSE;
    }
    if ((MBInfoPtr->btn_win=XCreateSimpleWindow(mainDisplay,
          MBInfoPtr->main_win, X_MARGIN, Y_MARGIN+Y_GAP+MBInfoPtr->icon_win_h,
          MBInfoPtr->btn_win_w, MBInfoPtr->btn_win_h, 0,
-         myBorderPixel, myBgPixel)) == 0) {
-      fprintf(stderr, "Fatal error!  Can not create MsgBox window!\n");
+         myBorderPixel, bg_pixel)) == 0) {
+      fprintf(stderr, "%s\n", TgLoadString(STID_FAIL_TO_CREATE_WINDOW));
       return FALSE;
    }
    win_attrs.save_under = True;
@@ -767,6 +711,7 @@ int SetupMBWindow(MBInfoPtr, Message, Title, IconAndBtns, IsDialog)
 #else
    XSetWMNormalHints(mainDisplay, MBInfoPtr->main_win, &sizehints);
 #endif /* NOTR4MODE */
+   RegisterWM_DELETE_WINDOW(MBInfoPtr->main_win);
    XStoreName(mainDisplay, MBInfoPtr->main_win, Title);
 
 #ifdef MAPBEFORESELECT
@@ -774,27 +719,23 @@ int SetupMBWindow(MBInfoPtr, Message, Title, IconAndBtns, IsDialog)
    XMapWindow(mainDisplay, MBInfoPtr->icon_win);
    XMapWindow(mainDisplay, MBInfoPtr->msg_win);
    XMapWindow(mainDisplay, MBInfoPtr->btn_win);
-   XSelectInput(mainDisplay, MBInfoPtr->main_win,
-         ButtonPressMask | KeyPressMask |
+   XSelectInput(mainDisplay, MBInfoPtr->main_win, ButtonPressMask |
          KeyPressMask | StructureNotifyMask | VisibilityChangeMask);
    XSelectInput(mainDisplay, MBInfoPtr->icon_win,
          ButtonPressMask | KeyPressMask | ExposureMask);
    XSelectInput(mainDisplay, MBInfoPtr->msg_win,
          ButtonPressMask | KeyPressMask | ExposureMask);
    XSelectInput(mainDisplay, MBInfoPtr->btn_win, ButtonReleaseMask |
-         ButtonPressMask | KeyPressMask |
-         ButtonPressMask | KeyPressMask | ExposureMask);
+         ButtonPressMask | PointerMotionMask | KeyPressMask | ExposureMask);
 #else /* !MAPBEFORESELECT */
-   XSelectInput(mainDisplay, MBInfoPtr->main_win,
-         ButtonPressMask | KeyPressMask |
+   XSelectInput(mainDisplay, MBInfoPtr->main_win, ButtonPressMask |
          KeyPressMask | StructureNotifyMask | VisibilityChangeMask);
    XSelectInput(mainDisplay, MBInfoPtr->icon_win,
          ButtonPressMask | KeyPressMask | ExposureMask);
    XSelectInput(mainDisplay, MBInfoPtr->msg_win,
          ButtonPressMask | KeyPressMask | ExposureMask);
    XSelectInput(mainDisplay, MBInfoPtr->btn_win, ButtonReleaseMask |
-         ButtonPressMask | KeyPressMask |
-         ButtonPressMask | KeyPressMask | ExposureMask);
+         ButtonPressMask | PointerMotionMask | KeyPressMask | ExposureMask);
    XMapWindow(mainDisplay, MBInfoPtr->main_win);
    XMapWindow(mainDisplay, MBInfoPtr->icon_win);
    XMapWindow(mainDisplay, MBInfoPtr->msg_win);
@@ -821,10 +762,10 @@ void DisplayInput(MBInfoPtr)
    if (buf == NULL) return;
 
    buf_len = strlen(buf);
-   if (msgFontPtr == NULL) {
+   if (msgFontSet == NULL && msgFontPtr == NULL) {
       MBInfoPtr->str_w = defaultFontWidth*strlen(buf);
    } else {
-      MBInfoPtr->str_w = XTextWidth(msgFontPtr, buf, buf_len);
+      MBInfoPtr->str_w = MsgTextWidth(msgFontPtr, buf, buf_len);
    }
    MBInfoPtr->cur_x = ((MBInfoPtr->btn_win_w-MBInfoPtr->str_w)>>1);
    MBInfoPtr->cursor_x = MBInfoPtr->cur_x+MBInfoPtr->str_w+1;
@@ -841,9 +782,9 @@ void DisplayInput(MBInfoPtr)
    if (msgFontPtr != NULL) {
       XSetFont(mainDisplay, defaultGC, msgFontPtr->fid);
    }
-   XDrawString(mainDisplay, win, defaultGC, MBInfoPtr->cur_x, MBInfoPtr->cur_y,
-         msg, buf_len);
-   if (msgFontPtr != NULL) {
+   DrawMsgString(mainDisplay, win, defaultGC, MBInfoPtr->cur_x,
+         MBInfoPtr->cur_y, msg, buf_len);
+   if (msgFontSet != NULL || msgFontPtr != NULL) {
       XSetFont(mainDisplay, defaultGC, defaultFontPtr->fid);
    }
    if (dup_buf != NULL) free(dup_buf);
@@ -858,6 +799,7 @@ int HandleMsgBoxKeyEvent(MBInfoPtr, input)
    KeySym key_sym;
    char buf[80];
    int i, j, has_ch;
+   int bg_pixel=(threeDLook ? myLtGryPixel : myBgPixel);
 
    has_ch = XLookupString(key_ev, buf, sizeof(buf), &key_sym, &c_stat);
    TranslateKeys(buf, &key_sym);
@@ -868,7 +810,7 @@ int HandleMsgBoxKeyEvent(MBInfoPtr, input)
       if (MBInfoPtr->exposed) {
          /* erase the old cursor */
          PutCursor(MBInfoPtr->btn_win, MBInfoPtr->cursor_x, MBInfoPtr->cursor_y,
-               myBgPixel);
+               bg_pixel);
       }
       if (CharIsESC(key_ev, buf, key_sym, &has_ch)) {
          if (MBInfoPtr->return_str != NULL) *MBInfoPtr->return_str = '\0';
@@ -932,6 +874,26 @@ int HandleMsgBoxKeyEvent(MBInfoPtr, input)
    return INVALID;
 }
 
+static
+int HandleDeleteMsgBox(MBInfoPtr)
+   struct MBRec *MBInfoPtr;
+{
+   int bg_pixel=(threeDLook ? myLtGryPixel : myBgPixel);
+
+   if (MBInfoPtr->is_dialog) {
+      /* for Dialog(), same as <ESC> is pressed */
+      if (MBInfoPtr->exposed) {
+         /* erase the old cursor */
+         PutCursor(MBInfoPtr->btn_win, MBInfoPtr->cursor_x, MBInfoPtr->cursor_y,
+               bg_pixel);
+      }
+      if (MBInfoPtr->return_str != NULL) *MBInfoPtr->return_str = '\0';
+      return INVALID;
+   }
+   /* for MsgBox(), return a button id */
+   return MB_ID_CANCEL;
+}
+
 static
 void HandlePasteInDialog(MBInfoPtr, input)
    struct MBRec *MBInfoPtr;
@@ -939,7 +901,7 @@ void HandlePasteInDialog(MBInfoPtr, input)
 {
    XButtonEvent *button_ev=(&(input->xbutton));
    int buf_len=0;
-   char *cut_buffer;
+   char *cut_buffer=NULL;
 
    if (button_ev->button != Button2) return;
 
@@ -953,7 +915,8 @@ void HandlePasteInDialog(MBInfoPtr, input)
       if (MBInfoPtr->exposed) {
          XClearWindow(mainDisplay, MBInfoPtr->btn_win);
       }
-      for ( ; MBInfoPtr->index < 80 && *c_ptr != '\0'; c_ptr++) {
+      for ( ; buf_len > 0 && MBInfoPtr->index < 80 && *c_ptr != '\0';
+            c_ptr++, buf_len--) {
          if (*c_ptr >= (unsigned char)('\040') &&
                *c_ptr < (unsigned char)('\377')) {
             MBInfoPtr->return_str[MBInfoPtr->index++] = (char)(*c_ptr);
@@ -961,6 +924,7 @@ void HandlePasteInDialog(MBInfoPtr, input)
             break;
          }
       }
+      MBInfoPtr->return_str[MBInfoPtr->index] = '\0';
       if (MBInfoPtr->exposed) {
          DisplayInput(MBInfoPtr);
          PutCursor(MBInfoPtr->btn_win, MBInfoPtr->cursor_x, MBInfoPtr->cursor_y,
@@ -990,11 +954,11 @@ int HandleMsgBoxBtnEvent(MBInfoPtr, input)
                   strlen(MBInfoPtr->btn_info[i].str),
                   &MBInfoPtr->btn_info[i].bbox, BUTTON_INVERT,
                   MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA);
-
-            XGrabPointer(mainDisplay, MBInfoPtr->btn_win, FALSE,
-                  PointerMotionMask | ButtonReleaseMask,
-                  GrabModeAsync, GrabModeAsync, None, defaultCursor,
-                  CurrentTime);
+            if (!debugNoPointerGrab) {
+               XGrabPointer(mainDisplay, MBInfoPtr->btn_win, FALSE,
+                     PointerMotionMask | ButtonReleaseMask, GrabModeAsync,
+                     GrabModeAsync, None, defaultCursor, CurrentTime);
+            }
             while (TRUE) {
                XEvent ev;
 
@@ -1075,28 +1039,28 @@ void RefreshMsgBox(MBInfoPtr)
          XSetFont(mainDisplay, defaultGC, boldMsgFontPtr->fid);
       }
       while (c_ptr != NULL) {
-         char *c_ptr1=strchr(c_ptr, '\n');
+         char *c_ptr1=BoldMsgStrChr(c_ptr, '\n');
          int len, w;
 
          if (c_ptr1 != NULL) *c_ptr1 = '\0';
          len = strlen(c_ptr);
-         if (boldMsgFontPtr == NULL) {
+         if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
             if (MBInfoPtr->is_dialog) {
                w = len * defaultFontWidth;
-               XDrawString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
+               DrawBoldMsgString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
                      (MBInfoPtr->msg_win_w-w)>>1, y+defaultFontAsc, c_ptr, len);
             } else {
-               XDrawString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
+               DrawBoldMsgString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
                      0, y+defaultFontAsc, c_ptr, len);
             }
             y += defaultFontHeight+SPACING;
          } else {
             if (MBInfoPtr->is_dialog) {
-               w = XTextWidth(boldMsgFontPtr, c_ptr, len);
-               XDrawString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
+               w = BoldMsgTextWidth(boldMsgFontPtr, c_ptr, len);
+               DrawBoldMsgString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
                      (MBInfoPtr->msg_win_w-w)>>1, y+boldMsgFontAsc, c_ptr, len);
             } else {
-               XDrawString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
+               DrawBoldMsgString(mainDisplay, MBInfoPtr->msg_win, defaultGC,
                      0, y+boldMsgFontAsc, c_ptr, len);
             }
             y += boldMsgFontHeight+SPACING;
@@ -1108,7 +1072,7 @@ void RefreshMsgBox(MBInfoPtr)
             break;
          }
       }
-      if (boldMsgFontPtr != NULL) {
+      if (boldMsgFontSet != NULL || boldMsgFontPtr != NULL) {
          XSetFont(mainDisplay, defaultGC, defaultFontPtr->fid);
       }
    }
@@ -1141,12 +1105,21 @@ void RefreshMsgBox(MBInfoPtr)
    while (XCheckWindowEvent(mainDisplay,MBInfoPtr->icon_win,ExposureMask,&ev)) ;
    while (XCheckWindowEvent(mainDisplay,MBInfoPtr->msg_win,ExposureMask,&ev)) ;
    while (XCheckWindowEvent(mainDisplay,MBInfoPtr->btn_win,ExposureMask,&ev)) ;
+
+   if (threeDLook && dialogboxUse3DBorder) {
+      struct BBRec bbox;
+
+      SetBBRec(&bbox, 0, 0, MBInfoPtr->main_win_w, MBInfoPtr->main_win_h);
+      TgDrawThreeDButton(mainDisplay, MBInfoPtr->main_win, textMenuGC, &bbox,
+            TGBS_RAISED, 2, FALSE);
+   }
 }
 
 int MsgBox(Message, Title, IconAndBtns)
    char *Message, *Title;
    int IconAndBtns;
 {
+   static int stShowing=FALSE;
    char *dup_msg=NULL;
    int rc=MB_ID_FAILED, looping=TRUE;
 
@@ -1159,11 +1132,21 @@ int MsgBox(Message, Title, IconAndBtns)
       FailAllocMessage();
       return rc;
    }
+   if (stShowing) return rc;
+
+   stShowing = TRUE;
    if (!SetupMBWindow(&mbInfo, dup_msg, Title, IconAndBtns, FALSE)) {
-      fprintf(stderr, "Invalid parameters passed to MsgBox().\n");
-      Msg("Invalid parameters passed to MsgBox().\n");
-      if (mbInfo.msg_copy != NULL) free(mbInfo.msg_copy);
+      char msg[MAXSTRING];
+
+      sprintf(msg, TgLoadString(STID_INVALID_PARAM_PASSED_TO_FUNC), "MsgBox()");
+      fprintf(stderr, "%s\n", msg);
+      Msg(msg);
+      if (mbInfo.msg_copy != NULL) {
+         free(mbInfo.msg_copy);
+         mbInfo.msg_copy = NULL;
+      }
       free(dup_msg);
+      stShowing = FALSE;
       return rc;
    }
    while (looping) {
@@ -1214,9 +1197,15 @@ int MsgBox(Message, Title, IconAndBtns)
          if ((rc=HandleMsgBoxBtnEvent(&mbInfo, &input)) != INVALID) {
             break;
          }
+      } else if (IsWM_DELETE_WINDOW(&input)) {
+         rc = HandleDeleteMsgBox(&mbInfo);
+         break;
       }
    }
-   if (mbInfo.msg_copy != NULL) free(mbInfo.msg_copy);
+   if (mbInfo.msg_copy != NULL) {
+      free(mbInfo.msg_copy);
+      mbInfo.msg_copy = NULL;
+   }
    free(dup_msg);
 
    XDestroyWindow(mainDisplay, mbInfo.main_win);
@@ -1224,6 +1213,7 @@ int MsgBox(Message, Title, IconAndBtns)
       XWarpPointer(mainDisplay, None, drawWindow, 0, 0, 0, 0,
             (int)(ZOOMED_SIZE(drawWinW)>>1), (int)(ZOOMED_SIZE(drawWinH)>>1));
    }
+   stShowing = FALSE;
    return rc;
 }
 
@@ -1238,18 +1228,27 @@ int DoDialog(Message, ReturnStr)
       FailAllocMessage();
       return INVALID;
    }
-   sprintf(szTitle, "%s - Input", TOOL_NAME);
+   sprintf(szTitle, TgLoadString(STID_TOOL_INPUT), TOOL_NAME);
    if (!SetupMBWindow(&mbInfo, dup_msg, szTitle, MB_ICON_DIALOG, TRUE)) {
-      fprintf(stderr, "Invalid parameters passed to MsgBox().\n");
-      Msg("Invalid parameters passed to MsgBox().\n");
-      if (mbInfo.msg_copy != NULL) free(mbInfo.msg_copy);
+      char msg[MAXSTRING];
+
+      sprintf(msg, TgLoadString(STID_INVALID_PARAM_PASSED_TO_FUNC), "MsgBox()");
+      fprintf(stderr, "%s\n", msg);
+      Msg(msg);
+      if (mbInfo.msg_copy != NULL) {
+         free(mbInfo.msg_copy);
+         mbInfo.msg_copy = NULL;
+      }
       free(dup_msg);
       return INVALID;
    }
-   if (ReturnStr != NULL) *ReturnStr = '\0';
+   if (ReturnStr != NULL) {
+      /* Use as is. */
+      /* *ReturnStr = '\0'; */
+   }
    mbInfo.cur_x = ((mbInfo.btn_win_w)>>1);
    mbInfo.cursor_x = mbInfo.cur_x + 1;
-   if (msgFontPtr == NULL) {
+   if (msgFontSet == NULL && msgFontPtr == NULL) {
       mbInfo.cur_y = ((mbInfo.btn_win_h-defaultFontHeight)>>1)+defaultFontAsc;
       mbInfo.cursor_y = mbInfo.cur_y - defaultFontAsc +
             ((defaultFontAsc-16)>>1);
@@ -1259,6 +1258,9 @@ int DoDialog(Message, ReturnStr)
             ((msgFontAsc-16)>>1);
    }
    mbInfo.index = 0;
+   if (ReturnStr != NULL) {
+      mbInfo.index = strlen(ReturnStr);
+   }
    mbInfo.return_str = ReturnStr;
 
    while (looping) {
@@ -1309,9 +1311,16 @@ int DoDialog(Message, ReturnStr)
          }
       } else if (input.type==ButtonPress && input.xany.window==mbInfo.btn_win) {
          HandlePasteInDialog(&mbInfo, &input);
+      } else if (IsWM_DELETE_WINDOW(&input)) {
+         looping = FALSE;
+         rc = INVALID;
+         break;
       }
    }
-   if (mbInfo.msg_copy != NULL) free(mbInfo.msg_copy);
+   if (mbInfo.msg_copy != NULL) {
+      free(mbInfo.msg_copy);
+      mbInfo.msg_copy = NULL;
+   }
    free(dup_msg);
 
    XDestroyWindow(mainDisplay, mbInfo.main_win);
@@ -1329,11 +1338,12 @@ int Dialog(Message, Comment, ReturnStr)
    /* if Comment is NULL, "( <CR> or <ESC> to continue )" is assumed */
    /* if ReturnStr is NULL, hitting any key will return INVALID */
 {
+   static int stDialoging=FALSE;
    char *real_msg=NULL, def_comment[MAXSTRING+1];
    int real_len=strlen(Message), rc;
 
    if (Comment == NULL) {
-      strcpy(def_comment, "( <CR> or <ESC> to continue )");
+      strcpy(def_comment, TgLoadCachedString(CSTID_DLG_DEF_CONTINUE));
       real_len += strlen(def_comment)+2;
    } else {
       real_len += strlen(Comment)+2;
@@ -1341,9 +1351,11 @@ int Dialog(Message, Comment, ReturnStr)
    real_msg = (char*)malloc((real_len+1)*sizeof(char));
    if (real_msg == NULL) {
       FailAllocMessage();
-      if (info_bits) { } /* do nothing but reference the variable */
       return INVALID;
    }
+   if (stDialoging) return INVALID;
+
+   stDialoging = TRUE;
    if (Comment == NULL) {
       sprintf(real_msg, "%s\n\n%s", Message, def_comment);
    } else {
@@ -1351,5 +1363,6 @@ int Dialog(Message, Comment, ReturnStr)
    }
    rc = DoDialog(real_msg, ReturnStr);
    free(real_msg);
+   stDialoging = FALSE;
    return rc;
 }
