@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,23 +28,20 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/dup.c,v 3.38 1998/11/05 22:56:10 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/dup.c,v 4.17 2000/01/15 22:23:37 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_DUP_C_
+
+#include "tgifdefs.h"
 
 #include "attr.e"
 #include "auxtext.e"
 #include "choice.e"
 #include "cmd.e"
 #include "drawing.e"
-#ifndef _NO_EXTERN
 #include "dup.e"
-#endif
+#include "file.e"
 #include "grid.e"
 #include "mark.e"
 #include "move.e"
@@ -70,16 +67,23 @@ void CopyObjId(FromObjPtr, ToObjPtr)
    register struct AttrRec *from_attr_ptr, *to_attr_ptr;
 
    ToObjPtr->id = FromObjPtr->id;
+   if (FromObjPtr->creator_full_id != NULL) {
+      UtilFree(ToObjPtr->creator_full_id);
+      ToObjPtr->creator_full_id = UtilStrDup(FromObjPtr->creator_full_id);
+      if (ToObjPtr->creator_full_id == NULL) FailAllocMessage();
+   }
    switch (FromObjPtr->type) {
    case OBJ_SYM:
    case OBJ_GROUP:
    case OBJ_ICON:
+   case OBJ_PIN:
       from_obj_ptr = FromObjPtr->detail.r->first;
       to_obj_ptr = ToObjPtr->detail.r->first;
       for ( ; from_obj_ptr!=NULL; from_obj_ptr=from_obj_ptr->next,
             to_obj_ptr=to_obj_ptr->next) {
          CopyObjId(from_obj_ptr, to_obj_ptr);
       }
+      break;
    }
    from_attr_ptr = FromObjPtr->fattr;
    to_attr_ptr = ToObjPtr->fattr;
@@ -100,12 +104,14 @@ void CopyObjLocks(FromObjPtr, ToObjPtr)
    case OBJ_SYM:
    case OBJ_GROUP:
    case OBJ_ICON:
+   case OBJ_PIN:
       from_obj_ptr = FromObjPtr->detail.r->first;
       to_obj_ptr = ToObjPtr->detail.r->first;
       for ( ; from_obj_ptr!=NULL; from_obj_ptr=from_obj_ptr->next,
             to_obj_ptr=to_obj_ptr->next) {
          CopyObjLocks(from_obj_ptr, to_obj_ptr);
       }
+      break;
    }
    from_attr_ptr = FromObjPtr->fattr;
    to_attr_ptr = ToObjPtr->fattr;
@@ -126,6 +132,7 @@ void UnlockAnObj(ObjPtr)
    case OBJ_SYM:
    case OBJ_GROUP:
    case OBJ_ICON:
+   case OBJ_PIN:
       for (obj_ptr=ObjPtr->detail.r->first; obj_ptr!=NULL;
             obj_ptr=obj_ptr->next) {
          UnlockAnObj(obj_ptr);
@@ -154,6 +161,11 @@ void DupObjBasics(FromObjPtr, ToObjPtr)
    ToObjPtr->x = FromObjPtr->x;
    ToObjPtr->y = FromObjPtr->y;
    ToObjPtr->color = FromObjPtr->color;
+   ToObjPtr->bg_color = FromObjPtr->bg_color;
+   memcpy(FromObjPtr->color_str, ToObjPtr->color_str,
+         sizeof(FromObjPtr->color_str));
+   memcpy(FromObjPtr->bg_color_str, ToObjPtr->bg_color_str,
+         sizeof(FromObjPtr->bg_color_str));
    ToObjPtr->id = objId++;
    ToObjPtr->dirty = FALSE;
    ToObjPtr->rotation = FromObjPtr->rotation;
@@ -168,6 +180,7 @@ void DupObjBasics(FromObjPtr, ToObjPtr)
    ToObjPtr->obbox.rby = FromObjPtr->obbox.rby;
    ToObjPtr->locked = FALSE;
    ToObjPtr->invisible = FromObjPtr->invisible;
+   ToObjPtr->trans_pat = FromObjPtr->trans_pat;
 
    memcpy(&ToObjPtr->orig_obbox, &FromObjPtr->orig_obbox,
          sizeof(struct BBRec));
@@ -257,6 +270,11 @@ void DupPolyObj(PolyPtr, ObjPtr)
       poly_ptr->rotated_n = 0;
       poly_ptr->rotated_vlist = NULL;
    }
+/*
+ * Need to check for pins.
+ * Need to handle the case where connection objects need to be dupped!
+ */
+   poly_ptr->start_conn = poly_ptr->end_conn = NULL;
 
    ObjPtr->detail.p = poly_ptr;
 }
@@ -498,11 +516,9 @@ void DupXBmObj(XBmPtr, ObjPtr)
    xbm_ptr->fill = XBmPtr->fill;
    xbm_ptr->image_w = image_w = XBmPtr->image_w;
    xbm_ptr->image_h = image_h = XBmPtr->image_h;
-   xbm_ptr->rotate = XBmPtr->rotate;
    xbm_ptr->flip = XBmPtr->flip;
    xbm_ptr->cached_zoom = XBmPtr->cached_zoom;
    xbm_ptr->cached_zoomed = XBmPtr->cached_zoomed;
-   xbm_ptr->cached_rotate = XBmPtr->cached_rotate;
    xbm_ptr->cached_flip = XBmPtr->cached_flip;
    xbm_ptr->cached_w = XBmPtr->cached_w;
    xbm_ptr->cached_h = XBmPtr->cached_h;
@@ -559,7 +575,7 @@ void DupXBmObj(XBmPtr, ObjPtr)
       xbm_ptr->epsflines = (char**)malloc(num_epsf_lines*sizeof(char*));
       if (xbm_ptr->epsflines == NULL) FailAllocMessage();
       for (i=0; i < num_epsf_lines; i++) {
-         len = strlen (XBmPtr->epsflines[i]);
+         len = strlen(XBmPtr->epsflines[i]);
          xbm_ptr->epsflines[i] = (char*)malloc((len+1)*sizeof(char));
          if (xbm_ptr->epsflines[i] == NULL) FailAllocMessage();
          strcpy(xbm_ptr->epsflines[i], XBmPtr->epsflines[i]);
@@ -588,11 +604,9 @@ void DupXPmObj(XPmPtr, ObjPtr)
    xpm_ptr->fill = XPmPtr->fill;
    xpm_ptr->image_w = image_w = XPmPtr->image_w;
    xpm_ptr->image_h = image_h = XPmPtr->image_h;
-   xpm_ptr->rotate = XPmPtr->rotate;
    xpm_ptr->flip = XPmPtr->flip;
    xpm_ptr->cached_zoom = XPmPtr->cached_zoom;
    xpm_ptr->cached_zoomed = XPmPtr->cached_zoomed;
-   xpm_ptr->cached_rotate = XPmPtr->cached_rotate;
    xpm_ptr->cached_flip = XPmPtr->cached_flip;
    xpm_ptr->cached_w = XPmPtr->cached_w;
    xpm_ptr->cached_h = XPmPtr->cached_h;
@@ -685,74 +699,144 @@ void DupXPmObj(XPmPtr, ObjPtr)
    }
 }
 
+void DupStrSeg(pToStrBlock, pFromStrSeg)
+   StrBlockInfo *pToStrBlock;
+   StrSegInfo *pFromStrSeg;
+   /* pToStrBlock->type is either SB_SIMPLE or SB_SUPSUB_CENTER */
+   /* pToStrBlock->seg is the newly created string segment */
+{
+   StrSegInfo *str_seg=(StrSegInfo*)malloc(sizeof(StrSegInfo));
+
+   if (str_seg == NULL) FailAllocMessage();
+   memcpy(str_seg, pFromStrSeg, sizeof(StrSegInfo));
+   if (PRTGIF && pFromStrSeg->font_name != NULL &&
+         *pFromStrSeg->font_name != '\0') {
+      /*
+       * this should never happen anyway (because DupStrSeg() is never called
+       *       in PRTGIF)
+       */
+      str_seg->font_name = UtilStrDup(pFromStrSeg->font_name);
+      if (str_seg->font_name == NULL) FailAllocMessage();
+   }
+   str_seg->dyn_str.s = NULL;
+   str_seg->dyn_str.sz = 0;
+   DynStrCpy(&str_seg->dyn_str, &pFromStrSeg->dyn_str);
+   str_seg->owner = pToStrBlock;
+   pToStrBlock->seg = str_seg;
+}
+
+void DupStrBlock(pStrBlock, pMiniLineOwner, ppFirstStrBlock, ppLastStrBlock)
+   StrBlockInfo *pStrBlock, **ppFirstStrBlock, **ppLastStrBlock;
+   MiniLineInfo *pMiniLineOwner;
+{
+   StrBlockInfo *str_block=(StrBlockInfo*)malloc(sizeof(StrBlockInfo));
+
+   if (str_block == NULL) FailAllocMessage();
+   memcpy(str_block, pStrBlock, sizeof(StrBlockInfo));
+   str_block->seg = NULL;
+   str_block->sup = str_block->sub = NULL;
+   str_block->owner_mini_line = pMiniLineOwner;
+   str_block->next = NULL;
+   str_block->prev = (*ppLastStrBlock);
+   if ((*ppLastStrBlock) == NULL) {
+      (*ppFirstStrBlock) = str_block;
+   } else {
+      (*ppLastStrBlock)->next = str_block;
+   }
+   (*ppLastStrBlock) = str_block;
+   switch (pStrBlock->type) {
+   case SB_SIMPLE:
+      DupStrSeg(str_block, pStrBlock->seg);
+      break;
+   case SB_CHAR_SPACE: break;
+
+   case SB_SUPSUB_LEFT:
+   case SB_SUPSUB_CENTER:
+   case SB_SUPSUB_RIGHT:
+      if (pStrBlock->sup != NULL) {
+         DupMiniLines(pStrBlock->sup, str_block, &str_block->sup);
+      }
+      if (pStrBlock->sub != NULL) {
+         DupMiniLines(pStrBlock->sub, str_block, &str_block->sub);
+      }
+      if (pStrBlock->type == SB_SUPSUB_CENTER) {
+         DupStrSeg(str_block, pStrBlock->seg);
+      }
+      break;
+   }
+}
+
+void DupMiniLine(pMiniLine, pOwnerMiniLines, pOwnerBlock, ppFirstMiniLine,
+      ppLastMiniLine)
+   MiniLineInfo *pMiniLine, **ppFirstMiniLine, **ppLastMiniLine;
+   MiniLinesInfo *pOwnerMiniLines;
+   StrBlockInfo *pOwnerBlock;
+{
+   MiniLineInfo *new_mini_line=(MiniLineInfo*)malloc(sizeof(MiniLineInfo));
+   StrBlockInfo *pStrBlock=NULL;
+
+   if (new_mini_line == NULL) FailAllocMessage();
+   memcpy(new_mini_line, pMiniLine, sizeof(MiniLineInfo));
+   new_mini_line->first_block = new_mini_line->last_block = NULL;
+   new_mini_line->owner_minilines = pOwnerMiniLines;
+   new_mini_line->next = NULL;
+   new_mini_line->prev = (*ppLastMiniLine);
+   if ((*ppLastMiniLine) == NULL) {
+      (*ppFirstMiniLine) = new_mini_line;
+   } else {
+      (*ppLastMiniLine)->next = new_mini_line;
+   }
+   (*ppLastMiniLine) = new_mini_line;
+   for (pStrBlock=pMiniLine->first_block; pStrBlock != NULL;
+         pStrBlock=pStrBlock->next) {
+      DupStrBlock(pStrBlock, new_mini_line, &new_mini_line->first_block,
+            &new_mini_line->last_block);
+   }
+}
+
+void DupMiniLines(minilines, pOwnerBlock, ppMinilines)
+   MiniLinesInfo *minilines, **ppMinilines;
+   StrBlockInfo *pOwnerBlock;
+{
+   MiniLinesInfo *new_minilines=(MiniLinesInfo*)malloc(sizeof(MiniLinesInfo));
+   MiniLineInfo *pMiniLine=NULL, *pFirstMiniLine=NULL, *pLastMiniLine=NULL;
+
+   if (new_minilines == NULL) FailAllocMessage();
+   memcpy(new_minilines, minilines, sizeof(MiniLinesInfo));
+   new_minilines->first = new_minilines->last = NULL;
+   new_minilines->owner_block = pOwnerBlock;
+   for (pMiniLine=minilines->first; pMiniLine != NULL;
+         pMiniLine=pMiniLine->next) {
+      DupMiniLine(pMiniLine, new_minilines, pOwnerBlock, &pFirstMiniLine,
+            &pLastMiniLine);
+   }
+   new_minilines->first = pFirstMiniLine;
+   new_minilines->last = pLastMiniLine;
+
+   if (ppMinilines != NULL) {
+      *ppMinilines = new_minilines;
+   }
+}
+
 void DupTextObj(TextPtr, FromObjPtr, ToObjPtr)
    struct TextRec *TextPtr;
    struct ObjRec *FromObjPtr, *ToObjPtr;
 {
-   register int i, num_lines;
-   register struct TextRec *text_ptr;
-   struct StrRec *first_str;
-   struct StrRec *from_str_ptr, *to_str_ptr;
+   struct TextRec *text_ptr=NULL;
+   MiniLinesInfo *minilines=NULL;
+   MiniLineInfo *pMiniLine=NULL;
 
    text_ptr = (struct TextRec *)malloc(sizeof(struct TextRec));
    if (text_ptr == NULL) FailAllocMessage();
-   memset(text_ptr, 0, sizeof(struct TextRec));
-   text_ptr->just = TextPtr->just;
-   text_ptr->v_space = TextPtr->v_space;
-   num_lines = text_ptr->lines = TextPtr->lines;
-   from_str_ptr = TextPtr->last;
-   first_str = NULL;
-   for (i=0; i < num_lines; i++, from_str_ptr=from_str_ptr->prev) {
-      to_str_ptr = NewStr();
-      DynStrCpy(&to_str_ptr->dyn_str, &from_str_ptr->dyn_str);
-      to_str_ptr->next = first_str;
-      if (first_str == NULL) {
-         text_ptr->last = to_str_ptr;
-      } else {
-         first_str->prev = to_str_ptr;
-      }
-      first_str = to_str_ptr;
-   }
-   first_str->prev = NULL;
-   text_ptr->first = first_str;
+   memcpy(text_ptr, TextPtr, sizeof(struct TextRec));
 
-   text_ptr->font = TextPtr->font;
-   text_ptr->style = TextPtr->style;
-   text_ptr->size = TextPtr->size;
-   text_ptr->rotate = TextPtr->rotate;
-   text_ptr->pen = TextPtr->pen;
-   text_ptr->fill = TextPtr->fill;
-   text_ptr->asc = TextPtr->asc;
-   text_ptr->des = TextPtr->des;
-   text_ptr->underline_on = TextPtr->underline_on;
-   text_ptr->underline = TextPtr->underline;
-   text_ptr->min_lbearing = TextPtr->min_lbearing;
-   text_ptr->max_rextra = TextPtr->max_rextra;
-   text_ptr->double_byte = TextPtr->double_byte;
-   text_ptr->direction = TextPtr->direction;
-   if (TextPtr->custom_screen_font_name == NULL) {
-      text_ptr->custom_screen_font_name = NULL;
-   } else {
-      text_ptr->custom_screen_font_name =
-            UtilStrDup(TextPtr->custom_screen_font_name);
-   }
-   text_ptr->cached_zoom = TextPtr->cached_zoom;
+   text_ptr->attr = NULL;
    text_ptr->cached_bitmap = None;
-   text_ptr->image = NULL;
-   text_ptr->cached_zoomed = TextPtr->cached_zoomed;
-   text_ptr->cached_rotate = TextPtr->cached_rotate;
-
-   text_ptr->read_only = TextPtr->read_only;
-   text_ptr->orig_w = TextPtr->orig_w;
-   text_ptr->orig_h = TextPtr->orig_h;
-   text_ptr->font_name = NULL;
+   text_ptr->cached_pixmap = None;
+   text_ptr->cached_bg_bitmap = None;
 
    ToObjPtr->detail.t = text_ptr;
 
-   if (ToObjPtr->ctm != NULL) {
-      memcpy(&text_ptr->cached_ctm, &TextPtr->cached_ctm,
-            sizeof(struct XfrmMtrxRec));
-   }
    if (TextPtr->cached_bitmap != None) {
       int w=ZOOMED_SIZE(FromObjPtr->bbox.rbx - FromObjPtr->bbox.ltx - 2);
       int h=ZOOMED_SIZE(FromObjPtr->bbox.rby - FromObjPtr->bbox.lty - 2);
@@ -767,14 +851,49 @@ void DupTextObj(TextPtr, FromObjPtr, ToObjPtr)
       XCopyArea(mainDisplay, TextPtr->cached_bitmap, text_ptr->cached_bitmap,
             rotateGC, 0, 0, w, h, 0, 0);
    }
-   if (ToObjPtr->ctm != NULL) {
-      memcpy(&text_ptr->orig_bbox,&TextPtr->orig_bbox,sizeof(struct BBRec));
+   if (TextPtr->cached_bg_bitmap != None) {
+      int w=ZOOMED_SIZE(FromObjPtr->bbox.rbx - FromObjPtr->bbox.ltx - 2);
+      int h=ZOOMED_SIZE(FromObjPtr->bbox.rby - FromObjPtr->bbox.lty - 2);
+
+      if (w == 0) w = 1;
+      if (h == 0) h = 1;
+      text_ptr->cached_bg_bitmap = XCreatePixmap(mainDisplay,mainWindow,w,h,1);
+      if (text_ptr->cached_bg_bitmap == None) {
+         FailAllocBitmapMessage(w, h);
+         return;
+      }
+      XCopyArea(mainDisplay, TextPtr->cached_bg_bitmap,
+            text_ptr->cached_bg_bitmap, rotateGC, 0, 0, w, h, 0, 0);
+   }
+   if (TextPtr->cached_pixmap != None) {
+      int w=ZOOMED_SIZE(FromObjPtr->bbox.rbx - FromObjPtr->bbox.ltx - 2);
+      int h=ZOOMED_SIZE(FromObjPtr->bbox.rby - FromObjPtr->bbox.lty - 2);
+
+      if (w == 0) w = 1;
+      if (h == 0) h = 1;
+      text_ptr->cached_pixmap = XCreatePixmap(mainDisplay, mainWindow, w, h,
+            mainDepth);
+      if (text_ptr->cached_pixmap == None) {
+         FailAllocBitmapMessage(w, h);
+         return;
+      }
+      XCopyArea(mainDisplay, TextPtr->cached_pixmap, text_ptr->cached_pixmap,
+            xpmGC, 0, 0, w, h, 0, 0);
+   }
+   DupMiniLines(&FromObjPtr->detail.t->minilines, NULL, &minilines);
+   memcpy(&text_ptr->minilines, minilines, sizeof(MiniLinesInfo));
+   free(minilines);
+   minilines = (&text_ptr->minilines);
+   for (pMiniLine=minilines->first; pMiniLine != NULL;
+         pMiniLine=pMiniLine->next) {
+      pMiniLine->owner_minilines = minilines;
    }
 }
 
-#ifdef _NO_EXTERN
-struct ObjRec *DupObj ARGS_DECL((struct ObjRec *));
-#endif /* _NO_EXTERN */
+void DupConnection(FromConnPtr, ToConnPtr)
+   struct ConnRec *FromConnPtr, *ToConnPtr;
+{
+}
 
 void DupGroupObj(GroupPtr, ObjPtr)
    struct GroupRec *GroupPtr;
@@ -852,13 +971,20 @@ struct ObjRec *DupObj(ObjPtr)
    case OBJ_SYM:
    case OBJ_GROUP:
    case OBJ_ICON:
+   case OBJ_PIN:
       DupGroupObj(ObjPtr->detail.r, obj_ptr);
       DupAttrs(ObjPtr, obj_ptr);
-      if (obj_ptr->type == OBJ_ICON) {
+      if (obj_ptr->type == OBJ_ICON || obj_ptr->type == OBJ_PIN) {
          strcpy(obj_ptr->detail.r->s, ObjPtr->detail.r->s);
          obj_ptr->detail.r->rotate = ObjPtr->detail.r->rotate;
          obj_ptr->detail.r->flip = ObjPtr->detail.r->flip;
          obj_ptr->detail.r->deck_index = (-1);
+/*
+ * Need to check for pins.
+ * Need to handle the case where connection objects need to be dupped!
+ */
+         obj_ptr->detail.r->pin_connected = 0;
+         obj_ptr->detail.r->first_conn = obj_ptr->detail.r->last_conn = NULL;
       }
       break;
    }
@@ -872,7 +998,10 @@ void DupSelObj()
    int dx=0, dy=0;
 
    if (topSel==NULL || curChoice==VERTEXMODE) return;
-
+/*
+ * Need to check for pins.
+ * Need to handle the case where connection objects need to be dupped!
+ */
    top_obj = bot_obj = NULL;
    for (sel_ptr=botSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
       obj_ptr = DupObj(sel_ptr->obj);
@@ -907,14 +1036,14 @@ void DupSelObj()
    } else {
       switch (gridSystem) {
       case ENGLISH_GRID:
-         if (gridOn) {
+         if (snapOn) {
             dupDx = dupDy = dx = dy = GRID_ABS_SIZE(xyEnglishGrid);
          } else {
             dupDx = dupDy = dx = dy = GRID_ABS_SIZE(DEFAULT_ENGLISH_GRID);
          }
          break;
       case METRIC_GRID:
-         if (gridOn) {
+         if (snapOn) {
             dupDx = dupDy = dx = dy = GRID_ABS_SIZE(xyMetricGrid);
          } else {
             dupDx = dupDy = dx = dy = GRID_ABS_SIZE(DEFAULT_METRIC_GRID);
