@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,18 +28,12 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/eps.c,v 3.31 1998/11/05 22:56:11 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.20 2000/01/15 22:23:38 william Exp $
  */
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <time.h>
-#include <stdio.h>
-#include <string.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_EPS_C_
+
+#include "tgifdefs.h"
 
 #include "cmd.e"
 #include "color.e"
@@ -47,18 +41,20 @@
 #include "dialog.e"
 #include "drawing.e"
 #include "dup.e"
-#ifndef _NO_EXTERN
 #include "eps.e"
-#endif
 #include "file.e"
 #include "grid.e"
 #include "mark.e"
 #include "msg.e"
+#include "names.e"
 #include "obj.e"
 #include "pattern.e"
+#include "ps.e"
 #include "rect.e"
 #include "select.e"
 #include "setup.e"
+#include "special.e"
+#include "strtbl.e"
 #include "util.e"
 #include "xbitmap.e"
 
@@ -66,65 +62,55 @@
 #define SEEK_SET 0
 #endif /* ~SEEK_SET */
 
-extern char *mktemp ARGS_DECL((char*));
-
-#if defined (__hpux) || defined (linux) || defined(sco)
-extern double	atof ARGS_DECL((const char *));
-#else
-extern double	atof ARGS_DECL((char *));
-#endif
-
-typedef struct LineRec {
-   char			* s;
-   struct LineRec	* next, * prev;
-} * LineRecPtr;
+typedef struct EPSLineRec {
+   char *s;
+   struct EPSLineRec *next, *prev;
+} *EPSLineRecPtr;
 
 float	defaultEPSScaling=1.0;
 char	defaultEPSScalingStr[80];
 
-static struct LineRec	* topLine=NULL, * botLine=NULL;
-static int		numLines = 0;
+static struct EPSLineRec *topLine=NULL, *botLine=NULL;
+static int numLines=0;
 
 static char tiffToXbmCmd[MAXSTRING+1];
 static int tiffToXbmCmdInitialized=FALSE;
 
+/* do not translate -- program constants */
 static char hexValue[]="0123456789abcdef";
 static char flippedHexValue[]="084c2a6e195d3b7f";
 
-void InitEPS ()
+static char defTiffToXbmCmd[]="tifftopnm %s | pgmtopbm | pbmtoxbm";
+
+void InitEPS()
 {
-   char		* c_ptr;
-
-   stripEPSComments = TRUE;
-   if ((c_ptr = XGetDefault(mainDisplay,TOOL_NAME,"StripEPSComments")) != NULL)
-      if (strcmp ("False", c_ptr) == 0 || strcmp ("false", c_ptr) == 0)
-         stripEPSComments = FALSE;
-
-   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ForceClearAfterEPS")) != NULL)
-      fprintf (stderr, "%s*ForceClearAfterEPS is obsolete.\n", TOOL_NAME);
+   char *c_ptr=NULL;
 
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ForceClearAfterEPS")) !=
+         NULL) {
+      fprintf(stderr, TgLoadString(STID_NAMED_XDEF_IS_OBSOLETE), TOOL_NAME,
+            "ForceClearAfterEPS");
+      fprintf(stderr, "\n");
+   }
    defaultEPSScaling = 1.0;
-   strcpy (defaultEPSScalingStr, "1");
-   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"DefaultEPSScaling")) != NULL)
-   {
-      strcpy (defaultEPSScalingStr, c_ptr);
-      defaultEPSScaling = (float) atof (c_ptr);
-      if (defaultEPSScaling <= 0.0)
-      {
-         fprintf (stderr, "Invalid %s*DefaultEPSScaling: '%s', %s.\n",
-               TOOL_NAME, c_ptr, "1.0 is used");
+   strcpy(defaultEPSScalingStr, "1");
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"DefaultEPSScaling")) != NULL) {
+      strcpy(defaultEPSScalingStr, c_ptr);
+      defaultEPSScaling = (float)atof(c_ptr);
+      if (defaultEPSScaling <= 0.0) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "DefaultEPSScaling", c_ptr, 1);
          defaultEPSScaling = 1.0;
-         strcpy (defaultEPSScalingStr, "1");
-      }
-      else if (strcmp(defaultEPSScalingStr,"1")==0 ||
-            strcmp(defaultEPSScalingStr,"1.0")==0)
-      {
+         strcpy(defaultEPSScalingStr, "1");
+      } else if (strcmp(defaultEPSScalingStr,"1")==0 ||
+            strcmp(defaultEPSScalingStr,"1.0")==0) {
          defaultEPSScaling = 1.0;
-         strcpy (defaultEPSScalingStr, "1");
+         strcpy(defaultEPSScalingStr, "1");
       }
    }
    tiffToXbmCmdInitialized = TRUE;
-   strcpy(tiffToXbmCmd, "tifftopnm %s | pgmtopbm | pbmtoxbm");
+   /* do not translate -- program constants */
+   strcpy(tiffToXbmCmd, defTiffToXbmCmd);
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"TiffToXbm")) != NULL) {
       int count=0;
 
@@ -134,10 +120,9 @@ void InitEPS ()
          count++;
       }
       if (count != 1) {
-         fprintf(stderr, "Invalid %s*%s: '%s'.\n\n'%s' used.",
-               TOOL_NAME, "TiffToXbm", tiffToXbmCmd,
-               "tifftopnm %s | pgmtopbm | pbmtoxbm");
-         strcpy(tiffToXbmCmd, "tifftopnm %s | pgmtopbm | pbmtoxbm");
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "TiffToXbm", tiffToXbmCmd, defTiffToXbmCmd);
+         strcpy(tiffToXbmCmd, defTiffToXbmCmd);
       }
    }
 }
@@ -145,9 +130,9 @@ void InitEPS ()
 static
 void CleanUpLines()
 {
-   register struct LineRec *line_ptr, *next_line;
+   struct EPSLineRec *line_ptr, *next_line;
 
-   for (line_ptr=topLine; line_ptr != NULL; line_ptr = next_line) {
+   for (line_ptr=topLine; line_ptr != NULL; line_ptr=next_line) {
       next_line = line_ptr->next;
       if (line_ptr->s != NULL) free(line_ptr->s);
       free(line_ptr);
@@ -156,10 +141,10 @@ void CleanUpLines()
    numLines = 0;
 }
 
-void CleanUpEPS ()
+void CleanUpEPS()
 {
-   CleanUpLines ();
-   stripEPSComments = TRUE;
+   CleanUpLines();
+   stripEPSComments = FALSE;
 }
 
 static
@@ -188,33 +173,34 @@ void DumpXBmByte(fp, num_nibbles, image_h, nibble_index, row_index,
 }
 
 static
-int ReadPreviewBitmap (fp, image_w, image_h, bps, bitmap, image)
-   FILE		* fp;
-   int		image_w, image_h, bps;
-   Pixmap	* bitmap;
-   XImage	* * image;
+int ReadPreviewBitmap(fp, file_name, image_w, image_h, bps, bitmap, image)
+   FILE *fp;
+   char *file_name;
+   int image_w, image_h, bps;
+   Pixmap *bitmap;
+   XImage **image;
 {
-   register int	k, j;
-   int		i, num_nibbles, bit_count, out_byte_count=0;
-   char		* line, * c_ptr;
+   int k, j;
+   int i, num_nibbles, bit_count, out_byte_count=0;
+   char *line, *c_ptr;
 
-   switch (bps)
-   {
-      case 1:
-         num_nibbles = ((image_w & 0x3)==0) ? (int)(image_w>>2) :
-               (int)(image_w>>2)+1;
-         if (num_nibbles & 0x1) num_nibbles++;
-         break;
-      default:
-         sprintf(gszMsgBox, "%1d bits per sample preview not supported.", bps);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         return FALSE;
+   switch (bps) {
+   case 1:
+      num_nibbles = ((image_w & 0x3)==0) ? (int)(image_w>>2) :
+            (int)(image_w>>2)+1;
+      if (num_nibbles & 0x1) num_nibbles++;
+      break;
+   default:
+      sprintf(gszMsgBox, TgLoadString(STID_GIVEN_BPS_PREVIEW_NOT_SUP), bps);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return FALSE;
    }
    line = (char*)malloc((num_nibbles+10)*sizeof(char));
    if (line == NULL) FailAllocMessage();
    if (cmdLineDosEpsFilter && cmdLinePreviewOnly) {
       *bitmap = None;
       *image = NULL;
+      /* do not translate -- program constants */
       printf("#define noname_width %1d\n", image_w);
       printf("#define noname_height %1d\n", image_h);
       printf("#define noname_x_hot 0\n");
@@ -230,9 +216,10 @@ int ReadPreviewBitmap (fp, image_w, image_h, bps, bitmap, image)
 
       for (j=0, c_ptr = line; j < num_nibbles; j++, c_ptr++) {
          while (TRUE) {
-            if ((*c_ptr = (char) getc (fp)) == EOF) {
-               MsgBox("Invalid preview bitmap in EPS file.", TOOL_NAME,
-                     INFO_MB);
+            if ((*c_ptr=(char)getc(fp)) == (char)EOF) {
+               sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BMP_IN_EPS),
+                     file_name);
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
                free(line);
                if (*bitmap != None) {
                   XFreePixmap(mainDisplay, *bitmap);
@@ -292,7 +279,9 @@ int ReadPreviewBitmap (fp, image_w, image_h, bps, bitmap, image)
       }
    }
    if (fgets(line, MAXSTRING, fp) == NULL) {
-      MsgBox("Invalid preview bitmap in EPS file.", TOOL_NAME, INFO_MB);
+      sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BMP_IN_EPS),
+            file_name);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       free(line);
       if (*bitmap != None) {
          XFreePixmap(mainDisplay, *bitmap);
@@ -309,25 +298,26 @@ int ReadPreviewBitmap (fp, image_w, image_h, bps, bitmap, image)
       XPutImage(mainDisplay,*bitmap,xbmGC,*image,0,0,0,0,image_w,image_h);
    }
    free(line);
-   return (TRUE);
+   return TRUE;
 }
 
 static
 void AddLine(s)
    char *s;
 {
-   struct LineRec *line_ptr;
+   struct EPSLineRec *line_ptr;
 
-   line_ptr = (struct LineRec *)malloc(sizeof(struct LineRec));
+   line_ptr = (struct EPSLineRec *)malloc(sizeof(struct EPSLineRec));
    if (line_ptr == NULL) FailAllocMessage();
    line_ptr->s = s;
 
    line_ptr->next = NULL;
    line_ptr->prev = botLine;
-   if (botLine == NULL)
+   if (botLine == NULL) {
       topLine = line_ptr;
-   else
+   } else {
       botLine->next = line_ptr;
+   }
    botLine = line_ptr;
 
    numLines++;
@@ -396,6 +386,7 @@ int XbmToPreviewBitmap(FP, xbm_fname)
          }
       }
    }
+   /* do not translate -- program constants */
    fprintf(FP, "%%!\n");
    fprintf(FP, "%%%%BeginPreview: %1d %1d 1 %1d\n", image_w, image_h,
          num_lines*image_h);
@@ -418,7 +409,7 @@ int XbmToPreviewBitmap(FP, xbm_fname)
          fprintf(FP, "%c", hexValue[(data[row][col]>>4) & 0xf]);
          fprintf(FP, "%c", hexValue[data[row][col] & 0xf]);
       }
-      fprintf (FP, "\n");
+      fprintf(FP, "\n");
    }
    fprintf(FP, "%%%%EndImage\n");
    fprintf(FP, "%%%%EndPreview\n");
@@ -440,9 +431,7 @@ int ErrorInConvertTiffToXbm(fp, tiff_fp, xbm_fp, fname)
    if (xbm_fp != NULL) fclose(xbm_fp);
    if (fname != NULL) {
       if (writeFileFailed) {
-         sprintf(gszMsgBox,
-               "Fail to write to '%s'.\n\nFile system may be full.", fname);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         FailToWriteFileMessage(fname);
       }
       unlink(fname);
    }
@@ -465,6 +454,7 @@ int ConvertTiffToXbm(fp, tiff_offset, tiff_sz, xbm_fname)
    if (fseek(fp, tiff_offset, SEEK_SET) != 0) {
       return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
    }
+   /* do not translate -- program constants */
    sprintf(tiff_fname, "%sTgifXXXXXX", TMP_DIR);
    mktemp(tiff_fname);
    unlink(tiff_fname);
@@ -497,7 +487,7 @@ int ConvertTiffToXbm(fp, tiff_offset, tiff_sz, xbm_fname)
       strcpy(tiffToXbmCmd, "tifftopnm %s | pgmtopbm | pbmtoxbm");
    }
    sprintf(cmd, tiffToXbmCmd, tiff_fname);
-   sprintf(gszMsgBox, "Executing '%s'...", cmd);
+   sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM), cmd);
    SetStringStatus(gszMsgBox);
    if (mainDisplay != NULL) XSync(mainDisplay, False);
    if ((pfp=(FILE*)popen(cmd,"r")) == NULL) {
@@ -512,7 +502,7 @@ int ConvertTiffToXbm(fp, tiff_offset, tiff_sz, xbm_fname)
       }
    }
    pclose(pfp);
-   SetStringStatus("...Done");
+   SetStringStatus(TgLoadCachedString(CSTID_DOTS_DONE));
    fclose(xbm_fp);
    xbm_fp = NULL;
    unlink(tiff_fname);
@@ -532,9 +522,7 @@ FILE *ErrorInGetTiffEPSIInfo(fp, eps_fp, fname)
    if (eps_fp != NULL) fclose(eps_fp);
    if (fname != NULL) {
       if (writeFileFailed) {
-         sprintf(gszMsgBox,
-               "Fail to write to '%s'.\n\nFile system may be full.", fname);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         FailToWriteFileMessage(fname);
       }
       unlink(fname);
    }
@@ -586,8 +574,7 @@ FILE *GetTiffEPSIInfo(fp, pszEPS, pnPreviewOK)
          }
          fclose(xbm_fp);
          if (writeFileFailed) {
-            fprintf(stderr,
-                  "Fail to write to a file.\n\nFile system may be full.");
+            fprintf(stderr, TgLoadString(STID_FAIL_TO_WRITE_TO_STDOUT));
          }
          unlink(xbm_fname);
          *xbm_fname = '\0';
@@ -653,6 +640,32 @@ FILE *GetTiffEPSIInfo(fp, pszEPS, pnPreviewOK)
    return fopen(pszEPS, "r");
 }
 
+static
+void GetEPSFullPath(file_name, full_path, buf_sz)
+   char *file_name, *full_path;
+   int buf_sz;
+{
+   if (*file_name == DIR_SEP) {
+      UtilStrCpyN(full_path, buf_sz, file_name);
+   } else {
+      char *c_ptr=NULL;
+
+      if (PRTGIF && !cmdLineOpenDisplay && *cmdLineOpenFile != '\0') {
+         UtilStrCpyN(full_path, buf_sz, cmdLineOpenFile);
+      } else if (*scanFileFullPath == DIR_SEP) {
+         UtilStrCpyN(full_path, buf_sz, scanFileFullPath);
+      } else {
+         sprintf(full_path, "%s%c%s", curDir, DIR_SEP, scanFileFullPath);
+      }
+      if ((c_ptr=UtilStrRChr(full_path, (int)DIR_SEP)) != NULL) {
+         strcpy(++c_ptr, file_name);
+      } else {
+         /* Umm... this should not happen */
+         UtilStrCpyN(full_path, buf_sz, file_name);
+      }
+   }
+}
+
 static
 int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
       pn_num_lines, ppsz_epsflines, pn_epsf_level, pf_llx, pf_lly,
@@ -664,10 +677,11 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
    char ***ppsz_epsflines, *psz_write_date;
    float *pf_llx, *pf_lly, *pf_urx, *pf_ury;
 {
-   register struct LineRec *line_ptr, *next_line;
-   register int i;
+   struct EPSLineRec *line_ptr, *next_line;
+   int i;
    char *line=NULL, *c_ptr, loc_time[MAXSTRING+1];
    char tiff_eps_fname[MAXPATHLENGTH+1];
+   char real_fname[MAXPATHLENGTH+1];
    int first_line=TRUE, image_w=0, image_h=0, epsf_level=0;
    int boundingbox_found=FALSE, preview_found=FALSE, found;
    int boundingbox_atend=FALSE, tiff_preview_ok=FALSE;
@@ -688,7 +702,10 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
       *pf_llx = llx; *pf_lly = lly; *pf_urx = urx; *pf_ury = ury;
       *psz_write_date = '\0';
    }
-   if ((fp=fopen(file_name, "r")) == NULL) return BitmapOpenFailed;
+   *real_fname = '\0';
+   GetEPSFullPath(file_name, real_fname, sizeof(real_fname));
+
+   if ((fp=fopen(real_fname, "r")) == NULL) return BitmapOpenFailed;
 
    CleanUpLines();
 
@@ -704,8 +721,9 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
             if (cmdLineDosEpsFilter) {
                if ((fp=GetTiffEPSIInfo(fp, tiff_eps_fname,
                      &tiff_preview_ok)) == NULL) {
-                  sprintf(gszMsgBox, "Fail to parse Windows EPS file '%s'.",
-                        file_name);
+                  sprintf(gszMsgBox,
+                        TgLoadString(STID_FAIL_TO_PARSE_WINEPS_FILE),
+                        real_fname);
                   if (PRTGIF || mainWindow==None) {
                      fprintf(stderr, "%s\n", gszMsgBox);
                   } else {
@@ -721,8 +739,8 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
             } else if ((fp=GetTiffEPSIInfo(fp, tiff_eps_fname,
                   &tiff_preview_ok)) == NULL ||
                   (line=UtilGetALine(fp)) == NULL) {
-               sprintf(gszMsgBox, "Fail to parse Windows EPS file '%s'.",
-                     file_name);
+               sprintf(gszMsgBox,
+                     TgLoadString(STID_FAIL_TO_PARSE_WINEPS_FILE), real_fname);
                if (PRTGIF || mainWindow==None) {
                   fprintf(stderr, "%s\n", gszMsgBox);
                } else {
@@ -777,14 +795,16 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
          char *preview_line;
 
          if (sscanf(&(line[15]), "%d %d %d", &image_w, &image_h, &bps) != 3) {
-            MsgBox("Invalid preview box in EPS file.", TOOL_NAME, INFO_MB);
+            sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BOX_IN_EPS),
+                  real_fname);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             if (need_to_free_line) free(line);
             fclose(fp);
             if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
             CleanUpLines();
             return BitmapFileInvalid;
          } else if (bps != 1) {
-            sprintf(gszMsgBox, "%1d bits per sample preview not supported.",
+            sprintf(gszMsgBox, TgLoadString(STID_GIVEN_BPS_PREVIEW_NOT_SUP),
                   bps);
             MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             if (need_to_free_line) free(line);
@@ -820,8 +840,11 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
             }
          }
          preview_found = TRUE;
-         if (!ReadPreviewBitmap(fp, image_w, image_h, bps, &bitmap, &image)) {
-            MsgBox("Invalid preview box in EPS file.", TOOL_NAME, INFO_MB);
+         if (!ReadPreviewBitmap(fp, real_fname, image_w, image_h, bps, &bitmap,
+               &image)) {
+            sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BOX_IN_EPS),
+                  real_fname);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             if (need_to_free_line) free(line);
             fclose(fp);
             if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
@@ -838,7 +861,9 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
             free(preview_line);
          }
          if (!found) {
-            MsgBox("Invalid preview box in EPS file.", TOOL_NAME, INFO_MB);
+            sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BOX_IN_EPS),
+                  real_fname);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             if (need_to_free_line) free(line);
             if (bitmap != None) {
                XFreePixmap(mainDisplay, bitmap);
@@ -855,7 +880,9 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
          }
       } else {
          if (line[0] == '%' && line[1] == '!') epsf_level++;
-         if (!stripEPSComments || line[0] != '%') {
+         if (line[0] == '%' && line[1] == '%' && line[2] != '%') {
+            /* Actaully, need to check DCS conformance! */
+         } else if (!stripEPSComments || line[0] != '%') {
             if (cmdLineDosEpsFilter) {
                if (!cmdLinePreviewOnly) {
                   printf("%s\n", line);
@@ -872,8 +899,8 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
    if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
    if (!cmdLineDosEpsFilter) {
       if (!boundingbox_found) {
-         sprintf(gszMsgBox, "%s '%s'.",
-               "Can not find bounding box information in", file_name);
+         sprintf(gszMsgBox, TgLoadString(STID_CANNOT_BBOX_IN_GIVEN_FILE),
+               real_fname);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          if (bitmap != None) {
             XFreePixmap(mainDisplay, bitmap);
@@ -887,14 +914,15 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
          return BitmapFileInvalid;
       }
       if (*tiff_eps_fname != '\0' && !tiff_preview_ok) {
-         sprintf(gszMsgBox, "%s '%s'.",
-               "Unable to convert Windows EPS preview bitmap in", file_name);
+         sprintf(gszMsgBox, TgLoadString(STID_CANNOT_CONVERT_WINEPS_PREVIEW),
+               real_fname);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       }
-      if (stat(file_name, &stat_buf) == 0) {
+      if (stat(real_fname, &stat_buf) == 0) {
          strcpy(loc_time, ctime (&(stat_buf.st_mtime)));
          loc_time[24] = '\0';
       } else {
+         /* do not translate -- program constants */
          strcpy(loc_time, "(unknown)");
       }
       *pn_image_w = image_w;
@@ -1006,28 +1034,37 @@ FILE *JustGetTiffEPSIInfo(fp, pszEPS)
 }
 
 static
-int JustReadEPSLines (xbm_ptr)
-   struct XBmRec	* xbm_ptr;
+int JustReadEPSLines(xbm_ptr)
+   struct XBmRec *xbm_ptr;
 {
-   register struct LineRec	* line_ptr, * next_line;
-   register int	i;
-   char		* c_ptr, * line, loc_time[MAXSTRING+1];
-   char		tiff_eps_fname[MAXPATHLENGTH+1];
-   int		first_line=TRUE;
-   int		boundingbox_found=FALSE, preview_found=FALSE, found;
-   int		boundingbox_atend=FALSE;
-   float	llx, lly, urx, ury;
-   char		* file_name=xbm_ptr->filename;
-   FILE		* fp;
-   struct stat	stat_buf;
+   struct EPSLineRec *line_ptr, *next_line;
+   int i=0;
+   char *c_ptr, *line, loc_time[MAXSTRING+1];
+   char tiff_eps_fname[MAXPATHLENGTH+1];
+   char real_fname[MAXPATHLENGTH+1];
+   int first_line=TRUE;
+   int boundingbox_found=FALSE, preview_found=FALSE, found;
+   int boundingbox_atend=FALSE;
+   float llx, lly, urx, ury;
+   char *file_name=xbm_ptr->filename;
+   FILE *fp=NULL;
+   struct stat stat_buf;
 
-   if ((fp=fopen(file_name, "r")) == NULL) {
-      sprintf(gszMsgBox, "Can not open EPS file '%s'.", file_name);
+   *real_fname = '\0';
+   GetEPSFullPath(file_name, real_fname, sizeof(real_fname));
+
+   if ((fp=fopen(real_fname, "r")) == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_EPS_FILE_FOR_READ),
+            real_fname);
       if (PRTGIF) {
          fprintf(stderr, "%s\n", gszMsgBox);
-         fprintf(stderr, "   EPS object skipped for printing.\n");
+         fprintf(stderr, "   %s\n",
+               TgLoadString(STID_EPS_OBJ_SKIPPED_FOR_PRINT));
       } else {
-         TwoLineMsg(gszMsgBox, "   EPS object skipped for printing.");
+         char msg1[MAXSTRING];
+
+         sprintf(msg1, "   %s", TgLoadString(STID_EPS_OBJ_SKIPPED_FOR_PRINT));
+         TwoLineMsg(gszMsgBox, msg1);
       }
       return FALSE;
    }
@@ -1044,12 +1081,10 @@ int JustReadEPSLines (xbm_ptr)
    CleanUpLines();
 
    *tiff_eps_fname = '\0';
-   while ((line=UtilGetALine(fp)) != NULL)
-   {
+   while ((line=UtilGetALine(fp)) != NULL) {
       int need_to_free_line=TRUE;
 
-      if (first_line)
-      {
+      if (first_line) {
          if (line[0] == ((char)0xc5) && line[1] == ((char)0xd0) &&
                line[2] == ((char)0xd3) && line[3] == ((char)0xc6) &&
                line[4] == ((char)0x1e)) {
@@ -1063,90 +1098,86 @@ int JustReadEPSLines (xbm_ptr)
             }
          }
          first_line = FALSE;
-         if (line[0] != '%' || line[1] != '!')
-         {
+         if (line[0] != '%' || line[1] != '!') {
             if (need_to_free_line) free(line);
             fclose (fp);
-            CleanUpLines ();
-            return (FALSE);
+            CleanUpLines();
+            return FALSE;
          }
-         AddLine (line);
+         AddLine(line);
          need_to_free_line = FALSE;
-      }
-      else if ((!boundingbox_found || boundingbox_atend) &&
-            strncmp (line, "%%BoundingBox:", 14) == 0)
-      {
-         if (sscanf (&(line[14]), "%f %f %f %f", &llx, &lly, &urx, &ury) == 4)
+      } else if ((!boundingbox_found || boundingbox_atend) &&
+            strncmp(line, "%%BoundingBox:", 14) == 0) {
+         if (sscanf(&(line[14]), "%f %f %f %f", &llx, &lly, &urx, &ury) == 4) {
             boundingbox_found = TRUE;
-         else if (!boundingbox_found)
-         {
-            c_ptr = FindChar ((int)'(', &(line[14]));
-            if (strncmp (c_ptr, "atend)", 6) == 0)
+         } else if (!boundingbox_found) {
+            c_ptr = FindChar((int)'(', &(line[14]));
+            if (strncmp(c_ptr, "atend)", 6) == 0) {
                boundingbox_atend = TRUE;
+            }
          }
-      }
-      else if (!preview_found && strncmp (line, "%%BeginPreview:", 15) == 0)
-      {
+      } else if (!preview_found && strncmp(line, "%%BeginPreview:", 15) == 0) {
          char *preview_line;
 
          preview_found = TRUE;
          found = FALSE;
-         while ((preview_line=UtilGetALine(fp)) != NULL)
-         {
-            if (strncmp (preview_line, "%%EndPreview", 12) == 0)
-            {
+         while ((preview_line=UtilGetALine(fp)) != NULL) {
+            if (strncmp(preview_line, "%%EndPreview", 12) == 0) {
                found = TRUE;
                free(preview_line);
                break;
             }
             free(preview_line);
          }
-         if (!found)
-         {
+         if (!found) {
             if (PRTGIF) {
-               fprintf(stderr, "Invalid preview box in EPS file.\n");
+               fprintf(stderr, TgLoadString(STID_INVALID_PREVIEW_BMP_IN_EPS),
+                     real_fname);
             } else {
-               MsgBox("Invalid preview box in EPS file.", TOOL_NAME, INFO_MB);
+               sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BMP_IN_EPS),
+                     real_fname);
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             }
             free(line);
             if (need_to_free_line) free(line);
-            fclose (fp);
-            CleanUpLines ();
-            return (FALSE);
+            fclose(fp);
+            CleanUpLines();
+            return FALSE;
          }
-      }
-      else
-      {
+      } else {
          if (line[0] == '%' && line[1] == '!') (xbm_ptr->epsf_level)++;
-         if (!stripEPSComments || line[0] != '%') {
-            AddLine (line);
+         if (line[0] == '%' && line[1] == '%' && line[2] != '%') {
+            /* Actaully, need to check DCS conformance! */
+         } else if (!stripEPSComments || line[0] != '%') {
+            AddLine(line);
             need_to_free_line = FALSE;
          }
       }
       if (need_to_free_line) free(line);
    }
-   fclose (fp);
+   fclose(fp);
    if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
-   if (!boundingbox_found)
-   {
+   if (!boundingbox_found) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_BBOX_IN_GIVEN_FILE),
+            real_fname);
       if (PRTGIF) {
-         fprintf(stderr, "Can not find bounding box in EPS file.\n");
+         fprintf(stderr, "%s\n", gszMsgBox);
       } else {
-         MsgBox("Can not find bounding box in EPS file.", TOOL_NAME, INFO_MB);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       }
       CleanUpLines();
-      return (FALSE);
+      return FALSE;
    }
-   if (stat(file_name, &stat_buf) == 0) {
-      strcpy(loc_time, ctime (&(stat_buf.st_mtime)));
+   if (stat(real_fname, &stat_buf) == 0) {
+      strcpy(loc_time, ctime(&(stat_buf.st_mtime)));
       loc_time[24] = '\0';
    } else {
+      /* do not translate -- program constants */
       strcpy(loc_time, "(unknown)");
    }
-   if (strcmp (xbm_ptr->write_date, loc_time) != 0)
-   {
-      sprintf(gszMsgBox,
-            "Warning:  EPS file '%s' is newer than the EPS object.", file_name);
+   if (strcmp(xbm_ptr->write_date, loc_time) != 0) {
+      sprintf(gszMsgBox, TgLoadString(STID_EPS_FILE_NEWER_THAN_EPS_OBJ),
+            real_fname);
       if (PRTGIF) {
          fprintf(stderr, "%s\n", gszMsgBox);
       } else {
@@ -1159,14 +1190,14 @@ int JustReadEPSLines (xbm_ptr)
 
    for (i=0, line_ptr=topLine; line_ptr != NULL; line_ptr = next_line, i++) {
       next_line = line_ptr->next;
-      (xbm_ptr->epsflines)[i] = UtilStrDup (line_ptr->s);
+      (xbm_ptr->epsflines)[i] = UtilStrDup(line_ptr->s);
       free(line_ptr->s);
       free(line_ptr);
    }
    topLine = botLine = NULL;
    numLines = 0;
 
-   return (TRUE);
+   return TRUE;
 }
 
 void DumpEPSObj(FP, ObjPtr)
@@ -1187,18 +1218,21 @@ void DumpEPSObj(FP, ObjPtr)
             return;
          }
       } else {
-         char loc_time[MAXSTRING+1];
+         char loc_time[MAXSTRING+1], real_fname[MAXPATHLENGTH+1];
          struct stat stat_buf;
 
-         if (stat(xbm_ptr->filename, &stat_buf) == 0) {
+         *real_fname = '\0';
+         GetEPSFullPath(xbm_ptr->filename, real_fname, sizeof(real_fname));
+
+         if (stat(real_fname, &stat_buf) == 0) {
             strcpy(loc_time, ctime(&(stat_buf.st_mtime)));
             loc_time[24] = '\0';
          } else {
+            /* do not translate -- program constants */
             strcpy(loc_time, "(unknown)");
          }
          if (strcmp(xbm_ptr->write_date, loc_time) != 0) {
-            sprintf(gszMsgBox,
-                  "Warning:  EPS file '%s' is newer than the EPS object.",
+            sprintf(gszMsgBox, TgLoadString(STID_EPS_FILE_NEWER_THAN_EPS_OBJ),
                   xbm_ptr->filename);
             if (PRTGIF) {
                fprintf(stderr, "%s\n", gszMsgBox);
@@ -1234,7 +1268,7 @@ void DumpEPSObj(FP, ObjPtr)
    memset(&mtrx, 0, sizeof(struct MtrxRec));
    mtrx.image_w = (float)bbox_w_psu; mtrx.image_h = (float)bbox_h_psu;
    mtrx.w = w_psu; mtrx.h = h_psu;
-   mtrx.rotate = xbm_ptr->rotate; mtrx.flip = xbm_ptr->flip;
+   mtrx.rotate = ROTATE0; mtrx.flip = xbm_ptr->flip;
 
    CalcTransform(&mtrx);
 
@@ -1245,32 +1279,10 @@ void DumpEPSObj(FP, ObjPtr)
       ltx_psu = (float)((float)(ObjPtr->obbox.ltx) * scale);
       rby_psu = (float)(((float)psDotsPerInch)*psYOff[pageStyle] -
             ((float)(ObjPtr->obbox.rby) * scale));
-      switch (xbm_ptr->rotate) {
-      case ROTATE0:
-         x_psu = (mtrx.transformed_w >= 0.0) ?
-               (ltx_psu)-dx_psu : (ltx_psu+w_psu)-dx_psu;
-         y_psu = (mtrx.transformed_h >= 0.0) ?
-               (rby_psu)-dy_psu : (rby_psu+h_psu)-dy_psu;
-         break;
-      case ROTATE90:
-         x_psu = (mtrx.transformed_w >= 0.0) ?
-               (ltx_psu+w_psu)+dx_psu : (ltx_psu)+dx_psu;
-         y_psu = (mtrx.transformed_h >= 0.0) ?
-               (rby_psu+h_psu)+dy_psu : (rby_psu)+dy_psu;
-         break;
-      case ROTATE180:
-         x_psu = (mtrx.transformed_w >= 0.0) ?
-               (ltx_psu)-dx_psu : (ltx_psu+w_psu)-dx_psu;
-         y_psu = (mtrx.transformed_h >= 0.0) ?
-               (rby_psu)-dy_psu : (rby_psu+h_psu)-dy_psu;
-         break;
-      case ROTATE270:
-         x_psu = (mtrx.transformed_w >= 0.0) ?
-               (ltx_psu+w_psu)+dx_psu : (ltx_psu)+dx_psu;
-         y_psu = (mtrx.transformed_h >= 0.0) ?
-               (rby_psu+h_psu)+dy_psu : (rby_psu)+dy_psu;
-         break;
-      }
+      x_psu = (mtrx.transformed_w >= 0.0) ?
+            (ltx_psu)-dx_psu : (ltx_psu+w_psu)-dx_psu;
+      y_psu = (mtrx.transformed_h >= 0.0) ?
+            (rby_psu)-dy_psu : (rby_psu+h_psu)-dy_psu;
    } else {
       float tmp_dx, tmp_dy;
       int dx, dy, ltx, rby, x, y, tmp_x, tmp_y;
@@ -1302,7 +1314,7 @@ void DumpEPSObj(FP, ObjPtr)
                (((float)tmp_y) * mtrx.dump_v_scale));
       }
    }
-
+   /* do not translate -- program constants */
    fprintf(FP, "%% EPS\n");
    fprintf(FP, "end\n");
    fprintf(FP, "/tgiflevel%1d save def\n", xbm_ptr->epsf_level);
@@ -1312,8 +1324,9 @@ void DumpEPSObj(FP, ObjPtr)
    fprintf(FP, "/showpage {} def\n");
    fprintf(FP, "/letter {} def\n");
    fprintf(FP, "/legal {} def\n");
-   fprintf(FP, "/setpagedevice {} def\n");
    fprintf(FP, "/a4 {} def\n");
+   fprintf(FP, "/setpagedevice {} def\n");
+   fprintf(FP, "/erasepage {} def\n");
    fprintf(FP, "/bop-hook {} def\n");
    fprintf(FP, "/eop-hook {} def\n");
    fprintf(FP, "/start-hook {} def\n");
@@ -1373,6 +1386,43 @@ void DumpEPSObj(FP, ObjPtr)
    fprintf(FP, "\n");
 }
 
+void SetEPSObjCTM(obj_ptr, p_orig_obbox)
+   struct ObjRec *obj_ptr;
+   struct BBRec *p_orig_obbox;
+{
+   struct XBmRec *xbm_ptr=obj_ptr->detail.xbm;
+   int w=(p_orig_obbox->rbx-p_orig_obbox->ltx);
+   int h=(p_orig_obbox->rby-p_orig_obbox->lty), image_w=0, image_h=0;
+
+   if (obj_ptr->ctm != NULL) return;
+
+   if (xbm_ptr->real_type == XBM_XBM) {
+      image_w = xbm_ptr->image_w;
+      image_h = xbm_ptr->image_h;
+   } else {
+      image_w = xbm_ptr->eps_w;
+      image_h = xbm_ptr->eps_h;
+   }
+   if (image_w != w || image_h != h) {
+      float fval=(float)0;
+      struct XfrmMtrxRec ctm;
+
+      memset(&ctm, 0, sizeof(struct XfrmMtrxRec));
+      obj_ptr->orig_obbox.ltx = p_orig_obbox->ltx;
+      obj_ptr->orig_obbox.lty = p_orig_obbox->lty;
+      obj_ptr->orig_obbox.rbx = p_orig_obbox->rbx;
+      obj_ptr->orig_obbox.rby = p_orig_obbox->rby;
+      fval = ((float)w)/((float)image_w)*((float)1000.0);
+      ctm.m[CTM_SX] = (double)fval;
+      fval = ((float)h)/((float)image_h)*((float)1000.0);
+      ctm.m[CTM_SY] = (double)fval;
+      obj_ptr->obbox.rbx = obj_ptr->bbox.rbx = p_orig_obbox->ltx+image_w;
+      obj_ptr->obbox.rby = obj_ptr->bbox.rby = p_orig_obbox->lty+image_h;
+      SetCTM(obj_ptr, &ctm);
+      AdjObjBBox(obj_ptr);
+   }
+}
+
 struct ObjRec *CreateEPSObj(FileName, ImageW, ImageH, bitmap, image,
       num_lines, lines, epsf_level, llx, lly, urx, ury, write_date)
    char *FileName, **lines, *write_date;
@@ -1381,14 +1431,15 @@ struct ObjRec *CreateEPSObj(FileName, ImageW, ImageH, bitmap, image,
    XImage *image;
    float *llx, *lly, *urx, *ury;
 {
-   struct XBmRec *xbm_ptr;
-   struct ObjRec *obj_ptr;
-   int len=strlen(FileName), w, h;
-   char *name;
+   struct XBmRec *xbm_ptr=NULL;
+   struct ObjRec *obj_ptr=NULL;
+   struct BBRec orig_obbox;
+   int len=strlen(FileName), w=0, h=0;
+   char *name=NULL;
 
    name = (char*)malloc((len+1)*sizeof(char));
    if (name == NULL) FailAllocMessage();
-   strcpy (name, FileName);
+   strcpy(name, FileName);
 
    xbm_ptr = (struct XBmRec *)malloc(sizeof(struct XBmRec));
    if (xbm_ptr == NULL) FailAllocMessage();
@@ -1400,24 +1451,21 @@ struct ObjRec *CreateEPSObj(FileName, ImageW, ImageH, bitmap, image,
    xbm_ptr->bitmap = bitmap;
    xbm_ptr->data = NULL;
 
-   if (bitmap == None)
-   {
+   if (bitmap == None) {
       xbm_ptr->eps_w = w = ((*urx) >= (*llx)) ? (int)((*urx)-(*llx)) :
             (int)((*llx)-(*urx));
       xbm_ptr->eps_h = h = ((*ury) >= (*lly)) ? (int)((*ury)-(*lly)) :
             (int)((*lly)-(*ury));
-   }
-   else
-   {
+   } else {
       /* same as above! */
       xbm_ptr->eps_w = w = ((*urx) >= (*llx)) ? (int)((*urx)-(*llx)) :
             (int)((*llx)-(*urx));
       xbm_ptr->eps_h = h = ((*ury) >= (*lly)) ? (int)((*ury)-(*lly)) :
             (int)((*lly)-(*ury));
+      w = ImageW;
+      h = ImageH;
    }
-
    xbm_ptr->fill = objFill;
-   xbm_ptr->rotate = xbm_ptr->cached_rotate = ROTATE0;
    xbm_ptr->flip = xbm_ptr->cached_flip = 0;
    xbm_ptr->llx = (int)((*llx)*1000.0);
    xbm_ptr->lly = (int)((*lly)*1000.0);
@@ -1429,7 +1477,7 @@ struct ObjRec *CreateEPSObj(FileName, ImageW, ImageH, bitmap, image,
 
    xbm_ptr->real_type = XBM_EPS;
    xbm_ptr->filename = name;
-   strcpy (xbm_ptr->write_date, write_date);
+   strcpy(xbm_ptr->write_date, write_date);
    xbm_ptr->epsflines = lines;
    xbm_ptr->num_epsf_lines = num_lines;
    xbm_ptr->epsf_level = epsf_level;
@@ -1453,7 +1501,10 @@ struct ObjRec *CreateEPSObj(FileName, ImageW, ImageH, bitmap, image,
    obj_ptr->fattr = obj_ptr->lattr = NULL;
    obj_ptr->ctm = NULL;
 
-   return (obj_ptr);
+   memcpy(&orig_obbox, &obj_ptr->obbox, sizeof(struct BBRec));
+   SetEPSObjCTM(obj_ptr, &orig_obbox);
+
+   return obj_ptr;
 }
 
 void UpdateEPS()
@@ -1468,7 +1519,7 @@ void UpdateEPS()
       }
    }
    if (count == 0) {
-      MsgBox("No top-level EPS object selected.", TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_NO_TOP_LEVEL_EPS_SELECTED), TOOL_NAME, INFO_MB);
       return;
    }
    HighLightReverse();
@@ -1500,10 +1551,10 @@ void UpdateEPS()
                &image, &num_lines, &lines, &epsf_level, &llx, &lly, &urx, &ury,
                write_date);
          SetDefaultCursor(mainWindow);
-         SetDefaultCursor(drawWindow);
+         ShowCursor(); 
 
          if (rc != BitmapSuccess) {
-            sprintf(gszMsgBox, "Can not import EPS file '%s'.",
+            sprintf(gszMsgBox, TgLoadString(STID_CANNOT_IMPORT_GIVEN_EPS),
                   xbm_ptr->filename);
             MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             importingFile = FALSE;
@@ -1538,7 +1589,6 @@ void UpdateEPS()
             new_obj_ptr->obbox.rby = obj_ptr->obbox.rby;
          }
          new_xbm_ptr = new_obj_ptr->detail.xbm;
-         new_xbm_ptr->rotate = xbm_ptr->rotate;
          new_xbm_ptr->flip = xbm_ptr->flip;
 
          ReplaceObj(obj_ptr, new_obj_ptr);
@@ -1557,8 +1607,7 @@ void UpdateEPS()
             selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
       SetFileModified(TRUE);
       justDupped = FALSE;
-      sprintf(gszMsgBox, "EPS object%s updated.", (count>1 ? "s" : ""));
-      Msg(gszMsgBox);
+      Msg(TgLoadString(STID_EPS_OBJECT_UPDATED));
    }
    HighLightForward();
 }
