@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,26 +28,52 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/rect.c,v 3.24 1998/11/05 22:56:23 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/rect.c,v 4.13 2000/01/15 22:24:07 william Exp $
  */
 
-#include <stdio.h>
-#include <math.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_RECT_C_
+
+#include "tgifdefs.h"
 
 #include "arc.e"
 #include "color.e"
 #include "dialog.e"
 #include "drawing.e"
+#include "file.e"
 #include "msg.e"
 #include "poly.e"
-#ifndef _NO_EXTERN
+#include "ps.e"
 #include "rect.e"
-#endif
 #include "setup.e"
 #include "spline.e"
+#include "strtbl.e"
+
+void DumpRectPath(FP, LtX, LtY, RbX, RbY, Indent, LastLF)
+   FILE *FP;
+   int LtX, LtY, RbX, RbY, Indent;
+{
+   register int i;
+
+   if (psUseShortHand) {
+      for (i=0; i < Indent; i++) fprintf(FP, " ");
+      fprintf(FP, "%s %1d %1d %s %1d %1d %s %1d %1d %s %1d %1d %s %s",
+            gPsCmd[PS_NEWPATH],
+            LtX, LtY, gPsCmd[PS_MOVETO], RbX, LtY, gPsCmd[PS_LINETO],
+            RbX, RbY, gPsCmd[PS_LINETO], LtX, RbY, gPsCmd[PS_LINETO],
+            gPsCmd[PS_CLOSEPATH]);
+   } else {
+      for (i=0; i < Indent; i++) fprintf(FP, " ");
+      fprintf(FP, "%s\n", gPsCmd[PS_NEWPATH]);
+      for (i=0; i < Indent; i++) fprintf(FP, " ");
+      fprintf(FP, "   %1d %1d %s ", LtX, LtY, gPsCmd[PS_MOVETO]);
+      fprintf(FP, "%1d %1d %s ", RbX, LtY, gPsCmd[PS_LINETO]);
+      fprintf(FP, "%1d %1d %s ", RbX, RbY, gPsCmd[PS_LINETO]);
+      fprintf(FP, "%1d %1d %s\n", LtX, RbY, gPsCmd[PS_LINETO]);
+      for (i=0; i < Indent; i++) fprintf(FP, " ");
+      fprintf(FP, "%s", gPsCmd[PS_CLOSEPATH]);
+   }
+   fprintf(FP, "%s", (LastLF ? "\n" : " "));
+}
 
 void SetPoint(pt, x, y)
    IntPoint *pt;
@@ -77,14 +103,10 @@ void ClipRect(bbox, w, h)
    }
 }
 
-void SetRect(bbox, ltx, lty, rbx, rby)
-   struct BBRec *bbox;
-   int ltx, lty, rbx, rby;
+int CompareRect(pBBox1, pBBox2)
+   struct BBRec *pBBox1, *pBBox2;
 {
-   bbox->ltx = ltx;
-   bbox->lty = lty;
-   bbox->rbx = rbx;
-   bbox->rby = rby;
+   return memcmp(pBBox1, pBBox2, sizeof(struct BBRec));
 }
 
 void SetRotateVs(vs, ltx, lty, rbx, rby)
@@ -101,7 +123,20 @@ void SetBBRec(bbox, ltx, lty, rbx, rby)
    struct BBRec *bbox;
    int ltx, lty, rbx, rby;
 {
-   bbox->ltx = ltx; bbox->lty = lty; bbox->rbx = rbx; bbox->rby = rby;
+   if (ltx <= rbx) {
+      bbox->ltx = ltx;
+      bbox->rbx = rbx;
+   } else {
+      bbox->ltx = rbx;
+      bbox->rbx = ltx;
+   }
+   if (lty <= rby) {
+      bbox->lty = lty;
+      bbox->rby = rby;
+   } else {
+      bbox->lty = rby;
+      bbox->rby = lty;
+   }
 }
 
 void ConcatCTM(ctm, orig_ctm, new_ctm)
@@ -109,26 +144,18 @@ void ConcatCTM(ctm, orig_ctm, new_ctm)
    /* Note: CTM_SX, CTM_SIN, CTM_MSIN, and CTM_SY are scaled 1000 times */
    /*       while CTM_TX and CTM_TY are not scaled */
 {
-   new_ctm->m[0] = round(((double)(ctm->m[0]*orig_ctm->m[0] +
-         ctm->m[1]*orig_ctm->m[2]))/1000.0);
-   new_ctm->m[1] = round(((double)(ctm->m[0]*orig_ctm->m[1] +
-         ctm->m[1]*orig_ctm->m[3]))/1000.0);
-   new_ctm->m[2] = round(((double)(ctm->m[2]*orig_ctm->m[0] +
-         ctm->m[3]*orig_ctm->m[2]))/1000.0);
-   new_ctm->m[3] = round(((double)(ctm->m[2]*orig_ctm->m[1] +
-         ctm->m[3]*orig_ctm->m[3]))/1000.0);
-   new_ctm->m[4] = round(((double)(ctm->m[4]*orig_ctm->m[0] +
-         ctm->m[5]*orig_ctm->m[2]))/1000.0) + orig_ctm->m[4];
-   new_ctm->m[5] = round(((double)(ctm->m[4]*orig_ctm->m[1] +
-         ctm->m[5]*orig_ctm->m[3]))/1000.0) + orig_ctm->m[5];
-}
-
-void TransformPointThroughCTM(X, Y, ctm, NewX, NewY)
-   int X, Y, *NewX, *NewY;
-   struct XfrmMtrxRec *ctm;
-{
-   *NewX = round(((double)(X*ctm->m[0] + Y*ctm->m[2]))/1000.0) + ctm->m[CTM_TX];
-   *NewY = round(((double)(X*ctm->m[1] + Y*ctm->m[3]))/1000.0) + ctm->m[CTM_TY];
+   new_ctm->m[0] = ((double)(ctm->m[0]*orig_ctm->m[0] +
+         ctm->m[1]*orig_ctm->m[2]))/1000.0;
+   new_ctm->m[1] = ((double)(ctm->m[0]*orig_ctm->m[1] +
+         ctm->m[1]*orig_ctm->m[3]))/1000.0;
+   new_ctm->m[2] = ((double)(ctm->m[2]*orig_ctm->m[0] +
+         ctm->m[3]*orig_ctm->m[2]))/1000.0;
+   new_ctm->m[3] = ((double)(ctm->m[2]*orig_ctm->m[1] +
+         ctm->m[3]*orig_ctm->m[3]))/1000.0;
+   new_ctm->t[CTM_TX] = round(((double)(ctm->t[CTM_TX]*orig_ctm->m[0] +
+         ctm->t[CTM_TY]*orig_ctm->m[2]))/1000.0) + orig_ctm->t[CTM_TX];
+   new_ctm->t[CTM_TY] = round(((double)(ctm->t[CTM_TX]*orig_ctm->m[1] +
+         ctm->t[CTM_TY]*orig_ctm->m[3]))/1000.0) + orig_ctm->t[CTM_TY];
 }
 
 void TransformDoublePointThroughCTM(X, Y, ctm, NewX, NewY)
@@ -136,14 +163,28 @@ void TransformDoublePointThroughCTM(X, Y, ctm, NewX, NewY)
    struct XfrmMtrxRec *ctm;
 {
    *NewX = (X*((double)ctm->m[0]) + Y*((double)ctm->m[2]))/1000.0 +
-         (double)ctm->m[CTM_TX];
+         (double)ctm->t[CTM_TX];
    *NewY = (X*((double)ctm->m[1]) + Y*((double)ctm->m[3]))/1000.0 +
-         (double)ctm->m[CTM_TY];
+         (double)ctm->t[CTM_TY];
 }
 
-void ReverseTransformPointThroughCTM(X, Y, ctm, NewX, NewY)
+void TransformPointThroughCTM(X, Y, ctm, NewX, NewY)
    int X, Y, *NewX, *NewY;
    struct XfrmMtrxRec *ctm;
+{
+   double x=(double)0, y=(double)0;
+
+   x = ((double)(((double)X)*ctm->m[0]+((double)Y)*ctm->m[2])) / 1000.0 +
+         (double)(ctm->t[CTM_TX]);
+   y = ((double)(((double)X)*ctm->m[1]+((double)Y)*ctm->m[3])) / 1000.0 +
+         (double)(ctm->t[CTM_TY]);
+   *NewX = round(x);
+   *NewY = round(y);
+}
+
+void ReverseTransformDoublePointThroughCTM(X, Y, ctm, NewDx, NewDy)
+   double X, Y, *NewDx, *NewDy;
+   struct XfrmMtrxRec *ctm;
    /*
     *           [ a b 0 ]
     * Let ctm = [ c d 0 ]
@@ -158,14 +199,23 @@ void ReverseTransformPointThroughCTM(X, Y, ctm, NewX, NewY)
     *              bc - ad
     */
 {
-   int ad_bc=ctm->m[0]*ctm->m[3]-ctm->m[1]*ctm->m[2];
-   int term_1=X-ctm->m[CTM_TX], term_2=Y-ctm->m[CTM_TY];
-   double x, y;
+   double ad_bc=ctm->m[0]*ctm->m[3]-ctm->m[1]*ctm->m[2];
+   double term_1=X-((double)(ctm->t[CTM_TX]));
+   double term_2=Y-((double)(ctm->t[CTM_TY]));
 
-   x = ((double)1000.0) *
+   *NewDx = ((double)1000.0) *
          ((double)(ctm->m[3]*term_1 - ctm->m[2]*term_2)) / ((double)ad_bc);
-   y = ((double)1000.0) *
+   *NewDy = ((double)1000.0) *
          ((double)(ctm->m[0]*term_2 - ctm->m[1]*term_1)) / ((double)(ad_bc));
+}
+
+void ReverseTransformPointThroughCTM(X, Y, ctm, NewX, NewY)
+   int X, Y, *NewX, *NewY;
+   struct XfrmMtrxRec *ctm;
+{
+   double x=(double)0, y=(double)0;
+
+   ReverseTransformDoublePointThroughCTM(((double)X), ((double)Y), ctm, &x, &y);
    *NewX = round(x);
    *NewY = round(y);
 }
@@ -275,8 +325,8 @@ void SetCTM(ObjPtr, ctm)
    int ltx, lty, rbx, rby;
 
    if (ObjPtr->ctm != NULL) {
-      sprintf(gszMsgBox, "Error: SetCTM() is called with ObjPtr->ctm != NULL");
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_ERR_SETCTM_CALLED_CTM_NONNULL), TOOL_NAME,
+            INFO_MB);
       free(ObjPtr->ctm);
    }
    memcpy(&ObjPtr->orig_obbox, &ObjPtr->obbox, sizeof(struct BBRec));
@@ -286,9 +336,9 @@ void SetCTM(ObjPtr, ctm)
    }
    ObjPtr->ctm = (struct XfrmMtrxRec *)malloc(sizeof(struct XfrmMtrxRec));
    if (ObjPtr->ctm == NULL) FailAllocMessage();
-   ObjPtr->ctm->m[CTM_SX] = ObjPtr->ctm->m[CTM_SY] = 1000;
-   ObjPtr->ctm->m[CTM_SIN] = ObjPtr->ctm->m[CTM_MSIN] = 0;
-   ObjPtr->ctm->m[CTM_TX] = ObjPtr->ctm->m[CTM_TY] = 0;
+   ObjPtr->ctm->m[CTM_SX] = ObjPtr->ctm->m[CTM_SY] = (double)1000;
+   ObjPtr->ctm->m[CTM_SIN] = ObjPtr->ctm->m[CTM_MSIN] = (double)0;
+   ObjPtr->ctm->t[CTM_TX] = ObjPtr->ctm->t[CTM_TY] = 0;
    ConcatCTM(ObjPtr->ctm, ctm, &new_ctm);
    memcpy(ObjPtr->ctm, &new_ctm, sizeof(struct XfrmMtrxRec));
 
@@ -308,6 +358,45 @@ void SetCTM(ObjPtr, ctm)
    GetTransformedOBBoxOffsetVs(ObjPtr, ObjPtr->rotated_obbox);
 }
 
+void ExpandExtents(pBBox, pnLtX, pnLtY, pnRbX, pnRbY)
+   struct BBRec *pBBox;
+   int *pnLtX, *pnLtY, *pnRbX, *pnRbY;
+{
+   if (pBBox->ltx < (*pnLtX)) (*pnLtX) = pBBox->ltx;
+   if (pBBox->lty < (*pnLtY)) (*pnLtY) = pBBox->lty;
+   if (pBBox->rbx > (*pnRbX)) (*pnRbX) = pBBox->rbx;
+   if (pBBox->rby > (*pnRbY)) (*pnRbY) = pBBox->rby;
+}
+
+void ExpandBBox(pBBox, pBBoxToExpand)
+   struct BBRec *pBBox, *pBBoxToExpand;
+{
+   if (pBBox->ltx < pBBoxToExpand->ltx) pBBoxToExpand->ltx = pBBox->ltx;
+   if (pBBox->lty < pBBoxToExpand->lty) pBBoxToExpand->lty = pBBox->lty;
+   if (pBBox->rbx > pBBoxToExpand->rbx) pBBoxToExpand->rbx = pBBox->rbx;
+   if (pBBox->rby > pBBoxToExpand->rby) pBBoxToExpand->rby = pBBox->rby;
+}
+
+void InflateBBox(pBBoxIn, dx, dy, pBBoxOut)
+   struct BBRec *pBBoxIn, *pBBoxOut;
+   int dx, dy;
+{
+   int ltx=pBBoxIn->ltx+dx, lty=pBBoxIn->lty+dy;
+   int rbx=pBBoxIn->rbx-dx, rby=pBBoxIn->rby-dy;
+
+   SetBBRec(pBBoxOut, ltx, lty, rbx, rby);
+}
+
+void OffsetBBox(pBBoxIn, dx, dy, pBBoxOut)
+   struct BBRec *pBBoxIn, *pBBoxOut;
+   int dx, dy;
+{
+   int ltx=pBBoxIn->ltx+dx, lty=pBBoxIn->lty+dy;
+   int rbx=pBBoxIn->rbx+dx, rby=pBBoxIn->rby+dy;
+
+   SetBBRec(pBBoxOut, ltx, lty, rbx, rby);
+}
+
 int BBoxIntersect(Rect1, Rect2)
    struct BBRec Rect1, Rect2;
 {
@@ -340,6 +429,15 @@ int IntersectRect(BBox1, BBox2, BBox3)
    return FALSE;
 }
 
+void UnionRect(p_bbox1, p_bbox2, p_union_bbox)
+   struct BBRec *p_bbox1, *p_bbox2, *p_union_bbox;
+{
+   p_union_bbox->ltx = min(p_bbox1->ltx, p_bbox2->ltx);
+   p_union_bbox->lty = min(p_bbox1->lty, p_bbox2->lty);
+   p_union_bbox->rbx = max(p_bbox1->rbx, p_bbox2->rbx);
+   p_union_bbox->rby = max(p_bbox1->rby, p_bbox2->rby);
+}
+
 int Inside(BBox1, BBox2)
    struct BBRec BBox1, BBox2;
    /* returns TRUE if BBox1 is inside BBox2 */
@@ -355,7 +453,44 @@ int PointInBBox(X, Y, Rect)
    return (X >= Rect.ltx && Y >= Rect.lty && X <= Rect.rbx && Y <= Rect.rby);
 }
 
-#define POLYGON_TOL (1.0e-5)
+int PointInIntPolygon(X, Y, NumPts, IV)
+   int X, Y, NumPts;
+   IntPoint *IV;
+{
+   register double x1, x2, y1, y2;
+   double m, y_int;
+   int n, count=0;
+
+   x2 = (double)IV[0].x;
+   y2 = (double)IV[0].y;
+   for (n=0; n < NumPts-1; n++) {
+      x1 = x2;
+      y1 = y2;
+      x2 = (double)(IV[n+1].x);
+      y2 = (double)(IV[n+1].y);
+      if (x2 == x1) {
+         if (X == x1 && Y >= min(y1,y2) && Y <= max(y1,y2)) count++;
+         continue;
+      }
+      if (x2 > x1) {
+         if (X >= x2 || X < x1) continue;
+      } else {
+         if (X > x1 || X <= x2) continue;
+      }
+      m = (y1 - y2) / (x1 - x2);
+      y_int = m * X + (y1 - m * x1);
+      if (Y <= y_int) {
+         if (fabs(X-x1) < INT_TOL) {
+            double x3=(double)((n == 0) ? IV[NumPts-2].x : IV[n-1].x);
+
+            if ((x2 > X && X > x3) || (x3 > X && X > x2)) count++;
+         } else {
+            count++;
+         }
+      }
+   }
+   return (count & 0x1);
+}
 
 int PointInPolygon(X, Y, NumPts, V)
    int X, Y, NumPts;
@@ -381,11 +516,10 @@ int PointInPolygon(X, Y, NumPts, V)
       } else {
          if (X > x1 || X <= x2) continue;
       }
-
       m = (y1 - y2) / (x1 - x2);
       y_int = m * X + (y1 - m * x1);
       if (Y <= y_int) {
-         if (fabs(X-x1) < POLYGON_TOL) {
+         if (fabs(X-x1) < INT_TOL) {
             double x3=(double)((n == 0) ? V[NumPts-2].x : V[n-1].x);
 
             if ((x2 > X && X > x3) || (x3 > X && X > x2)) count++;
@@ -527,8 +661,10 @@ int FindGoodBox(XOff, YOff, BoxObj)
    if (!PointInBBox(XOff, YOff, bbox)) return FALSE;
 
    if (BoxObj->ctm == NULL) {
-      if (BoxObj->detail.b->fill != NONEPAT) return TRUE;
-
+      if (!(BoxObj->detail.b->fill == NONEPAT ||
+            (BoxObj->detail.b->fill == BACKPAT && BoxObj->trans_pat))) {
+         return TRUE;
+      }
       w = HALF_W(BoxObj->detail.b->width);
       bbox.ltx = OFFSET_X(BoxObj->obbox.ltx+w)+3;
       bbox.lty = OFFSET_Y(BoxObj->obbox.lty+w)+3;
@@ -536,7 +672,8 @@ int FindGoodBox(XOff, YOff, BoxObj)
       bbox.rby = OFFSET_Y(BoxObj->obbox.rby-w)-3;
       return (!PointInBBox(XOff, YOff, bbox));
    } else {
-      if (BoxObj->detail.b->fill != NONEPAT) {
+      if (!(BoxObj->detail.b->fill == NONEPAT ||
+            (BoxObj->detail.b->fill == BACKPAT && BoxObj->trans_pat))) {
          if (PointInPolygon(XOff, YOff, 5, BoxObj->rotated_obbox)) {
             return TRUE;
          }
@@ -552,7 +689,7 @@ int FindGoodRCBox(XOff, YOff, RCBoxObj)
    /* XOff and YOff are screen offsets */
 {
    struct RCBoxRec *rcbox_ptr=RCBoxObj->detail.rcb;
-   int fill=rcbox_ptr->fill;
+   int fill=rcbox_ptr->fill, trans_pat=RCBoxObj->trans_pat;
 
    if (colorLayers && RCBoxObj->tmp_parent == NULL &&
          !ObjInVisibleLayer(RCBoxObj)) {
@@ -571,8 +708,9 @@ int FindGoodRCBox(XOff, YOff, RCBoxObj)
       bbox.rby = OFFSET_Y(obbox->rby)+3;
       if (!PointInBBox(XOff, YOff, bbox)) return FALSE;
 
-      if (fill != NONEPAT) return TRUE;
-
+      if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
+         return TRUE;
+      }
       r = rcbox_ptr->radius;
       w = HALF_W(rcbox_ptr->width);
 
@@ -588,8 +726,8 @@ int FindGoodRCBox(XOff, YOff, RCBoxObj)
          bbox.rby = OFFSET_Y(obbox->rby-w)-3;
          return (!PointInBBox(XOff, YOff, bbox));
       }
-   } else {
-      if (fill != NONEPAT) {
+   } else if (rcbox_ptr->rotated_vlist != NULL) {
+      if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
          if (PointInPolygon(XOff, YOff, rcbox_ptr->rotated_n,
                rcbox_ptr->rotated_vlist)) {
             return TRUE;
@@ -598,6 +736,7 @@ int FindGoodRCBox(XOff, YOff, RCBoxObj)
       return PointOnPoly(XOff, YOff, rcbox_ptr->rotated_n,
             rcbox_ptr->rotated_vlist, ZOOMED_HALF_W(rcbox_ptr->width));
    }
+   return FALSE;
 }
 
 int FindGoodOval(XOff, YOff, OvalObj)
@@ -606,7 +745,7 @@ int FindGoodOval(XOff, YOff, OvalObj)
    /* XOff and YOff are screen offsets */
 {
    struct OvalRec *oval_ptr=OvalObj->detail.o;
-   int w, ltx, lty, rbx, rby, fill=oval_ptr->fill;
+   int w, ltx, lty, rbx, rby, fill=oval_ptr->fill, trans_pat=OvalObj->trans_pat;
 
    if (colorLayers && OvalObj->tmp_parent == NULL &&
          !ObjInVisibleLayer(OvalObj)) {
@@ -637,7 +776,7 @@ int FindGoodOval(XOff, YOff, OvalObj)
          x2 = cx + tmp_x;
       }
 
-      if (fill != NONEPAT) {
+      if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
          if (rx >= ry) {
             /* flat oval */
             if (YOff >= y1-4 && y2+4 >= YOff) return TRUE;
@@ -652,8 +791,8 @@ int FindGoodOval(XOff, YOff, OvalObj)
       } else {
          return ((fabs((double)(XOff-x1))<=w) || (fabs((double)(XOff-x2))<=w));
       }
-   } else {
-      if (fill != NONEPAT) {
+   } else if (oval_ptr->rotated_vlist != NULL) {
+      if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
          if (PointInPolygon(XOff, YOff, oval_ptr->rotated_n,
                oval_ptr->rotated_vlist)) {
             return TRUE;
@@ -662,6 +801,7 @@ int FindGoodOval(XOff, YOff, OvalObj)
       return PointOnPoly(XOff, YOff, oval_ptr->rotated_n,
             oval_ptr->rotated_vlist, ZOOMED_HALF_W(oval_ptr->width));
    }
+   return FALSE;
 }
 
 int FindGoodPoly(XOff, YOff, PolyObj)
@@ -669,22 +809,22 @@ int FindGoodPoly(XOff, YOff, PolyObj)
    struct ObjRec *PolyObj;
    /* XOff and YOff are screen offsets */
 {
-   int sn=0, dx, dy;
    double len, sin, cos, aw, ah;
    XPoint tmp_v[4], *sv=NULL;
    struct PolyRec *poly_ptr=PolyObj->detail.p;
+   int sn=0, dx, dy, fill=poly_ptr->fill, trans_pat=PolyObj->trans_pat;
 
    if (colorLayers && PolyObj->tmp_parent == NULL &&
          !ObjInVisibleLayer(PolyObj)) {
       return FALSE;
    }
-   if (poly_ptr->fill != NONEPAT) {
+   if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
       if (PolyObj->ctm == NULL) {
          sn = poly_ptr->sn;
          sv = poly_ptr->svlist;
          sv[sn].x = sv[0].x; sv[sn].y = sv[0].y;
          if (PointInPolygon(XOff, YOff, sn+1, sv)) return TRUE;
-      } else {
+      } else if (poly_ptr->rotated_vlist != NULL) {
          sn = poly_ptr->rotated_n;
          sv = poly_ptr->rotated_vlist;
          sv[sn].x = sv[0].x; sv[sn].y = sv[0].y;
@@ -696,7 +836,7 @@ int FindGoodPoly(XOff, YOff, PolyObj)
          if (PointOnPoly(XOff, YOff, poly_ptr->sn, poly_ptr->svlist,
                ZOOMED_HALF_W(poly_ptr->width)))
             return TRUE;
-      } else {
+      } else if (poly_ptr->rotated_vlist != NULL) {
          if (PointOnPoly(XOff, YOff, poly_ptr->rotated_n,
                poly_ptr->rotated_vlist, ZOOMED_HALF_W(poly_ptr->width)))
             return TRUE;
@@ -706,7 +846,7 @@ int FindGoodPoly(XOff, YOff, PolyObj)
          if (PointOnPoly(XOff, YOff, poly_ptr->asn, poly_ptr->asvlist,
                ZOOMED_HALF_W(poly_ptr->width)))
             return TRUE;
-      } else {
+      } else if (poly_ptr->rotated_vlist != NULL) {
          if (PointOnPoly(XOff, YOff, poly_ptr->rotated_n,
                poly_ptr->rotated_vlist, ZOOMED_HALF_W(poly_ptr->width)))
             return TRUE;
@@ -715,6 +855,8 @@ int FindGoodPoly(XOff, YOff, PolyObj)
    if (PolyObj->ctm == NULL) {
       sv = poly_ptr->svlist;
       sn = poly_ptr->sn;
+   } else if (poly_ptr->rotated_vlist == NULL) {
+      return FALSE;
    } else {
       sv = poly_ptr->rotated_vlist;
       sn = poly_ptr->rotated_n;
@@ -766,21 +908,22 @@ int FindGoodPolygon(XOff, YOff, PolygonObj)
    /* XOff and YOff are screen offsets */
 {
    struct PolygonRec *polygon_ptr=PolygonObj->detail.g;
+   int fill=polygon_ptr->fill, trans_pat=PolygonObj->trans_pat;
 
    if (colorLayers && PolygonObj->tmp_parent == NULL &&
          !ObjInVisibleLayer(PolygonObj)) {
       return FALSE;
    }
    if (PolygonObj->ctm == NULL) {
-      if (polygon_ptr->fill != NONEPAT) {
+      if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
          if (PointInPolygon(XOff, YOff, polygon_ptr->sn, polygon_ptr->svlist)) {
             return TRUE;
          }
       }
       return PointOnPoly(XOff, YOff, polygon_ptr->sn, polygon_ptr->svlist,
             ZOOMED_HALF_W(polygon_ptr->width));
-   } else {
-      if (polygon_ptr->fill != NONEPAT) {
+   } else if (polygon_ptr->rotated_vlist != NULL) {
+      if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
          if (PointInPolygon(XOff, YOff, polygon_ptr->rotated_n,
                polygon_ptr->rotated_vlist)) {
             return TRUE;
@@ -789,6 +932,7 @@ int FindGoodPolygon(XOff, YOff, PolygonObj)
       return PointOnPoly(XOff, YOff, polygon_ptr->rotated_n,
             polygon_ptr->rotated_vlist, ZOOMED_HALF_W(polygon_ptr->width));
    }
+   return FALSE;
 }
 
 static
@@ -995,8 +1139,6 @@ int YInPieRange(Y, dir, angle, cy, ry)
    return FALSE;
 }
 
-#define ARC_TOL (1.0e-5)
-
 int FindGoodArc(XOff, YOff, ArcObj)
    int XOff, YOff;
    struct ObjRec *ArcObj;
@@ -1010,7 +1152,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
    double rad_int_x1=0.0, rad_int_x2=0.0;
    double rad_int_y1=0.0, rad_int_y2=0.0;
    double len, sine, cosine;
-   int fill=arc_ptr->fill, angle1, angle2;
+   int fill=arc_ptr->fill, angle1, angle2, trans_pat=ArcObj->trans_pat;
    int arc_x1, arc_y1, arc_x2, arc_y2, theta1, theta2;
    int pass_theta1, just_pass_theta1, angle, dir, full_circle;
    XPoint tmp_v[4];
@@ -1020,7 +1162,10 @@ int FindGoodArc(XOff, YOff, ArcObj)
       return FALSE;
    }
    if (ArcObj->ctm != NULL) {
-      if (fill != NONEPAT) {
+      if (arc_ptr->rotated_vlist == NULL) {
+         return FALSE;
+      }
+      if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
          if (PointInPolygon(XOff, YOff, arc_ptr->rotated_n+2,
                arc_ptr->rotated_vlist)) {
             return TRUE;
@@ -1076,19 +1221,19 @@ int FindGoodArc(XOff, YOff, ArcObj)
    w = ZOOMED_HALF_W(arc_ptr->width)+4;
    if (rx >= ry) {
       /* flat oval */
-      if (fabs(arc_x1-cx) < ARC_TOL) {
+      if (fabs(arc_x1-cx) < INT_TOL) {
          switch (theta1) {
-         case 90: rad_int_y1 = cy - 1/ARC_TOL; break;
-         case 270: rad_int_y1 = cy + 1/ARC_TOL; break;
+         case 90: rad_int_y1 = cy - 1/INT_TOL; break;
+         case 270: rad_int_y1 = cy + 1/INT_TOL; break;
          default: fprintf(stderr, "theta1 = %1d (flat)\n", theta1); break;
          }
       } else {
          rad_int_y1 = cy + (XOff-cx)*(arc_y1-cy)/(arc_x1-cx);
       }
-      if (fabs(arc_x2-cx) < ARC_TOL) {
+      if (fabs(arc_x2-cx) < INT_TOL) {
          switch (theta2) {
-         case 90: rad_int_y2 = cy - 1/ARC_TOL; break;
-         case 270: rad_int_y2 = cy + 1/ARC_TOL; break;
+         case 90: rad_int_y2 = cy - 1/INT_TOL; break;
+         case 270: rad_int_y2 = cy + 1/INT_TOL; break;
          default: fprintf(stderr, "theta2 = %1d (flat)\n", theta2); break;
          }
       } else {
@@ -1096,21 +1241,21 @@ int FindGoodArc(XOff, YOff, ArcObj)
       }
    } else {
       /* tall oval */
-      if (fabs(arc_y1-cy) < ARC_TOL) {
+      if (fabs(arc_y1-cy) < INT_TOL) {
          switch (theta1) {
          case 0:
-         case 360: rad_int_x1 = cx + 1/ARC_TOL; break;
-         case 180: rad_int_x1 = cx - 1/ARC_TOL; break;
+         case 360: rad_int_x1 = cx + 1/INT_TOL; break;
+         case 180: rad_int_x1 = cx - 1/INT_TOL; break;
          default: fprintf(stderr, "theta1 = %1d (tall)\n", theta1); break;
          }
       } else {
          rad_int_x1 = cx + (YOff-cy)*(arc_x1-cx)/(arc_y1-cy);
       }
-      if (fabs(arc_y2-cy) < ARC_TOL) {
+      if (fabs(arc_y2-cy) < INT_TOL) {
          switch (theta2) {
          case 0:
-         case 360: rad_int_x2 = cx + 1/ARC_TOL; break;
-         case 180: rad_int_x2 = cx - 1/ARC_TOL; break;
+         case 360: rad_int_x2 = cx + 1/INT_TOL; break;
+         case 180: rad_int_x2 = cx - 1/INT_TOL; break;
          default: fprintf(stderr, "theta2 = %1d (tall)\n", theta2); break;
          }
       } else {
@@ -1127,7 +1272,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
             just_pass_theta1 = TRUE;
             if (theta2 > theta1 && angle >= theta2 && !full_circle) {
                /* theta1 and theta2 are in the same quadrant */
-               if (fill != NONEPAT) {
+               if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                   if (rx >= ry) {
                      /* flat oval */
                      if (PointInFlatPie(YOff, dir, angle, ov_int_y1,
@@ -1181,7 +1326,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
                case 180: y = cy; break;
                case 270: y = cy+ry; break;
                }
-               if (fill != NONEPAT) {
+               if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                   if (XInPieRange(XOff, dir, angle, cx, rx) &&
                         PointInFlatPie (YOff, dir, angle, ov_int_y1,
                         ov_int_y2, rad_int_y1, y)) {
@@ -1222,7 +1367,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
                case 180: x = cx-rx; break;
                case 270: x = cx; break;
                }
-               if (fill != NONEPAT) {
+               if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                   if (YInPieRange(YOff, dir, angle, cy, ry) &&
                         PointInTallPie(XOff, dir, angle, ov_int_x1,
                         ov_int_x2, rad_int_x1, x)) {
@@ -1261,7 +1406,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
             if (rx >= ry) {
                /* flat oval */
                if (XInPieRange(XOff, dir, angle, cx, rx)) {
-                  if (fill != NONEPAT) {
+                  if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                      switch ((angle+360)%360) {
                      case 90:
                      case 180:
@@ -1288,7 +1433,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
             } else {
                /* tall oval */
                if (YInPieRange(YOff, dir, angle, cy, ry)) {
-                  if (fill != NONEPAT) {
+                  if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                      switch ((angle+360)%360) {
                      case 0:
                      case 90:
@@ -1326,7 +1471,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
          case 90:
          case 270: y = cy; break;
          }
-         if (fill != NONEPAT) {
+         if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
             if (XInPieRange(XOff, dir, angle, cx, rx) &&
                   PointInFlatPie(YOff, dir, angle, ov_int_y1, ov_int_y2,
                   y, rad_int_y2)) {
@@ -1368,7 +1513,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
          case 90: x = cx+rx; break;
          case 270: x = cx-rx; break;
          }
-         if (fill != NONEPAT) {
+         if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
             if (YInPieRange(YOff, dir, angle, cy, ry) &&
                   PointInTallPie(XOff, dir, angle, ov_int_x1, ov_int_x2,
                   x, rad_int_x2)) {
@@ -1412,7 +1557,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
             just_pass_theta1 = TRUE;
             if (theta2 < theta1 && angle <= theta2 && !full_circle) {
                /* theta1 and theta2 are in the same quadrant */
-               if (fill != NONEPAT) {
+               if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                   if (rx >= ry) {
                      if (PointInFlatPie(YOff, dir, angle, ov_int_y1,
                            ov_int_y2, rad_int_y1, rad_int_y2)) {
@@ -1464,7 +1609,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
                case 180: y = cy; break;
                case 270: y = cy+ry; break;
                }
-               if (fill != NONEPAT) {
+               if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                   if (XInPieRange(XOff, dir, angle, cx, rx) &&
                         PointInFlatPie(YOff, dir, angle, ov_int_y1,
                         ov_int_y2, rad_int_y1, y)) {
@@ -1505,7 +1650,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
                case 180: x = cx-rx; break;
                case 270: x = cx; break;
                }
-               if (fill != NONEPAT) {
+               if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                   if (YInPieRange(YOff, dir, angle, cy, ry) &&
                         PointInTallPie(XOff, dir, angle, ov_int_x1,
                         ov_int_x2, rad_int_x1, x)) {
@@ -1544,7 +1689,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
             if (rx >= ry) {
                /* flat oval */
                if (XInPieRange(XOff, dir, angle, cx, rx)) {
-                  if (fill != NONEPAT) {
+                  if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                      switch ((angle+360)%360) {
                      case 0:
                      case 90:
@@ -1571,7 +1716,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
             } else {
                /* tall oval */
                if (YInPieRange(YOff, dir, angle, cy, ry)) {
-                  if (fill != NONEPAT) {
+                  if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
                      switch ((angle+360)%360) {
                      case 0:
                      case 270:
@@ -1607,7 +1752,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
          case 90:
          case 270: y = cy; break;
          }
-         if (fill != NONEPAT) {
+         if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
             if (XInPieRange(XOff, dir, angle, cx, rx) &&
                   PointInFlatPie(YOff, dir, angle, ov_int_y1, ov_int_y2,
                   y, rad_int_y2)) {
@@ -1649,7 +1794,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
          case 90: x = cx-rx; break;
          case 270: x = cx+rx; break;
          }
-         if (fill != NONEPAT) {
+         if (!(fill == NONEPAT || (fill == BACKPAT && trans_pat))) {
             if (YInPieRange(YOff, dir, angle, cy, ry) &&
                   PointInTallPie(XOff, dir, angle, ov_int_x1, ov_int_x2,
                   x, rad_int_x2)) {
@@ -1690,7 +1835,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
    aw = ZOOMED_SIZE(arc_ptr->aw); if (aw == 0) aw = 1;
    ah = ZOOMED_SIZE(arc_ptr->ah); if (ah == 0) ah = 1;
 
-   if ((arc_ptr->style & LS_LEFT) && (angle2%(360*64) != 0)) {
+   if ((arc_ptr->style & LS_LEFT) && (angle2%(360<<6) != 0)) {
       /* the arrow should appear at angle1 */
       switch (dir) {
       case ARC_CCW: theta = (int)(angle1/64)-90; break;
@@ -1719,7 +1864,7 @@ int FindGoodArc(XOff, YOff, ArcObj)
 
       if (PointInPolygon(XOff, YOff, 4, tmp_v)) return TRUE;
    }
-   if ((arc_ptr->style & LS_RIGHT) && (angle2%(360*64) != 0)) {
+   if ((arc_ptr->style & LS_RIGHT) && (angle2%(360<<6) != 0)) {
       switch (dir) {
       case ARC_CCW: theta = (int)((angle1+angle2)/64)-90; break;
       case ARC_CW: theta = (int)((angle1+angle2)/64)+90; break;
@@ -1762,7 +1907,6 @@ int FindGoodObj(XOff, YOff, FirstObj, SubObj, ImmediateChildObj)
 
    for (obj_ptr=FirstObj; obj_ptr != NULL; obj_ptr=obj_ptr->next) {
       obj_ptr->tmp_child = NULL;
-      obj_ptr->tmp_parent = NULL;
       if (ImmediateChildObj != NULL) *ImmediateChildObj = obj_ptr;
       if (colorLayers && !ObjInVisibleLayer(obj_ptr)) {
          continue;
@@ -1777,12 +1921,11 @@ int FindGoodObj(XOff, YOff, FirstObj, SubObj, ImmediateChildObj)
             return TRUE;
          }
       }
-
       if (XOff >= OFFSET_X(obj_ptr->bbox.ltx)-3 &&
             YOff >= OFFSET_Y(obj_ptr->bbox.lty)-3 &&
             XOff <= OFFSET_X(obj_ptr->bbox.rbx)+3 &&
             YOff <= OFFSET_Y(obj_ptr->bbox.rby)+3) {
-         struct ObjRec *next_level_child=NULL;
+         struct ObjRec *next_level_child=NULL, *visible_obj=NULL;
 
          switch (obj_ptr->type) {
          case OBJ_TEXT:
@@ -1830,6 +1973,24 @@ int FindGoodObj(XOff, YOff, FirstObj, SubObj, ImmediateChildObj)
                return TRUE;
             }
             break;
+         case OBJ_PIN:
+            visible_obj = GetPinObj(obj_ptr);
+            if (colorLayers) {
+               struct ObjRec *tmp_obj;
+
+               visible_obj->tmp_parent = obj_ptr;
+               for (tmp_obj=visible_obj->detail.r->first; tmp_obj != NULL;
+                     tmp_obj=tmp_obj->next) {
+                  tmp_obj->tmp_parent = visible_obj;
+               }
+            }
+            obj_ptr->tmp_child = visible_obj;
+            if (FindGoodObj(XOff, YOff, visible_obj->detail.r->first,
+                  SubObj, &next_level_child)) {
+               visible_obj->tmp_child = next_level_child;
+               return TRUE;
+            }
+            break;
          }
       }
    }
