@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,14 +28,12 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/ruler.c,v 3.21 1998/11/05 22:56:24 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/ruler.c,v 4.10 2000/01/15 22:24:09 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_RULER_C_
+
+#include "tgifdefs.h"
 
 #include "choice.e"
 #include "cursor.e"
@@ -45,31 +43,35 @@
 #include "mainmenu.e"
 #include "msg.e"
 #include "raster.e"
-#ifndef _NO_EXTERN
 #include "ruler.e"
-#endif
 #include "setup.e"
+#include "strtbl.e"
 #include "util.e"
 
-#define INCH_H 8
-#define HALF_INCH_H 6
-#define QUARTER_INCH_H 4
-#define MIN_INCH_H 2
+#define INCH_H (RULER_W-2)
 
-int	showMeasurement=FALSE;
-int	showCrossHair=FALSE;
+#define HALF_INCH_H 10
+#define QUARTER_INCH_H 6
+#define MIN_INCH_H 3
 
-static float	gfPixelsPerUnit=(float)1.0;
-static float	gfNumUnits=(float)1.0;
-static float	gfNumFracUnits=(float)1.0;
-static char	numUnitStr[80], baseUnitStr[80], unitStr[80], formatUnitStr[80];
+#define HALF_CM_H 8
+#define MIN_CM_H 4
 
-static GC	rulerGC;
+int showMeasurement=FALSE;
+int showCrossHair=FALSE;
 
-static int	oldXOff=(-1), oldYOff=(-1);
-static int	rulerLen=0;
-static int	hRulerJustRedrawn=TRUE, justUnFrozen=FALSE;
-static int	freezeMarkRulerText=FALSE, frozenXOff=0, frozenYOff=0;
+static float gfPixelsPerUnit=(float)1.0;
+static float gfNumUnits=(float)1.0;
+static float gfNumFracUnits=(float)1.0;
+
+static char numUnitStr[80], baseUnitStr[80], unitStr[80], formatUnitStr[80];
+
+static GC rulerGC;
+
+static int oldXOff=(-1), oldYOff=(-1);
+static int rulerLen=0;
+static int hRulerJustRedrawn=TRUE, justUnFrozen=FALSE;
+static int freezeMarkRulerText=FALSE, frozenXOff=0, frozenYOff=0;
 
 void GetUnitSpec(buf)
    char *buf;
@@ -92,21 +94,19 @@ void GetUnitSpec(buf)
 
 void ShowUnitMsg()
 {
-   sprintf(gszMsgBox, "Measurement will be shown in '%s' (%s %s).",
+   sprintf(gszMsgBox, TgLoadString(STID_MEASUREMENT_SHOWN_IN_UNIT),
          (*unitStr)=='\0' ? "pixel" : unitStr,
          (*numUnitStr)=='\0' ? "1" : numUnitStr,
          (*baseUnitStr)=='\0' ? "pixel" : baseUnitStr);
    Msg(gszMsgBox);
 }
 
-#define INT_TOL (1.0e-5)
-
 static
 int BadUnit(spec)
    char *spec;
 {
    if (msgWindow != None) {
-      sprintf(gszMsgBox, "Bad measurement unit specification '%s'.", spec);
+      sprintf(gszMsgBox, TgLoadString(STID_BAD_MEASUREMENT_UNIT_SPEC), spec);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
    }
    return FALSE;
@@ -214,17 +214,17 @@ int SetUnit(spec)
    return ok;
 }
 
-void InitRuler ()
+void InitRuler()
 {
-   XGCValues	values;
-   char		* c_ptr;
+   XGCValues values;
+   char *c_ptr;
 
    values.foreground = myFgPixel;
-   values.background = myBgPixel;
+   values.background = (threeDLook ? myLtGryPixel : myBgPixel);
    values.fill_style = FillSolid;
    values.font = rulerFontPtr->fid;
 
-   rulerGC = XCreateGC (mainDisplay, mainWindow,
+   rulerGC = XCreateGC(mainDisplay, mainWindow,
          GCForeground | GCBackground | GCFillStyle | GCFont, &values);
 
    showMeasurement = FALSE;
@@ -254,16 +254,17 @@ void InitRuler ()
          strcpy(spec, c_ptr);
       }
       if (!SetUnit(spec)) {
-         fprintf (stderr, "Invalid %s*%s: '%s', pixel is used.\n",
-               TOOL_NAME, "ShowMeasurementUnit", spec);
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "ShowMeasurementUnit", spec, "pixel");
+         fprintf(stderr, "\n");
       }
    }
    rulerLen = rulerW-1;
 }
 
-void CleanUpRuler ()
+void CleanUpRuler()
 {
-   XFreeGC (mainDisplay, rulerGC);
+   XFreeGC(mainDisplay, rulerGC);
 }
 
 void PixelToMeasurementUnit(Buf, NumPixels)
@@ -292,204 +293,194 @@ void PixelToMeasurementUnit(Buf, NumPixels)
    }
 }
 
-void RedrawHRuler ()
+void RedrawHRuler(dpy, win)
+   Display *dpy;
+   Window win;
 {
-   register int	i, pos, len, index;
-   int		inc, abs_inc;
-   char		s[5];
-   XEvent	ev;
+   register int i, pos, len, index;
+   int inc, abs_inc, y=rulerW-(threeDLook ? (windowPadding>>1) : 0);
+   char s[5];
 
-   XClearWindow (mainDisplay, hRuleWindow);
+   switch (gridSystem) {
+   case ENGLISH_GRID:
+      inc = (zoomedIn ? (xyEnglishGrid<<zoomScale) : xyEnglishGrid);
+      abs_inc = ABS_SIZE(inc);
 
-   XSync (mainDisplay, False);
-   while (XCheckWindowEvent (mainDisplay, hRuleWindow, ExposureMask, &ev)) ;
+      if (drawOrigX % abs_inc == 0) {
+         i = 0;
+         pos = 0;
+      } else {
+         i = ((int)(drawOrigX / abs_inc) + 1) * abs_inc - drawOrigX;
+         pos = ZOOMED_SIZE(i);
+      }
+      pos--; /* since drawWindow has no border */
+      for ( ; i<drawWinW; i+=abs_inc, pos+=inc) {
+         if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % ONE_INCH == 0) {
+            index = (int)((GRID_ZOOMED_SIZE(i+drawOrigX)) / ONE_INCH);
+            sprintf(s, "%1d", GRID_ABS_SIZE(index));
+            len = strlen(s);
+            XDrawString(dpy, win, rulerGC, pos+2, y-INCH_H+rulerFontAsc,
+                  s, len);
+            XDrawLine(dpy, win, defaultGC, pos, y, pos, y-INCH_H);
+         } else if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % HALF_INCH == 0) {
+            XDrawLine(dpy, win, defaultGC, pos, y, pos, y-HALF_INCH_H);
+         } else if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % QUARTER_INCH == 0) {
+            XDrawLine(dpy, win, defaultGC, pos, y, pos, y-QUARTER_INCH_H);
+         } else {
+            XDrawLine(dpy, win, defaultGC, pos, y, pos, y-MIN_INCH_H);
+         }
+      }
+      break;
+   case METRIC_GRID:
+      inc = (zoomedIn ? (xyMetricGrid<<zoomScale) : xyMetricGrid);
+      abs_inc = ABS_SIZE(inc);
 
-   switch (gridSystem)
-   {
-      case ENGLISH_GRID:
-         inc = (zoomedIn ? (xyEnglishGrid<<zoomScale) : xyEnglishGrid);
-         abs_inc = ABS_SIZE(inc);
-
-         if (drawOrigX % abs_inc == 0)
-         {
-            i = 0;
-            pos = 0;
+      if (drawOrigX % abs_inc == 0) {
+         i = 0;
+         pos = 0;
+      } else {
+         i = ((int)(drawOrigX / abs_inc) + 1) * abs_inc - drawOrigX;
+         pos = ZOOMED_SIZE(i);
+      }
+      pos--; /* since drawWindow has no border */
+      for ( ; i<drawWinW; i+=abs_inc, pos+=inc) {
+         if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % ONE_CM == 0) {
+            index = (int)((GRID_ZOOMED_SIZE(i+drawOrigX)) / ONE_CM);
+            sprintf(s, "%1d", GRID_ABS_SIZE(index));
+            len = strlen(s);
+            XDrawString(dpy, win, rulerGC, pos+2, y-INCH_H+rulerFontAsc,
+                  s, len);
+            XDrawLine(dpy, win, defaultGC, pos, y, pos, y-INCH_H);
+         } else if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % FIVE_MM == 0) {
+            XDrawLine(dpy, win, defaultGC, pos, y, pos, y-HALF_CM_H);
+         } else {
+            XDrawLine(dpy, win, defaultGC, pos, y, pos, y-MIN_CM_H);
          }
-         else
-         {
-            i = ((int)(drawOrigX / abs_inc) + 1) * abs_inc - drawOrigX;
-            pos = ZOOMED_SIZE(i);
-         }
-         pos--; /* since drawWindow has no border */
-         for ( ; i<drawWinW; i+=abs_inc, pos+=inc)
-         {
-            if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % ONE_INCH == 0)
-            {
-               index = (int)((GRID_ZOOMED_SIZE(i+drawOrigX)) / ONE_INCH);
-               sprintf (s, "%1d", GRID_ABS_SIZE(index));
-               len = strlen (s);
-               XDrawString (mainDisplay, hRuleWindow, rulerGC,
-                     pos-((len*rulerFontWidth)>>1),
-                     rulerW-INCH_H-rulerFontDes, s, len);
-               XDrawLine (mainDisplay, hRuleWindow, defaultGC, pos, rulerW,
-                     pos, rulerW-INCH_H);
-            }
-            else if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % HALF_INCH == 0)
-               XDrawLine (mainDisplay, hRuleWindow, defaultGC, pos, rulerW,
-                     pos, rulerW-HALF_INCH_H);
-            else if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % QUARTER_INCH == 0)
-               XDrawLine (mainDisplay, hRuleWindow, defaultGC, pos, rulerW,
-                     pos, rulerW-QUARTER_INCH_H);
-            else
-               XDrawLine (mainDisplay, hRuleWindow, defaultGC, pos, rulerW,
-                     pos, rulerW-MIN_INCH_H);
-         }
-         break;
-      case METRIC_GRID:
-         inc = (zoomedIn ? (xyMetricGrid<<zoomScale) : xyMetricGrid);
-         abs_inc = ABS_SIZE(inc);
-
-         if (drawOrigX % abs_inc == 0)
-         {
-            i = 0;
-            pos = 0;
-         }
-         else
-         {
-            i = ((int)(drawOrigX / abs_inc) + 1) * abs_inc - drawOrigX;
-            pos = ZOOMED_SIZE(i);
-         }
-         for ( ; i<drawWinW; i+=abs_inc, pos+=inc)
-         {
-            if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % ONE_CM == 0)
-            {
-               index = (int)((GRID_ZOOMED_SIZE(i+drawOrigX)) / ONE_CM);
-               sprintf (s, "%1d", GRID_ABS_SIZE(index));
-               len = strlen (s);
-               XDrawString (mainDisplay, hRuleWindow, rulerGC,
-                     pos-rulerFontWidth/2, rulerW-HALF_INCH_H-rulerFontDes,
-                     s, len);
-               XDrawLine (mainDisplay, hRuleWindow, defaultGC, pos, rulerW,
-                     pos, rulerW-HALF_INCH_H);
-            }
-            else if ((GRID_ZOOMED_SIZE(i+drawOrigX)) % FIVE_MM == 0)
-               XDrawLine (mainDisplay, hRuleWindow, defaultGC, pos, rulerW,
-                     pos, rulerW-QUARTER_INCH_H);
-            else
-               XDrawLine (mainDisplay, hRuleWindow, defaultGC, pos, rulerW,
-                     pos, rulerW-MIN_INCH_H);
-         }
-         break;
+      }
+      break;
    }
+}
+
+static
+void DrawHRuleTick(XOff)
+   int XOff;
+{
+   XDrawLine(mainDisplay, hRuleWindow, revDefaultGC, XOff, 0, XOff, rulerLen);
+}
+
+static
+void DrawVRuleTick(YOff)
+{
+   XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, YOff, rulerLen, YOff);
+}
+
+void RedrawHRulerWindow()
+{
+   XEvent ev;
+
+   XClearWindow(mainDisplay, hRuleWindow);
+
+   XSync(mainDisplay, False);
+   while (XCheckWindowEvent(mainDisplay, hRuleWindow, ExposureMask, &ev)) ;
+
+   RedrawHRuler(mainDisplay, hRuleWindow);
+
    oldXOff = (-1);
-   XDrawLine (mainDisplay, hRuleWindow, revDefaultGC, oldXOff, 0,
-         oldXOff, rulerLen);
+   DrawHRuleTick(oldXOff-1);
    hRulerJustRedrawn = TRUE;
    justUnFrozen = FALSE;
 }
 
-void RedrawVRuler ()
+void RedrawVRuler(dpy, win)
+   Display *dpy;
+   Window win;
 {
-   register int	i, pos, len, index;
-   int		inc, abs_inc;
-   char		s[5];
-   XEvent	ev;
+   register int i, pos, len, index;
+   int inc, abs_inc, x=rulerW-(threeDLook ? (windowPadding>>1) : 0);
+   char s[5];
 
-   XClearWindow (mainDisplay, vRuleWindow);
+   switch (gridSystem) {
+   case ENGLISH_GRID:
+      inc = (zoomedIn ? (xyEnglishGrid<<zoomScale) : xyEnglishGrid);
+      abs_inc = ABS_SIZE(inc);
 
-   XSync (mainDisplay, False);
-   while (XCheckWindowEvent (mainDisplay, vRuleWindow, ExposureMask, &ev)) ;
+      if (drawOrigY % abs_inc == 0) {
+         i = 0;
+         pos = 0;
+      } else {
+         i = ((int)(drawOrigY / abs_inc) + 1) * abs_inc - drawOrigY;
+         pos = ZOOMED_SIZE(i);
+      }
+      pos--; /* since drawWindow has no border */
+      for ( ; i<drawWinH; i+=abs_inc, pos+=inc) {
+         if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % ONE_INCH == 0) {
+            index = (int)((GRID_ZOOMED_SIZE(i+drawOrigY)) / ONE_INCH);
+            sprintf(s, "%1d", GRID_ABS_SIZE(index));
+            len = strlen(s);
+            XDrawString(dpy, win, rulerGC, 1, pos+rulerFontAsc+2, s, len);
+            XDrawLine(dpy, win, defaultGC, x, pos, x-INCH_H, pos);
+         } else if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % HALF_INCH == 0) {
+            XDrawLine(dpy, win, defaultGC, x, pos, x-HALF_INCH_H, pos);
+         } else if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % QUARTER_INCH == 0) {
+            XDrawLine(dpy, win, defaultGC, x, pos, x-QUARTER_INCH_H, pos);
+         } else {
+            XDrawLine(dpy, win, defaultGC, x, pos, x-MIN_INCH_H, pos);
+         }
+      }
+      break;
+   case METRIC_GRID:
+      inc = (zoomedIn ? (xyMetricGrid<<zoomScale) : xyMetricGrid);
+      abs_inc = ABS_SIZE(inc);
 
-   switch (gridSystem)
-   {
-      case ENGLISH_GRID:
-         inc = (zoomedIn ? (xyEnglishGrid<<zoomScale) : xyEnglishGrid);
-         abs_inc = ABS_SIZE(inc);
-
-         if (drawOrigY % abs_inc == 0)
-         {
-            i = 0;
-            pos = 0;
+      if (drawOrigY % abs_inc == 0) {
+         i = 0;
+         pos = 0;
+      } else {
+         i = ((int)(drawOrigY / abs_inc) + 1) * abs_inc - drawOrigY;
+         pos = ZOOMED_SIZE(i);
+      }
+      pos--; /* since drawWindow has no border */
+      for ( ; i<drawWinH; i+=abs_inc, pos+=inc) {
+         if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % ONE_CM == 0) {
+            index = (int)((GRID_ZOOMED_SIZE(i+drawOrigY)) / ONE_CM);
+            sprintf(s, "%1d", GRID_ABS_SIZE(index));
+            len = strlen(s);
+            XDrawString(dpy, win, rulerGC, 1, pos+rulerFontAsc+2, s, len);
+            XDrawLine(dpy, win, defaultGC, x, pos, x-INCH_H, pos);
+         } else if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % FIVE_MM == 0) {
+            XDrawLine(dpy, win, defaultGC, x, pos, x-HALF_CM_H, pos);
+         } else {
+            XDrawLine(dpy, win, defaultGC, x, pos, x-MIN_CM_H, pos);
          }
-         else
-         {
-            i = ((int)(drawOrigY / abs_inc) + 1) * abs_inc - drawOrigY;
-            pos = ZOOMED_SIZE(i);
-         }
-         pos--; /* since drawWindow has no border */
-         for ( ; i<drawWinH; i+=abs_inc, pos+=inc)
-         {
-            if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % ONE_INCH == 0)
-            {
-               index = (int)((GRID_ZOOMED_SIZE(i+drawOrigY)) / ONE_INCH);
-               sprintf (s, "%1d", GRID_ABS_SIZE(index));
-               len = strlen (s);
-               XDrawString (mainDisplay, vRuleWindow, rulerGC, 1,
-                     pos-rulerFontHeight/2+rulerFontAsc, s, len);
-               XDrawLine (mainDisplay, vRuleWindow, defaultGC, rulerW, pos,
-                     rulerW-INCH_H, pos);
-            }
-            else if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % HALF_INCH == 0)
-               XDrawLine (mainDisplay, vRuleWindow, defaultGC, rulerW, pos,
-                     rulerW-HALF_INCH_H, pos);
-            else if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % QUARTER_INCH == 0)
-               XDrawLine (mainDisplay, vRuleWindow, defaultGC, rulerW, pos,
-                     rulerW-QUARTER_INCH_H, pos);
-            else
-               XDrawLine (mainDisplay, vRuleWindow, defaultGC, rulerW, pos,
-                     rulerW-MIN_INCH_H, pos);
-         }
-         break;
-      case METRIC_GRID:
-         inc = (zoomedIn ? (xyMetricGrid<<zoomScale) : xyMetricGrid);
-         abs_inc = ABS_SIZE(inc);
-
-         if (drawOrigY % abs_inc == 0)
-         {
-            i = 0;
-            pos = 0;
-         }
-         else
-         {
-            i = ((int)(drawOrigY / abs_inc) + 1) * abs_inc - drawOrigY;
-            pos = ZOOMED_SIZE(i);
-         }
-         for ( ; i<drawWinH; i+=abs_inc, pos+=inc)
-         {
-            if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % ONE_CM == 0)
-            {
-               index = (int)((GRID_ZOOMED_SIZE(i+drawOrigY)) / ONE_CM);
-               sprintf (s, "%1d", GRID_ABS_SIZE(index));
-               len = strlen (s);
-               XDrawString (mainDisplay, vRuleWindow, rulerGC, 1,
-                     pos-rulerFontHeight/2+rulerFontAsc, s, len);
-               XDrawLine (mainDisplay, vRuleWindow, defaultGC, rulerW, pos,
-                     rulerW-HALF_INCH_H, pos);
-            }
-            else if ((GRID_ZOOMED_SIZE(i+drawOrigY)) % FIVE_MM == 0)
-               XDrawLine (mainDisplay, vRuleWindow, defaultGC, rulerW, pos,
-                     rulerW-QUARTER_INCH_H, pos);
-            else
-               XDrawLine (mainDisplay, vRuleWindow, defaultGC, rulerW, pos,
-                     rulerW-MIN_INCH_H, pos);
-         }
-         break;
+      }
+      break;
    }
+}
+
+void RedrawVRulerWindow()
+{
+   XEvent ev;
+
+   XClearWindow(mainDisplay, vRuleWindow);
+
+   XSync(mainDisplay, False);
+   while (XCheckWindowEvent(mainDisplay, vRuleWindow, ExposureMask, &ev)) ;
+
+   RedrawVRuler(mainDisplay, vRuleWindow);
+
    if (!freezeMarkRulerText && showMeasurement && !hRulerJustRedrawn &&
-         oldYOff != 0)
-   {
+         oldYOff != 0) {
       char x_buf[80], y_buf[80], buf[80];
 
       PixelToMeasurementUnit(x_buf, ABS_X(oldXOff));
       PixelToMeasurementUnit(y_buf, ABS_Y(oldYOff));
       sprintf(buf, "[%s,%s]", x_buf, y_buf);
-      XDrawString (mainDisplay, hRuleWindow, revDefaultGC, 8, 2+defaultFontAsc,
+      XDrawString(mainDisplay, hRuleWindow, revDefaultGC, 8, 2+defaultFontAsc,
             buf, strlen(buf));
       hRulerJustRedrawn = TRUE;
    }
    oldYOff = (-1);
-   XDrawLine (mainDisplay, vRuleWindow, revDefaultGC, 0, oldYOff,
-         rulerLen, oldYOff);
+   DrawVRuleTick(oldYOff-1);
 }
 
 void MarkRulers(XOff, YOff)
@@ -497,17 +488,14 @@ void MarkRulers(XOff, YOff)
 {
    char x_buf[80], y_buf[80], buf[80];
 
-   XDrawLine(mainDisplay, hRuleWindow, revDefaultGC, oldXOff-1, 0, oldXOff-1,
-         rulerLen);
-   XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldYOff-1, rulerLen,
-         oldYOff-1);
+   DrawHRuleTick(oldXOff-1);
+   DrawVRuleTick(oldYOff-1);
    if (showCrossHair) {
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, oldXOff, 0,
             oldXOff, ZOOMED_SIZE(drawWinH));
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, 0, oldYOff,
             ZOOMED_SIZE(drawWinW), oldYOff);
    }
-
    if (hRulerJustRedrawn) {
       hRulerJustRedrawn= FALSE;
    } else if (!freezeMarkRulerText && showMeasurement) {
@@ -524,15 +512,15 @@ void MarkRulers(XOff, YOff)
       XDrawString(mainDisplay, hRuleWindow, revDefaultGC, 8, 2+defaultFontAsc,
             buf, strlen(buf));
    }
-   XDrawLine(mainDisplay,hRuleWindow,revDefaultGC,XOff-1,0,XOff-1,rulerLen);
-   XDrawLine(mainDisplay,vRuleWindow,revDefaultGC,0,YOff-1,rulerLen,YOff-1);
+   DrawHRuleTick(XOff-1);
+   DrawVRuleTick(YOff-1);
+
    if (showCrossHair) {
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, XOff, 0,
             XOff, ZOOMED_SIZE(drawWinH));
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, 0, YOff,
             ZOOMED_SIZE(drawWinW), YOff);
    }
-
    if (!freezeMarkRulerText && showMeasurement) {
       PixelToMeasurementUnit(x_buf, ABS_X(XOff));
       PixelToMeasurementUnit(y_buf, ABS_Y(YOff));
@@ -546,8 +534,8 @@ void MarkRulers(XOff, YOff)
 
 void RedrawRulers()
 {
-   RedrawHRuler();
-   RedrawVRuler();
+   RedrawHRulerWindow();
+   RedrawVRulerWindow();
 }
 
 void GetCrossHairPosition(pnXOff, pnYOff, pnShown)
@@ -579,49 +567,55 @@ void ToggleShowCrossHair()
    showCrossHair = FALSE;
    RedrawRulers();
    if (showCrossHair) {
-      Msg("Showing cross-hair enabled.");
+      Msg(TgLoadString(STID_SHOW_CROSSHAIR_ENABLED));
       SetNullCursor(drawWindow);
    } else {
       SetDefaultCursor(drawWindow);
       ShowCursor();
-      Msg("Showing cross-hair disabled.");
+      Msg(TgLoadString(STID_SHOW_CROSSHAIR_DISABLED));
    }
 }
 
 void RulersEventHandler(input)
    XEvent *input;
 {
-   if (input->xany.window == vRuleWindow) {
+   if (input->type == ButtonPress) {
+      XButtonEvent *button_ev=(&input->xbutton);
+
+      if (button_ev->button == Button1) {
+         IncGrid();
+      } else if (button_ev->button == Button2) {
+         GridMenu(button_ev->x_root, button_ev->y_root, FALSE);
+      } else if (button_ev->button == Button3) {
+         DecGrid();
+      }
+   } else if (input->xany.window == vRuleWindow) {
       if (input->type == Expose) {
-         RedrawVRuler();
+         RedrawVRulerWindow();
       } else if (input->type == EnterNotify) {
-         SetMouseStatus("(none)", "(none)", "(none)");
-      } else if (input->type == ButtonPress) {
-         if (input->xbutton.button == Button3) {
-            ToggleShowCrossHair();
-         }
+         SetMouseStatus(TgLoadCachedString(CSTID_INC_GRID_SIZE),
+               TgLoadCachedString(CSTID_GRID_MENU),
+               TgLoadCachedString(CSTID_DEC_GRID_SIZE));
       }
    } else if (input->xany.window == hRuleWindow) {
       if (input->type == Expose) {
-         RedrawHRuler();
+         RedrawHRulerWindow();
       } else if (input->type == EnterNotify) {
-         SetMouseStatus("(none)", "(none)", "(none)");
-      } else if (input->type == ButtonPress) {
-         if (input->xbutton.button == Button3) {
-            ToggleShowCrossHair();
-         }
+         SetMouseStatus(TgLoadCachedString(CSTID_INC_GRID_SIZE),
+               TgLoadCachedString(CSTID_GRID_MENU),
+               TgLoadCachedString(CSTID_DEC_GRID_SIZE));
       }
    }
 }
 
-void FreezeMarkRulerText ()
+void FreezeMarkRulerText()
 {
    freezeMarkRulerText = TRUE;
    frozenXOff = oldXOff;
    frozenYOff = oldYOff;
 }
 
-void UnFreezeMarkRulerText ()
+void UnFreezeMarkRulerText()
 {
    freezeMarkRulerText = FALSE;
    justUnFrozen = TRUE;
@@ -630,7 +624,7 @@ void UnFreezeMarkRulerText ()
 static int oldLtX, oldLtY, oldRbX, oldRbY, oldMdX, oldMdY;
 
 static
-void DoIntervalRulers ()
+void DoIntervalRulers()
 {
    if (oldLtX == oldRbX) {
       XDrawLine(mainDisplay, hRuleWindow, revDefaultGC, oldLtX-1, 0, oldLtX-1,
@@ -657,12 +651,12 @@ void DoIntervalRulers ()
       XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldRbY-1, rulerLen-1,
             oldRbY);
    } else {
-      XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldLtY-1, rulerLen-1,
-            oldLtY);
-      XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldMdY-1, rulerLen-1,
-            oldMdY);
-      XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldRbY-1, rulerLen-1,
-            oldRbY);
+      XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldLtY-1, rulerLen,
+            oldLtY-1);
+      XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldMdY-1, rulerLen,
+            oldMdY-1);
+      XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldRbY-1, rulerLen,
+            oldRbY-1);
    }
    if (showCrossHair) {
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, oldRbX, 0,
@@ -672,13 +666,11 @@ void DoIntervalRulers ()
    }
 }
 
-void BeginIntervalRulers (ltx, lty, rbx, rby)
-   int	ltx, lty, rbx, rby;
+void BeginIntervalRulers(ltx, lty, rbx, rby)
+   int ltx, lty, rbx, rby;
 {
-   XDrawLine(mainDisplay, hRuleWindow, revDefaultGC, oldXOff-1, 0, oldXOff-1,
-         rulerLen);
-   XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldYOff-1, rulerLen,
-         oldYOff-1);
+   DrawHRuleTick(oldXOff-1);
+   DrawVRuleTick(oldYOff-1);
    if (showCrossHair) {
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, oldXOff, 0,
             oldXOff, ZOOMED_SIZE(drawWinH));
@@ -689,38 +681,36 @@ void BeginIntervalRulers (ltx, lty, rbx, rby)
    oldRbX = rbx; oldRbY = rby;
    oldMdX = (oldLtX+oldRbX)>>1;
    oldMdY = (oldLtY+oldRbY)>>1;
-   if (showMeasurement) FreezeMarkRulerText ();
-   DoIntervalRulers ();
+   if (showMeasurement) FreezeMarkRulerText();
+   DoIntervalRulers();
 }
 
-void DrawIntervalRulers (ltx, lty, rbx, rby)
-   int	ltx, lty, rbx, rby;
+void DrawIntervalRulers(ltx, lty, rbx, rby)
+   int ltx, lty, rbx, rby;
 {
-   DoIntervalRulers ();
+   DoIntervalRulers();
    oldLtX = ltx; oldLtY = lty;
    oldRbX = rbx; oldRbY = rby;
    oldMdX = (oldLtX+oldRbX)>>1; oldMdY = (oldLtY+oldRbY)>>1;
-   DoIntervalRulers ();
-   if (showMeasurement) UnFreezeMarkRulerText ();
+   DoIntervalRulers();
+   if (showMeasurement) UnFreezeMarkRulerText();
 }
 
-void EndIntervalRulers (x, y)
-   int	x, y;
+void EndIntervalRulers(x, y)
+   int x, y;
 {
-   DoIntervalRulers ();
+   DoIntervalRulers();
    oldXOff = x;
    oldYOff = y;
-   XDrawLine(mainDisplay, hRuleWindow, revDefaultGC, oldXOff-1, 0, oldXOff-1,
-         rulerLen);
-   XDrawLine(mainDisplay, vRuleWindow, revDefaultGC, 0, oldYOff-1, rulerLen,
-         oldYOff-1);
+   DrawHRuleTick(oldXOff-1);
+   DrawVRuleTick(oldYOff-1);
    if (showCrossHair) {
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, oldXOff, 0,
             oldXOff, ZOOMED_SIZE(drawWinH));
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, 0, oldYOff,
             ZOOMED_SIZE(drawWinW), oldYOff);
    }
-   MarkRulers (x, y);
+   MarkRulers(x, y);
    frozenXOff = oldXOff;
    frozenYOff = oldYOff;
 }
@@ -808,10 +798,10 @@ void ToggleShowMeasurement()
    showMeasurement = !showMeasurement;
    RedrawRulers();
    if (showMeasurement) {
-      Msg("Showing measurement enabled.");
+      Msg(TgLoadString(STID_SHOW_MEASUREMENT_ENABLED));
    } else {
-      Msg("Showing measurement disabled.");
+      Msg(TgLoadString(STID_SHOW_MEASUREMENT_DISABLED));
    }
-   UpdateSubMenu(MENU_LAYOUT);
+   UpdatePinnedMenu(MENU_LAYOUT);
 }
 
