@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,9 +28,9 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/tgif_dbg.c,v 3.10 1998/11/05 22:56:31 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.c,v 4.10 2000/01/15 22:24:21 william Exp $
  */
-#ifdef _TGIF_DBG
+#define _INCLUDE_FROM_TGIF_DBG_C_
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -43,6 +43,8 @@
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 
+#ifdef _TGIF_DBG /* debug, do not translate */
+
 #undef _Xconst
 #define _Xconst
 
@@ -65,6 +67,19 @@ void tgif_dbg_add_to_log(pszString)
    }
 }
 
+void tgif_dbg_enable(nEnable)
+   int nEnable;
+{
+   static FILE *st_fp=NULL;
+
+   if (nEnable) {
+      gpTraceFile = st_fp;
+   } else {
+      st_fp = gpTraceFile;
+      gpTraceFile = NULL;
+   }
+}
+
 /* --------------- utility functions --------------- */
 
 void got_trigger(nOne)
@@ -86,6 +101,9 @@ void tgif_track(nBegin)
       gpTraceFile = fopen(gszTraceFile, "w");
       if (gpTraceFile == NULL) {
          fprintf(stderr, "Cannot open '%s' for write.\n", gszTraceFile);
+      } else {
+         fprintf(stderr, "Set gnAllocTrigger for alloc().\n");
+         fprintf(stderr, "Set gnFreeTrigger for free().\n");
       }
       break;
    case 0:
@@ -159,6 +177,15 @@ void tgif_free(pVoid)
    free(pVoid);
 }
 
+size_t tgif_strlen(psz)
+   char *psz;
+{
+   if (psz == NULL) {
+      fprintf(stderr, "ERROR: strlen(NULL) called!\n");
+   }
+   return strlen(psz);
+}
+
 /* --------------- XCreateGC, XFreeGC --------------- */
 
 int gnCreateGCTrigger=0, gnFreeGCTrigger=0;
@@ -338,7 +365,7 @@ Pixmap Tgif_XCreatePixmap(display, d, width, height, depth)
          got_trigger(1);
       }
       fprintf(gpTraceFile, "p: %6d 0x%08x	%d\n",
-            gnCreatePixmapCount, (unsigned int)pixmap, width*height*depth);
+            gnCreatePixmapCount, (unsigned int)pixmap, width*height*(depth>>3));
    }
    return pixmap;
 }
@@ -437,6 +464,24 @@ char *Tgif_XGetAtomName(display, atom)
    return pszString;
 }
 
+Colormap *Tgif_XListInstalledColormaps(display, w, num_return)
+   Display *display;
+   Window w;
+   int *num_return;
+{
+   Colormap *pColormap=XListInstalledColormaps(display, w, num_return);
+
+   if (gpTraceFile != NULL) {
+      if (++gnCreateStringCount == gnCreateStringTrigger) {
+         got_trigger(1);
+      }
+      fprintf(gpTraceFile, "s: %6d 0x%08x	%d\n",
+            gnCreateStringCount, (unsigned int)pColormap,
+            (unsigned int)sizeof(Colormap*));
+   }
+   return pColormap;
+}
+
 void Tgif_XFree(pVoid)
    void *pVoid;
 {
