@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/tgif_dbg.h,v 3.8 1998/11/05 22:57:10 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.h,v 4.4 2000/01/15 22:25:20 william Exp $
  */
 
 #ifndef _TGIF_DBG_H_
@@ -41,11 +41,14 @@
 #include <X11/keysym.h>
 
 extern void tgif_dbg_add_to_log ARGS_DECL((char*));
+extern void tgif_dbg_enable ARGS_DECL((int));
 
 extern void *tgif_malloc ARGS_DECL((size_t));
 extern void *tgif_realloc ARGS_DECL((void*, size_t));
 extern void tgif_free ARGS_DECL((void*));
 
+extern size_t tgif_strlen ARGS_DECL((char*));
+
 extern GC Tgif_XCreateGC ARGS_DECL((Display*, Drawable, unsigned long,
        XGCValues*));
 extern void Tgif_XFreeGC ARGS_DECL((Display*, GC));
@@ -71,6 +74,8 @@ extern char *Tgif_XFetchBytes ARGS_DECL((Display*, int*));
 extern Status Tgif_XQueryTree ARGS_DECL((Display*, Window, Window*, Window*,
        Window**, unsigned int*));
 extern char *Tgif_XGetAtomName ARGS_DECL((Display*, Atom));
+extern Colormap *Tgif_XListInstalledColormaps ARGS_DECL((Display*, Window,
+       int*));
 extern void Tgif_XFree ARGS_DECL((void*));
 
 /* -------------------- defines -------------------- */
@@ -87,10 +92,16 @@ extern void Tgif_XFree ARGS_DECL((void*));
 #undef free
 #endif /* free */
 
+#ifdef strlen
+#undef strlen
+#endif /* strlen */
+
 #define malloc tgif_malloc
 #define realloc tgif_realloc
 #define free tgif_free
 
+#define strlen tgif_strlen
+
 #define XCreateGC Tgif_XCreateGC
 #define XFreeGC Tgif_XFreeGC
 
@@ -110,6 +121,7 @@ extern void Tgif_XFree ARGS_DECL((void*));
 #define XFetchBytes Tgif_XFetchBytes
 #define XQueryTree Tgif_XQueryTree
 #define XGetAtomName Tgif_XGetAtomName
+#define XListInstalledColormaps Tgif_XListInstalledColormaps
 #define XFree Tgif_XFree
 
 #endif /* ~PRTGIF_NO_TGIF_DBG */
